@@ -2,6 +2,7 @@ static char module_doc[] =
 "This module provides a BLAS optimized\nmatrix multiply, inner product and dot for numpy arrays";
 
 #include "Python.h"
+#include "npy_config.h"
 #include "numpy/ndarrayobject.h"
 #ifndef CBLAS_HEADER
 #define CBLAS_HEADER "cblas.h"
@@ -10,6 +11,12 @@ static char module_doc[] =
 
 #include <stdio.h>
 
+#if (PY_VERSION_HEX < 0x02060000)
+#define Py_TYPE(o)    (((PyObject*)(o))->ob_type)
+#define Py_REFCNT(o)  (((PyObject*)(o))->ob_refcnt)
+#define Py_SIZE(o)    (((PyVarObject*)(o))->ob_size)
+#endif
+
 static PyArray_DotFunc *oldFunctions[PyArray_NTYPES];
 
 static void
@@ -401,14 +408,14 @@ dotblas_matrixproduct(PyObject *NPY_UNUSED(dummy), PyObject *args)
     }
 
     /* Choose which subtype to return */
-    if (ap1->ob_type != ap2->ob_type) {
+    if (Py_TYPE(ap1) != Py_TYPE(ap2)) {
         prior2 = PyArray_GetPriority((PyObject *)ap2, 0.0);
         prior1 = PyArray_GetPriority((PyObject *)ap1, 0.0);
-        subtype = (prior2 > prior1 ? ap2->ob_type : ap1->ob_type);
+        subtype = (prior2 > prior1 ? Py_TYPE(ap2) : Py_TYPE(ap1));
     }
     else {
         prior1 = prior2 = 0.0;
-        subtype = ap1->ob_type;
+        subtype = Py_TYPE(ap1);
     }
 
     ret = (PyArrayObject *)PyArray_New(subtype, nd, dimensions,
@@ -900,7 +907,7 @@ dotblas_innerproduct(PyObject *NPY_UNUSED(dummy), PyObject *args)
     /* Choose which subtype to return */
     prior2 = PyArray_GetPriority((PyObject *)ap2, 0.0);
     prior1 = PyArray_GetPriority((PyObject *)ap1, 0.0);
-    subtype = (prior2 > prior1 ? ap2->ob_type : ap1->ob_type);
+    subtype = (prior2 > prior1 ? Py_TYPE(ap2) : Py_TYPE(ap1));
 
     ret = (PyArrayObject *)PyArray_New(subtype, nd, dimensions,
                                        typenum, NULL, NULL, 0, 0,
@@ -1168,13 +1175,38 @@ static struct PyMethodDef dotblas_module_methods[] = {
     {NULL, NULL, 0, NULL}               /* sentinel */
 };
 
+#if defined(NPY_PY3K)
+static struct PyModuleDef moduledef = {
+        PyModuleDef_HEAD_INIT,
+        "_dotblas",
+        NULL,
+        -1,
+        dotblas_module_methods,
+        NULL,
+        NULL,
+        NULL,
+        NULL
+};
+#endif
+
 /* Initialization function for the module */
-PyMODINIT_FUNC init_dotblas(void) {
+#if defined(NPY_PY3K)
+#define RETVAL m
+PyObject *PyInit__dotblas(void)
+#else
+#define RETVAL
+PyMODINIT_FUNC init_dotblas(void)
+#endif
+{
     int i;
-    PyObject *d, *s;
+    PyObject *d, *s, *m;
 
     /* Create the module and add the functions */
-    Py_InitModule3("_dotblas", dotblas_module_methods, module_doc);
+#if defined(NPY_PY3K)
+    m = PyModule_Create(&moduledef);
+#else
+    m = Py_InitModule3("_dotblas", dotblas_module_methods, module_doc);
+#endif
 
     /* Import the array object */
     import_array();
@@ -1189,4 +1221,5 @@ PyMODINIT_FUNC init_dotblas(void) {
     Py_DECREF(d);
     Py_DECREF(s);
 
+    return RETVAL;
 }
