@@ -110,7 +110,11 @@ class Ufunc(object):
 # String-handling utilities to avoid locale-dependence.
 
 import string
-UPPER_TABLE = string.maketrans(string.ascii_lowercase, string.ascii_uppercase)
+if sys.version_info[0] < 3:
+    UPPER_TABLE = string.maketrans(string.ascii_lowercase, string.ascii_uppercase)
+else:
+    UPPER_TABLE = bytes.maketrans(bytes(string.ascii_lowercase, "ascii"), 
+            bytes(string.ascii_uppercase, "ascii"))
 
 def english_upper(s):
     """ Apply English case rules to convert ASCII strings to all upper case.
@@ -192,7 +196,7 @@ noint = inexact+O
 nointP = inexact+P
 allP = bints+flts+cmplxP
 nobool = all[1:]
-noobj = all[:-2]
+noobj = all[:-2] 
 nobool_or_obj = all[1:-2]
 intflt = ints+flts
 intfltcmplx = ints+flts+cmplx
@@ -215,20 +219,12 @@ defdict = {
     Ufunc(2, 1, Zero,
           docstrings.get('numpy.core.umath.add'),
           TD(notimes_or_obj),
-          #[TypeDescription('M', UsesArraysAsData, 'Mm', 'M'),
-          # TypeDescription('m', UsesArraysAsData, 'mm', 'm'),
-          # TypeDescription('M', UsesArraysAsData, 'mM', 'M'),
-          #],
           TD(O, f='PyNumber_Add'),
           ),
 'subtract' :
     Ufunc(2, 1, Zero,
           docstrings.get('numpy.core.umath.subtract'),
           TD(notimes_or_obj),
-          #[TypeDescription('M', UsesArraysAsData, 'Mm', 'M'),
-          # TypeDescription('m', UsesArraysAsData, 'mm', 'm'),
-          # TypeDescription('M', UsesArraysAsData, 'MM', 'm'),
-          #],
           TD(O, f='PyNumber_Subtract'),
           ),
 'multiply' :
@@ -298,7 +294,7 @@ defdict = {
 'absolute' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.absolute'),
-          TD(nocmplx),
+          TD(bints+flts),
           TD(cmplx, out=('f', 'd', 'g')),
           TD(O, f='PyNumber_Absolute'),
           ),
@@ -310,7 +306,7 @@ defdict = {
 'negative' :
     Ufunc(1, 1, None,
           docstrings.get('numpy.core.umath.negative'),
-          TD(nocmplx),
+          TD(bints+flts),
           TD(cmplx, f='neg'),
           TD(O, f='PyNumber_Negative'),
           ),
@@ -674,6 +670,10 @@ defdict = {
           ),
 }
 
+if sys.version_info[0] >= 3:
+    # Will be aliased to true_divide in umathmodule.c.src:InitOtherOperators
+    del defdict['divide']
+
 def indent(st,spaces):
     indention = ' '*spaces
     indented = indention + st.replace('\n','\n'+indention)
@@ -709,7 +709,7 @@ def make_arrays(funcdict):
     #
     code1list = []
     code2list = []
-    names = funcdict.keys()
+    names = list(funcdict.keys())
     names.sort()
     for name in names:
         uf = funcdict[name]
@@ -773,13 +773,21 @@ def make_arrays(funcdict):
 
 def make_ufuncs(funcdict):
     code3list = []
-    names = funcdict.keys()
+    names = list(funcdict.keys())
     names.sort()
     for name in names:
         uf = funcdict[name]
         mlist = []
         docstring = textwrap.dedent(uf.docstring).strip()
-        docstring = docstring.encode('string-escape').replace(r'"', r'\"')
+        if sys.version_info[0] < 3:
+            docstring = docstring.encode('string-escape')
+            docstring = docstring.replace(r'"', r'\"')
+        else:
+            docstring = docstring.encode('unicode-escape').decode('ascii')
+            docstring = docstring.replace(r'"', r'\"')
+            # XXX: I don't understand why the following replace is not
+            # necessary in the python 2 case.
+            docstring = docstring.replace(r"'", r"\'")
         # Split the docstring because some compilers (like MS) do not like big
         # string literal in C code. We split at endlines because textwrap.wrap
         # do not play well with \n
