@@ -4,12 +4,12 @@ operations and methods.
 
 .. note::
    The `chararray` class exists for backwards compatibility with
-   Numarray, it is not recommended for new development. If one needs
-   arrays of strings, use arrays of `dtype` `object_`, `string_` or
-   `unicode_`, and use the free functions in the `numpy.char` module
-   for fast vectorized string operations.
+   Numarray, it is not recommended for new development. Starting from numpy
+   1.4, if one needs arrays of strings, it is recommended to use arrays of
+   `dtype` `object_`, `string_` or `unicode_`, and use the free functions
+   in the `numpy.char` module for fast vectorized string operations.
 
-Some methods will only be available if the corresponding str method is
+Some methods will only be available if the corresponding string method is
 available in your version of Python.
 
 The preferred alias for `defchararray` is `numpy.char`.
@@ -21,6 +21,7 @@ from numerictypes import string_, unicode_, integer, object_, bool_, character
 from numeric import ndarray, compare_chararrays
 from numeric import array as narray
 from numpy.core.multiarray import _vec_string
+from numpy.compat import asbytes
 import numpy
 
 __all__ = ['chararray',
@@ -36,7 +37,12 @@ __all__ = ['chararray',
            'array', 'asarray']
 
 _globalvar = 0
-_unicode = unicode
+if sys.version_info[0] >= 3:
+    _unicode = str
+    _bytes = bytes
+else:
+    _unicode = unicode
+    _bytes = str
 _len = len
 
 def _use_unicode(*args):
@@ -392,6 +398,8 @@ if sys.version_info >= (2, 4):
         a_arr = numpy.asarray(a)
         width_arr = numpy.asarray(width)
         size = long(numpy.max(width_arr.flat))
+        if numpy.issubdtype(a_arr.dtype, numpy.string_):
+            fillchar = asbytes(fillchar)
         return _vec_string(
             a_arr, (a_arr.dtype.type, size), 'center', (width_arr, fillchar))
 else:
@@ -454,7 +462,7 @@ def count(a, sub, start=0, end=None):
     --------
     >>> c = np.array(['aAaAaA', '  aA  ', 'abBABba'])
     >>> c
-    array(['aAaAaA', '  aA', 'abBABba'],
+    array(['aAaAaA', '  aA  ', 'abBABba'],
         dtype='|S7')
     >>> np.char.count(c, 'A')
     array([3, 1, 1])
@@ -502,7 +510,7 @@ def decode(a, encoding=None, errors=None):
     --------
     >>> c = np.array(['aAaAaA', '  aA  ', 'abBABba'])
     >>> c
-    array(['aAaAaA', '  aA', 'abBABba'],
+    array(['aAaAaA', '  aA  ', 'abBABba'],
         dtype='|S7')
     >>> np.char.encode(c, encoding='cp037')
     array(['\\x81\\xc1\\x81\\xc1\\x81\\xc1', '@@\\x81\\xc1@@',
@@ -581,9 +589,9 @@ def endswith(a, suffix, start=0, end=None):
     >>> s
     array(['foo', 'bar'],
         dtype='|S3')
-    >>> np.charendswith(s, 'ar')
+    >>> np.char.endswith(s, 'ar')
     array([False,  True], dtype=bool)
-    >>> s.endswith(s, 'a', start=1, end=2)
+    >>> np.char.endswith(s, 'a', start=1, end=2)
     array([False,  True], dtype=bool)
 
     """
@@ -912,6 +920,8 @@ if sys.version_info >= (2, 4):
         a_arr = numpy.asarray(a)
         width_arr = numpy.asarray(width)
         size = long(numpy.max(width_arr.flat))
+        if numpy.issubdtype(a_arr.dtype, numpy.string_):
+            fillchar = asbytes(fillchar)
         return _vec_string(
             a_arr, (a_arr.dtype.type, size), 'ljust', (width_arr, fillchar))
 else:
@@ -1007,15 +1017,17 @@ def lstrip(a, chars=None):
     --------
     >>> c = np.array(['aAaAaA', '  aA  ', 'abBABba'])
     >>> c
-    array(['aAaAaA', '  aA', 'abBABba'],
+    array(['aAaAaA', '  aA  ', 'abBABba'],
         dtype='|S7')
     >>> np.char.lstrip(c, 'a') # 'a' unstripped from c[1] because whitespace leading
-    array(['AaAaA', '  aA', 'bBABba'],
-        dtype='|S6')
+    array(['AaAaA', '  aA  ', 'bBABba'],
+        dtype='|S7')
     >>> np.char.lstrip(c, 'A') # leaves c unchanged
-    array(['aAaAaA', '  aA', 'abBABba'],
+    array(['aAaAaA', '  aA  ', 'abBABba'],
         dtype='|S7')
     >>> (np.char.lstrip(c, ' ') == np.char.lstrip(c, '')).all()
+    ... # XXX: is this a regression? this line now returns False
+    ... # np.char.lstrip(c,'') does not modify c at all.
     True
     >>> (np.char.lstrip(c, ' ') == np.char.lstrip(c, None)).all()
     True
@@ -1176,6 +1188,8 @@ if sys.version_info >= (2, 4):
         a_arr = numpy.asarray(a)
         width_arr = numpy.asarray(width)
         size = long(numpy.max(width_arr.flat))
+        if numpy.issubdtype(a_arr.dtype, numpy.string_):
+            fillchar = asbytes(fillchar)
         return _vec_string(
             a_arr, (a_arr.dtype.type, size), 'rjust', (width_arr, fillchar))
 else:
@@ -1310,7 +1324,7 @@ def rstrip(a, chars=None):
         dtype='|S7')
     >>> np.char.rstrip(c, 'a')
     array(['aAaAaA', 'abBABb'],
-        dtype='|S6')
+        dtype='|S7')
     >>> np.char.rstrip(c, 'A')
     array(['aAaAa', 'abBABba'],
         dtype='|S7')
@@ -1441,16 +1455,16 @@ def strip(a, chars=None):
     --------
     >>> c = np.array(['aAaAaA', '  aA  ', 'abBABba'])
     >>> c
-    array(['aAaAaA', '  aA', 'abBABba'],
+    array(['aAaAaA', '  aA  ', 'abBABba'],
         dtype='|S7')
     >>> np.char.strip(c)
     array(['aAaAaA', 'aA', 'abBABba'],
         dtype='|S7')
     >>> np.char.strip(c, 'a') # 'a' unstripped from c[1] because whitespace leads
-    array(['AaAaA', '  aA', 'bBABb'],
-        dtype='|S6')
+    array(['AaAaA', '  aA  ', 'bBABb'],
+        dtype='|S7')
     >>> np.char.strip(c, 'A') # 'A' unstripped from c[1] because (unprinted) ws trails
-    array(['aAaAa', '  aA', 'abBABba'],
+    array(['aAaAa', '  aA  ', 'abBABba'],
         dtype='|S7')
 
     """
@@ -1520,7 +1534,7 @@ def title(a):
     array(['a1b c', '1b ca', 'b ca1', 'ca1b'],
         dtype='|S5')
     >>> np.char.title(c)
-    chararray(['A1B C', '1B Ca', 'B Ca1', 'Ca1B'],
+    array(['A1B C', '1B Ca', 'B Ca1', 'Ca1B'],
         dtype='|S5')
     """
     a_arr = numpy.asarray(a)
@@ -1587,7 +1601,7 @@ def upper(a):
     >>> c = np.array(['a1b c', '1bca', 'bca1']); c
     array(['a1b c', '1bca', 'bca1'],
         dtype='|S5')
-    >>> numpy.char.upper(c)
+    >>> np.char.upper(c)
     array(['A1B C', '1BCA', 'BCA1'],
         dtype='|S5')
     """
@@ -1687,10 +1701,10 @@ class chararray(ndarray):
 
     .. note::
        The `chararray` class exists for backwards compatibility with
-       Numarray, it is not recommended for new development. If one needs
-       arrays of strings, use arrays of `dtype` `object_`, `string_` or
-       `unicode_`, and use the free functions in the `numpy.char` module
-       for fast vectorized string operations.
+       Numarray, it is not recommended for new development. Starting from numpy
+       1.4, if one needs arrays of strings, it is recommended to use arrays of
+       `dtype` `object_`, `string_` or `unicode_`, and use the free functions
+       in the `numpy.char` module for fast vectorized string operations.
 
     Versus a regular Numpy array of type `str` or `unicode`, this
     class adds the following functionality:
@@ -1713,6 +1727,71 @@ class chararray(ndarray):
     ``len(shape) >= 2`` and ``order='Fortran'``, in which case `strides`
     is in "Fortran order".
 
+    Methods
+    -------
+    astype
+    argsort
+    copy
+    count
+    decode
+    dump
+    dumps
+    encode
+    endswith
+    expandtabs
+    fill
+    find
+    flatten
+    getfield
+    index
+    isalnum
+    isalpha
+    isdecimal
+    isdigit
+    islower
+    isnumeric
+    isspace
+    istitle
+    isupper
+    item
+    join
+    ljust
+    lower
+    lstrip
+    nonzero
+    put
+    ravel
+    repeat
+    replace
+    reshape
+    resize
+    rfind
+    rindex
+    rjust
+    rsplit
+    rstrip
+    searchsorted
+    setfield
+    setflags
+    sort
+    split
+    splitlines
+    squeeze
+    startswith
+    strip
+    swapaxes
+    swapcase
+    take
+    title
+    tofile
+    tolist
+    tostring
+    translate
+    transpose
+    upper
+    view
+    zfill
+
     Parameters
     ----------
     shape : tuple
@@ -1739,7 +1818,7 @@ class chararray(ndarray):
     Examples
     --------
     >>> charar = np.chararray((3, 3))
-    >>> charar[:, :] = 'abc'
+    >>> charar[:] = 'a'
     >>> charar
     chararray([['a', 'a', 'a'],
            ['a', 'a', 'a'],
@@ -1747,7 +1826,7 @@ class chararray(ndarray):
           dtype='|S1')
 
     >>> charar = np.chararray(charar.shape, itemsize=5)
-    >>> charar[:, :] = 'abc'
+    >>> charar[:] = 'abc'
     >>> charar
     chararray([['abc', 'abc', 'abc'],
            ['abc', 'abc', 'abc'],
@@ -1769,6 +1848,13 @@ class chararray(ndarray):
         # strings in the new array.
         itemsize = long(itemsize)
 
+        if sys.version_info[0] >= 3 and isinstance(buffer, _unicode):
+            # On Py3, unicode objects do not have the buffer interface
+            filler = buffer
+            buffer = None
+        else:
+            filler = None
+
         _globalvar = 1
         if buffer is None:
             self = ndarray.__new__(subtype, shape, (dtype, itemsize),
@@ -1778,6 +1864,8 @@ class chararray(ndarray):
                                    buffer=buffer,
                                    offset=offset, strides=strides,
                                    order=order)
+        if filler is not None:
+            self[...] = filler
         _globalvar = 0
         return self
 
@@ -2513,7 +2601,7 @@ def array(obj, itemsize=None, copy=True, unicode=None, order=None):
         be in any order (either C-, Fortran-contiguous, or even
         discontiguous).
     """
-    if isinstance(obj, (str, _unicode)):
+    if isinstance(obj, (_bytes, _unicode)):
         if unicode is None:
             if isinstance(obj, _unicode):
                 unicode = True
@@ -2522,7 +2610,7 @@ def array(obj, itemsize=None, copy=True, unicode=None, order=None):
 
         if itemsize is None:
             itemsize = _len(obj)
-        shape = _len(obj) / itemsize
+        shape = _len(obj) // itemsize
 
         if unicode:
             if sys.maxunicode == 0xffff:
@@ -2551,7 +2639,7 @@ def array(obj, itemsize=None, copy=True, unicode=None, order=None):
         else:
             # Let the default Unicode -> string encoding (if any) take
             # precedence.
-            obj = str(obj)
+            obj = _bytes(obj)
 
         return chararray(shape, itemsize=itemsize, unicode=unicode,
                          buffer=obj, order=order)
