@@ -110,18 +110,6 @@ typedef struct {
         PyObject * obval;
 } PyObjectScalarObject;
 
-typedef struct {
-        PyObject_HEAD
-        npy_datetime obval;
-        PyArray_DatetimeMetaData obmeta;
-} PyDatetimeScalarObject;
-
-typedef struct {
-        PyObject_HEAD
-        npy_timedelta obval;
-        PyArray_DatetimeMetaData obmeta;
-} PyTimedeltaScalarObject;
-
 
 typedef struct {
         PyObject_HEAD
