@@ -1,7 +1,7 @@
 __all__ = ['newaxis', 'ndarray', 'flatiter', 'ufunc',
            'arange', 'array', 'zeros', 'empty', 'broadcast', 'dtype',
-           'fromstring', 'fromfile', 'frombuffer','newbuffer',
-           'getbuffer', 'int_asbuffer', 'where', 'argwhere',
+           'fromstring', 'fromfile', 'frombuffer',
+           'int_asbuffer', 'where', 'argwhere',
            'concatenate', 'fastCopyAndTranspose', 'lexsort',
            'set_numeric_ops', 'can_cast',
            'asarray', 'asanyarray', 'ascontiguousarray', 'asfortranarray',
@@ -19,7 +19,8 @@ __all__ = ['newaxis', 'ndarray', 'flatiter', 'ufunc',
            'seterrcall', 'geterrcall', 'errstate', 'flatnonzero',
            'Inf', 'inf', 'infty', 'Infinity',
            'nan', 'NaN', 'False_', 'True_', 'bitwise_not',
-           'CLIP', 'RAISE', 'WRAP', 'MAXDIMS', 'BUFSIZE', 'ALLOW_THREADS']
+           'CLIP', 'RAISE', 'WRAP', 'MAXDIMS', 'BUFSIZE', 'ALLOW_THREADS',
+           'ComplexWarning']
 
 import sys
 import warnings
@@ -29,6 +30,19 @@ from umath import *
 import numerictypes
 from numerictypes import *
 
+if sys.version_info[0] < 3:
+    __all__.extend(['getbuffer', 'newbuffer'])
+
+class ComplexWarning(RuntimeWarning):
+    """
+    The warning raised when casting a complex dtype to a real dtype.
+
+    As implemented, casting a complex number to a real discards its imaginary
+    part, but this behavior may not be what the user actually wants.
+
+    """
+    pass
+
 bitwise_not = invert
 
 CLIP = multiarray.CLIP
@@ -55,13 +69,13 @@ def zeros_like(a):
     Parameters
     ----------
     a : array_like
-        The shape and data-type of `a` define the parameters of
+        The shape and data-type of `a` define these same attributes of
         the returned array.
 
     Returns
     -------
     out : ndarray
-        Array of zeros with same shape and type as `a`.
+        Array of zeros with the same shape and type as `a`.
 
     See Also
     --------
@@ -110,7 +124,7 @@ def empty_like(a):
     Parameters
     ----------
     a : array_like
-        The shape and data-type of `a` define the parameters of the
+        The shape and data-type of `a` define these same attributes of the
         returned array.
 
     Returns
@@ -129,8 +143,8 @@ def empty_like(a):
     Notes
     -----
     This function does *not* initialize the returned array; to do that use
-    `zeros_like` or `ones_like` instead. It may be marginally faster than the
-    functions that do set the array values.
+    `zeros_like` or `ones_like` instead.  It may be marginally faster than
+    the functions that do set the array values.
 
     Examples
     --------
@@ -140,7 +154,7 @@ def empty_like(a):
            [          0,           0, -1073741821]])
     >>> a = np.array([[1., 2., 3.],[4.,5.,6.]])
     >>> np.empty_like(a)
-    array([[ -2.00000715e+000,   1.48219694e-323,  -2.00000572e+000], #random
+    array([[ -2.00000715e+000,   1.48219694e-323,  -2.00000572e+000],#random
            [  4.38791518e-305,  -2.00000715e+000,   4.17269252e-309]])
 
     """
@@ -186,8 +200,9 @@ fromstring = multiarray.fromstring
 fromiter = multiarray.fromiter
 fromfile = multiarray.fromfile
 frombuffer = multiarray.frombuffer
-newbuffer = multiarray.newbuffer
-getbuffer = multiarray.getbuffer
+if sys.version_info[0] < 3:
+    newbuffer = multiarray.newbuffer
+    getbuffer = multiarray.getbuffer
 int_asbuffer = multiarray.int_asbuffer
 where = multiarray.where
 concatenate = multiarray.concatenate
@@ -247,6 +262,14 @@ def asarray(a, dtype=None, order=None):
     >>> np.asarray(a) is a
     True
 
+    If `dtype` is set, array is copied only if dtype does not match:
+
+    >>> a = np.array([1, 2], dtype=np.float32)
+    >>> np.asarray(a, dtype=np.float32) is a
+    True
+    >>> np.asarray(a, dtype=np.float64) is a
+    False
+
     Contrary to `asanyarray`, ndarray subclasses are not passed through:
 
     >>> issubclass(np.matrix, np.ndarray)
@@ -262,14 +285,14 @@ def asarray(a, dtype=None, order=None):
 
 def asanyarray(a, dtype=None, order=None):
     """
-    Convert the input to a ndarray, but pass ndarray subclasses through.
+    Convert the input to an ndarray, but pass ndarray subclasses through.
 
     Parameters
     ----------
     a : array_like
         Input data, in any form that can be converted to an array.  This
         includes scalars, lists, lists of tuples, tuples, tuples of tuples,
-        tuples of lists and ndarrays.
+        tuples of lists, and ndarrays.
     dtype : data-type, optional
         By default, the data-type is inferred from the input data.
     order : {'C', 'F'}, optional
@@ -289,7 +312,8 @@ def asanyarray(a, dtype=None, order=None):
     asfarray : Convert input to a floating point ndarray.
     asfortranarray : Convert input to an ndarray with column-major
                      memory order.
-    asarray_chkfinite : Similar function which checks input for NaNs and Infs.
+    asarray_chkfinite : Similar function which checks input for NaNs and
+                        Infs.
     fromiter : Create an array from an iterator.
     fromfunction : Construct an array by executing a function on grid
                    positions.
@@ -616,15 +640,17 @@ def _mode_from_name(mode):
         return _mode_from_name_dict[mode.lower()[0]]
     return mode
 
-def correlate(a,v,mode='valid',old_behavior=True):
+def correlate(a, v, mode='valid', old_behavior=False):
     """
-    Discrete, linear correlation of two 1-dimensional sequences.
+    Cross-correlation of two 1-dimensional sequences.
 
-    This function is equivalent to
+    This function computes the correlation as generally defined in signal
+    processing texts::
 
-    >>> np.convolve(a, v[::-1], mode=mode)
+        z[k] = sum_n a[n] * conj(v[n+k])
 
-    where ``v[::-1]`` is the reverse of `v`.
+    with a and v sequences being zero-padded where necessary and conj being
+    the conjugate.
 
     Parameters
     ----------
@@ -634,27 +660,13 @@ def correlate(a,v,mode='valid',old_behavior=True):
         Refer to the `convolve` docstring.  Note that the default
         is `valid`, unlike `convolve`, which uses `full`.
     old_behavior : bool
-        If True, uses the old, numeric behavior (correlate(a,v) == correlate(v,
+        If True, uses the old behavior from Numeric, (correlate(a,v) == correlate(v,
         a), and the conjugate is not taken for complex arrays). If False, uses
         the conventional signal processing definition (see note).
 
     See Also
     --------
-    convolve : Discrete, linear convolution of two
-               one-dimensional sequences.
-    acorrelate : Discrete correlation following the usual signal processing
-               definition for complex arrays, and without assuming that
-               ``correlate(a, b) == correlate(b, a)``.
-
-    Notes
-    -----
-    If `old_behavior` is False, this function computes the correlation as
-    generally defined in signal processing texts::
-
-        z[k] = sum_n a[n] * conj(v[n+k])
-
-    with a and v sequences being zero-padded where necessary and conj being
-    the conjugate.
+    convolve : Discrete, linear convolution of two one-dimensional sequences.
 
     Examples
     --------
@@ -667,11 +679,13 @@ def correlate(a,v,mode='valid',old_behavior=True):
 
     """
     mode = _mode_from_name(mode)
+# the old behavior should be made available under a different name, see thread
+# http://thread.gmane.org/gmane.comp.python.numeric.general/12609/focus=12630
     if old_behavior:
         warnings.warn("""
-The current behavior of correlate is deprecated for 1.4.0, and will be removed
-for NumPy 1.5.0.
-    
+The old behavior of correlate was deprecated for 1.4.0, and will be completely removed
+for NumPy 2.0.
+
 The new behavior fits the conventional definition of correlation: inputs are
 never swapped, and the second argument is conjugated for complex arrays.""",
             DeprecationWarning)
@@ -719,15 +733,15 @@ def convolve(a,v,mode='full'):
 
     See Also
     --------
-    scipy.signal.fftconv : Convolve two arrays using the Fast Fourier
-                           Transform.
+    scipy.signal.fftconvolve : Convolve two arrays using the Fast Fourier
+                               Transform.
     scipy.linalg.toeplitz : Used to construct the convolution operator.
 
     Notes
     -----
     The discrete convolution operation is defined as
 
-    .. math:: (f * g)[n] = \\sum_{m = -\\infty}^{\\infty} f[m] f[n - m]
+    .. math:: (f * g)[n] = \\sum_{m = -\\infty}^{\\infty} f[m] g[n - m]
 
     It can be shown that a convolution :math:`x(t) * y(t)` in time/space
     is equivalent to the multiplication :math:`X(f) Y(f)` in the Fourier
@@ -948,6 +962,9 @@ def tensordot(a, b, axes=2):
             [aaaaaaacccccccc, bbbbbbbdddddddd]]], dtype=object)
 
     >>> np.tensordot(a, A, 0) # "Left for reader" (result too long to incl.)
+    array([[[[[a, b],
+              [c, d]],
+              ...
 
     >>> np.tensordot(a, A, (0, 1))
     array([[[abbbbb, cddddd],
@@ -1106,7 +1123,8 @@ def rollaxis(a, axis, start=0):
         The axis to roll backwards.  The positions of the other axes do not
         change relative to one another.
     start : int, optional
-        The axis is rolled until it lies before this position.
+        The axis is rolled until it lies before this position.  The default,
+        0, results in a "complete" roll.
 
     Returns
     -------
@@ -1116,7 +1134,7 @@ def rollaxis(a, axis, start=0):
     See Also
     --------
     roll : Roll the elements of an array by a number of positions along a
-           given axis.
+        given axis.
 
     Examples
     --------
@@ -1373,23 +1391,25 @@ def array_str(a, max_line_width=None, precision=None, suppress_small=None):
     """
     Return a string representation of the data in an array.
 
-    The data in the array is returned as a single string. This function
-    is similar to `array_repr`, the difference is that `array_repr` also
-    returns information on the type of array and data type.
+    The data in the array is returned as a single string.  This function is
+    similar to `array_repr`, the difference being that `array_repr` also
+    returns information on the kind of array and its data type.
 
     Parameters
     ----------
     a : ndarray
         Input array.
     max_line_width : int, optional
-        Inserts newlines if text is longer than `max_line_width`.
+        Inserts newlines if text is longer than `max_line_width`.  The
+        default is, indirectly, 75.
     precision : int, optional
-        Floating point precision. Default is the current printing precision
-        (usually 8), which can be altered using set_printoptions.
+        Floating point precision.  Default is the current printing precision
+        (usually 8), which can be altered using `set_printoptions`.
     suppress_small : bool, optional
-        Represent very small numbers as zero, default is False. Very small is
-        defined by precision, if the precision is 8 then numbers smaller than
-        5e-9 are represented as zero.
+        Represent numbers "very close" to zero as zero; default is False.
+        Very close is defined by precision: if the precision is 8, e.g.,
+        numbers smaller (in absolute value) than 5e-9 are represented as
+        zero.
 
     See Also
     --------
@@ -1398,12 +1418,70 @@ def array_str(a, max_line_width=None, precision=None, suppress_small=None):
     Examples
     --------
     >>> np.array_str(np.arange(3))
-    >>> '[0 1 2]'
+    '[0 1 2]'
 
     """
     return array2string(a, max_line_width, precision, suppress_small, ' ', "", str)
 
-set_string_function = multiarray.set_string_function
+def set_string_function(f, repr=True):
+    """
+    Set a Python function to be used when pretty printing arrays.
+
+    Parameters
+    ----------
+    f : function or None
+        Function to be used to pretty print arrays. The function should expect
+        a single array argument and return a string of the representation of
+        the array. If None, the function is reset to the default NumPy function
+        to print arrays.
+    repr : bool, optional
+        If True (default), the function for pretty printing (``__repr__``)
+        is set, if False the function that returns the default string
+        representation (``__str__``) is set.
+
+    See Also
+    --------
+    set_printoptions, get_printoptions
+
+    Examples
+    --------
+    >>> def pprint(arr):
+    ...     return 'HA! - What are you going to do now?'
+    ...
+    >>> np.set_string_function(pprint)
+    >>> a = np.arange(10)
+    >>> a
+    HA! - What are you going to do now?
+    >>> print a
+    [0 1 2 3 4 5 6 7 8 9]
+
+    We can reset the function to the default:
+
+    >>> np.set_string_function(None)
+    >>> a
+    array([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
+
+    `repr` affects either pretty printing or normal string representation.
+    Note that ``__repr__`` is still affected by setting ``__str__``
+    because the width of each array element in the returned string becomes
+    equal to the length of the result of ``__str__()``.
+
+    >>> x = np.arange(4)
+    >>> np.set_string_function(lambda x:'random', repr=False)
+    >>> x.__str__()
+    'random'
+    >>> x.__repr__()
+    'array([     0,      1,      2,      3])'
+
+    """
+    if f is None:
+        if repr:
+            return multiarray.set_string_function(array_repr, 1)
+        else:
+            return multiarray.set_string_function(array_str, 0)
+    else:
+        return multiarray.set_string_function(f, repr)
+
 set_string_function(array_str, 0)
 set_string_function(array_repr, 1)
 
@@ -1450,7 +1528,7 @@ def indices(dimensions, dtype=int):
     --------
     >>> grid = np.indices((2, 3))
     >>> grid.shape
-    (2,2,3)
+    (2, 2, 3)
     >>> grid[0]        # row indices
     array([[0, 0, 0],
            [1, 1, 1]])
@@ -1667,13 +1745,13 @@ def binary_repr(num, width=None):
         bin = bin.zfill(width)
     return sign + bin
 
-def base_repr (number, base=2, padding=0):
+def base_repr(number, base=2, padding=0):
     """
     Return a string representation of a number in the given base system.
 
     Parameters
     ----------
-    number : scalar
+    number : int
         The value to convert. Only positive values are handled.
     base : int, optional
         Convert `number` to the `base` number system. The valid range is 2-36,
@@ -1688,8 +1766,7 @@ def base_repr (number, base=2, padding=0):
 
     See Also
     --------
-    binary_repr : Faster version of `base_repr` for base 2 that also handles
-        negative numbers.
+    binary_repr : Faster version of `base_repr` for base 2.
 
     Examples
     --------
@@ -1706,29 +1783,24 @@ def base_repr (number, base=2, padding=0):
     '20'
 
     """
-    if number < 0:
-        raise ValueError("negative numbers not handled in base_repr")
-    if base > 36:
-        raise ValueError("bases greater than 36 not handled in base_repr")
+    digits = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
+    if base > len(digits):
+        raise ValueError("Bases greater than 36 not handled in base_repr.")
 
-    chars = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
-    import math
-    lnb = math.log(base)
-    res = padding*chars[0]
-    if number == 0:
-        return res + chars[0]
-    exponent = int (math.log (number)/lnb)
-    while(exponent >= 0):
-        term = long(base)**exponent
-        lead_digit = int(number / term)
-        res += chars[lead_digit]
-        number -= term*lead_digit
-        exponent -= 1
-    return res
+    num = abs(number)
+    res = []
+    while num:
+        res.append(digits[num % base])
+        num //= base
+    if padding:
+        res.append('0' * padding)
+    if number < 0:
+        res.append('-')
+    return ''.join(reversed(res or '0'))
 
 from cPickle import load, loads
 _cload = load
-_file = file
+_file = open
 
 def load(file):
     """
@@ -1765,11 +1837,11 @@ def ones(shape, dtype=None, order='C'):
     """
     Return a new array of given shape and type, filled with ones.
 
-    Please refer to the documentation for `zeros`.
+    Please refer to the documentation for `zeros` for further details.
 
     See Also
     --------
-    zeros
+    zeros, ones_like
 
     Examples
     --------
@@ -1958,19 +2030,19 @@ def array_equiv(a1, a2):
     Examples
     --------
     >>> np.array_equiv([1, 2], [1, 2])
-    >>> True
+    True
     >>> np.array_equiv([1, 2], [1, 3])
-    >>> False
+    False
 
     Showing the shape equivalence:
 
     >>> np.array_equiv([1, 2], [[1, 2], [1, 2]])
-    >>> True
+    True
     >>> np.array_equiv([1, 2], [[1, 2, 1, 2], [1, 2, 1, 2]])
-    >>> False
+    False
 
     >>> np.array_equiv([1, 2], [[1, 2], [1, 3]])
-    >>> False
+    False
 
     """
     try:
@@ -2049,11 +2121,12 @@ def seterr(all=None, divide=None, over=None, under=None, invalid=None):
 
     Examples
     --------
+    >>> old_settings = np.seterr(all='ignore')  #seterr to known value
     >>> np.seterr(over='raise')
     {'over': 'ignore', 'divide': 'ignore', 'invalid': 'ignore',
      'under': 'ignore'}
     >>> np.seterr(all='ignore')  # reset to default
-    {'over': 'raise', 'divide': 'warn', 'invalid': 'warn', 'under': 'warn'}
+    {'over': 'raise', 'divide': 'ignore', 'invalid': 'ignore', 'under': 'ignore'}
 
     >>> np.int16(32000) * np.int16(3)
     30464
@@ -2063,20 +2136,13 @@ def seterr(all=None, divide=None, over=None, under=None, invalid=None):
       File "<stdin>", line 1, in <module>
     FloatingPointError: overflow encountered in short_scalars
 
-    >>> np.seterr(all='print')
+    >>> old_settings = np.seterr(all='print')
+    >>> np.geterr()
     {'over': 'print', 'divide': 'print', 'invalid': 'print', 'under': 'print'}
     >>> np.int16(32000) * np.int16(3)
     Warning: overflow encountered in short_scalars
     30464
 
-    Calling `seterr` with no arguments resets treatment for all floating-point
-    errors to the defaults.
-
-    >>> old_settings = np.seterr()
-    >>> np.geterr()
-    {'over': 'ignore', 'divide': 'ignore', 'invalid': 'ignore',
-    'under': 'ignore'}
-
     """
 
     pyvals = umath.geterrobj()
@@ -2219,7 +2285,7 @@ def seterrcall(func):
     Callback upon error:
 
     >>> def err_handler(type, flag):
-            print "Floating point error (%s), with flag %s" % (type, flag)
+    ...     print "Floating point error (%s), with flag %s" % (type, flag)
     ...
 
     >>> saved_handler = np.seterrcall(err_handler)
@@ -2230,13 +2296,15 @@ def seterrcall(func):
     array([ Inf,  Inf,  Inf])
 
     >>> np.seterrcall(saved_handler)
+    <function err_handler at 0x...>
     >>> np.seterr(**save_err)
+    {'over': 'call', 'divide': 'call', 'invalid': 'call', 'under': 'call'}
 
     Log error message:
 
     >>> class Log(object):
-            def write(self, msg):
-                print "LOG: %s" % msg
+    ...     def write(self, msg):
+    ...         print "LOG: %s" % msg
     ...
 
     >>> log = Log()
@@ -2245,9 +2313,13 @@ def seterrcall(func):
 
     >>> np.array([1, 2, 3]) / 0.0
     LOG: Warning: divide by zero encountered in divide
+    <BLANKLINE>
+    array([ Inf,  Inf,  Inf])
 
     >>> np.seterrcall(saved_handler)
+    <__main__.Log object at 0x...>
     >>> np.seterr(**save_err)
+    {'over': 'log', 'divide': 'log', 'invalid': 'log', 'under': 'log'}
 
     """
     if func is not None and not callable(func):
@@ -2356,7 +2428,6 @@ class errstate(object):
     nan
     >>> with np.errstate(invalid='raise'):
     ...     np.sqrt(-1)
-    ...
     Traceback (most recent call last):
       File "<stdin>", line 2, in <module>
     FloatingPointError: invalid value encountered in sqrt
