@@ -104,8 +104,8 @@ def win32_checks(deflist):
     a = get_build_architecture()
 
     # Distutils hack on AMD64 on windows
-    print 'BUILD_ARCHITECTURE: %r, os.name=%r, sys.platform=%r' % \
-          (a, os.name, sys.platform)
+    print('BUILD_ARCHITECTURE: %r, os.name=%r, sys.platform=%r' % \
+          (a, os.name, sys.platform))
     if a == 'AMD64':
         deflist.append('DISTUTILS_USE_SDK')
 
@@ -181,11 +181,20 @@ def check_complex(config, mathlibs):
                 pub.append(('NPY_HAVE_%s' % type2def(t), 1))
 
         def check_prec(prec):
-            flist = [f + prec for f in C99_COMPLEX_TYPES]
-            if not config.check_funcs_once(flist):
+            flist = [f + prec for f in C99_COMPLEX_FUNCS]
+            decl = dict([(f, True) for f in flist])
+            if not config.check_funcs_once(flist, call=decl, decl=decl,
+                                           libraries=mathlibs):
                 for f in flist:
-                    if config.check_func(f):
+                    if config.check_func(f, call=True, decl=True,
+                                         libraries=mathlibs):
                         priv.append(fname2def(f))
+            else:
+                priv.extend([fname2def(f) for f in flist])
+
+        check_prec('')
+        check_prec('f')
+        check_prec('l')
 
     return priv, pub
 
@@ -195,6 +204,10 @@ def check_ieee_macros(config):
 
     macros = []
 
+    def _add_decl(f):
+        priv.append(fname2def("decl_%s" % f))
+        pub.append('NPY_%s' % fname2def("decl_%s" % f))
+
     # XXX: hack to circumvent cpp pollution from python: python put its
     # config.h in the public namespace, so we have a clash for the common
     # functions we test. We remove every function tested by python's
@@ -202,9 +215,14 @@ def check_ieee_macros(config):
     _macros = ["isnan", "isinf", "signbit", "isfinite"]
     if sys.version_info[:2] >= (2, 6):
         for f in _macros:
-            st = config.check_decl(fname2def("decl_%s" % f),
+            py_symbol = fname2def("decl_%s" % f)
+            already_declared = config.check_decl(py_symbol,
                     headers=["Python.h", "math.h"])
-            if not st:
+            if already_declared:
+                if config.check_macro_true(py_symbol,
+                        headers=["Python.h", "math.h"]):
+                    pub.append('NPY_%s' % fname2def("decl_%s" % f))
+            else:
                 macros.append(f)
     else:
         macros = _macros[:]
@@ -216,8 +234,7 @@ def check_ieee_macros(config):
     for f in macros:
         st = config.check_decl(f, headers = ["Python.h", "math.h"])
         if st:
-            priv.append(fname2def("decl_%s" % f))
-            pub.append('NPY_%s' % fname2def("decl_%s" % f))
+            _add_decl(f)
 
     return priv, pub
 
@@ -243,8 +260,12 @@ def check_types(config_cmd, ext, build_dir):
     result = config_cmd.check_header('Python.h')
     if not result:
         raise SystemError(
-                "Cannot compiler 'Python.h'. Perhaps you need to "\
+                "Cannot compile 'Python.h'. Perhaps you need to "\
                 "install python-dev|python-devel.")
+    res = config_cmd.check_header("endian.h")
+    if res:
+        private_defines.append(('HAVE_ENDIAN_H', 1))
+        public_defines.append(('NPY_HAVE_ENDIAN_H', 1))
 
     # Check basic types sizes
     for type in ('short', 'int', 'long'):
@@ -414,11 +435,16 @@ def configuration(parent_package='',top_path=None):
                 if rep in ['INTEL_EXTENDED_12_BYTES_LE',
                            'INTEL_EXTENDED_16_BYTES_LE',
                            'IEEE_QUAD_LE', 'IEEE_QUAD_BE',
-                           'IEEE_DOUBLE_LE', 'IEEE_DOUBLE_BE']:
+                           'IEEE_DOUBLE_LE', 'IEEE_DOUBLE_BE',
+                           'DOUBLE_DOUBLE_BE']:
                     moredefs.append(('HAVE_LDOUBLE_%s' % rep, 1))
                 else:
                     raise ValueError("Unrecognized long double format: %s" % rep)
 
+            # Py3K check
+            if sys.version_info[0] == 3:
+                moredefs.append(('NPY_PY3K', 1))
+
             # Generate the config.h file from moredefs
             target_f = open(target, 'w')
             for d in moredefs:
@@ -444,11 +470,11 @@ def configuration(parent_package='',top_path=None):
 """)
 
             target_f.close()
-            print 'File:',target
+            print('File:',target)
             target_f = open(target)
-            print target_f.read()
+            print(target_f.read())
             target_f.close()
-            print 'EOF'
+            print('EOF')
         else:
             mathlibs = []
             target_f = open(target)
@@ -529,11 +555,11 @@ def configuration(parent_package='',top_path=None):
             target_f.close()
 
             # Dump the numpyconfig.h header to stdout
-            print 'File: %s' % target
+            print('File: %s' % target)
             target_f = open(target)
-            print target_f.read()
+            print(target_f.read())
             target_f.close()
-            print 'EOF'
+            print('EOF')
         config.add_data_files((header_dir, target))
         return target
 
@@ -690,6 +716,7 @@ def configuration(parent_package='',top_path=None):
             join('src', 'multiarray', 'mapping.h'),
             join('src', 'multiarray', 'methods.h'),
             join('src', 'multiarray', 'multiarraymodule.h'),
+            join('src', 'multiarray', 'numpymemoryview.h'),
             join('src', 'multiarray', 'number.h'),
             join('src', 'multiarray', 'numpyos.h'),
             join('src', 'multiarray', 'refcount.h'),
@@ -702,6 +729,7 @@ def configuration(parent_package='',top_path=None):
     multiarray_src = [join('src', 'multiarray', 'multiarraymodule.c'),
         join('src', 'multiarray', 'hashdescr.c'),
         join('src', 'multiarray', 'arrayobject.c'),
+        join('src', 'multiarray', 'numpymemoryview.c'),
         join('src', 'multiarray', 'buffer.c'),
         join('src', 'multiarray', 'numpyos.c'),
         join('src', 'multiarray', 'conversion_utils.c'),
