@@ -8,6 +8,8 @@
 
 #include "npy_config.h"
 
+#include "numpy/npy_3kcompat.h"
+
 #include "common.h"
 #include "number.h"
 
@@ -99,7 +101,7 @@ PyArray_ArgMax(PyArrayObject *op, int axis, PyArrayObject *out)
     }
 
     if (!out) {
-        rp = (PyArrayObject *)PyArray_New(ap->ob_type, ap->nd-1,
+        rp = (PyArrayObject *)PyArray_New(Py_TYPE(ap), ap->nd-1,
                                           ap->dimensions, PyArray_INTP,
                                           NULL, NULL, 0, 0,
                                           (PyObject *)ap);
@@ -391,18 +393,23 @@ __New_PyArray_Std(PyArrayObject *self, int axis, int rtype, PyArrayObject *out,
         ret = PyArray_GenericUnaryFunction((PyAO *)obj1, n_ops.sqrt);
         Py_DECREF(obj1);
     }
-    if (ret == NULL || PyArray_CheckExact(self)) {
-        return ret;
+    if (ret == NULL) {
+        return NULL;
     }
-    if (PyArray_Check(self) && self->ob_type == ret->ob_type) {
-        return ret;
+    if (PyArray_CheckExact(self)) {
+        goto finish;
+    }
+    if (PyArray_Check(self) && Py_TYPE(self) == Py_TYPE(ret)) {
+        goto finish;
     }
     obj1 = PyArray_EnsureArray(ret);
     if (obj1 == NULL) {
         return NULL;
     }
-    ret = PyArray_View((PyAO *)obj1, NULL, self->ob_type);
+    ret = PyArray_View((PyAO *)obj1, NULL, Py_TYPE(self));
     Py_DECREF(obj1);
+
+finish:
     if (out) {
         if (PyArray_CopyAnyInto(out, (PyArrayObject *)ret) < 0) {
             Py_DECREF(ret);
@@ -667,7 +674,11 @@ PyArray_Mean(PyArrayObject *self, int axis, int rtype, PyArrayObject *out)
         return NULL;
     }
     if (!out) {
+#if defined(NPY_PY3K)
+        ret = PyNumber_TrueDivide(obj1, obj2);
+#else
         ret = PyNumber_Divide(obj1, obj2);
+#endif
     }
     else {
         ret = PyObject_CallFunction(n_ops.divide, "OOO", out, obj2, out);
@@ -940,7 +951,7 @@ PyArray_Clip(PyArrayObject *self, PyObject *min, PyObject *max, PyArrayObject *o
      */
     if (out == NULL) {
         Py_INCREF(indescr);
-        out = (NPY_AO*)PyArray_NewFromDescr(self->ob_type,
+        out = (NPY_AO*)PyArray_NewFromDescr(Py_TYPE(self),
                                             indescr, self->nd,
                                             self->dimensions,
                                             NULL, NULL,
