@@ -9,6 +9,8 @@
 
 #include "npy_config.h"
 
+#include "numpy/npy_3kcompat.h"
+
 #include "common.h"
 #include "iterators.h"
 #include "mapping.h"
@@ -52,7 +54,7 @@ array_big_item(PyArrayObject *self, intp i)
         return NULL;
     }
     Py_INCREF(self->descr);
-    r = (PyArrayObject *)PyArray_NewFromDescr(self->ob_type,
+    r = (PyArrayObject *)PyArray_NewFromDescr(Py_TYPE(self),
                                               self->descr,
                                               self->nd-1,
                                               self->dimensions+1,
@@ -231,7 +233,7 @@ PyArray_GetMap(PyArrayMapIterObject *mit)
     temp = mit->ait->ao;
     Py_INCREF(temp->descr);
     ret = (PyArrayObject *)
-        PyArray_NewFromDescr(temp->ob_type,
+        PyArray_NewFromDescr(Py_TYPE(temp),
                              temp->descr,
                              mit->nd, mit->dimensions,
                              NULL, NULL,
@@ -314,7 +316,7 @@ PyArray_SetMap(PyArrayMapIterObject *mit, PyObject *op)
             (PyArray_ISNOTSWAPPED(arr)));
     copyswap = PyArray_DESCR(arr)->f->copyswap;
     PyArray_MapIterReset(mit);
-    /* Need to decref hasobject arrays */
+    /* Need to decref arrays with objects in them */
     if (PyDataType_FLAGCHK(descr, NPY_ITEM_HASOBJECT)) {
         while (index--) {
             PyArray_Item_INCREF(it->dataptr, PyArray_DESCR(arr));
@@ -390,7 +392,7 @@ add_new_axes_0d(PyArrayObject *arr,  int newaxis_count)
     }
     Py_INCREF(arr->descr);
     if ((other = (PyArrayObject *)
-         PyArray_NewFromDescr(arr->ob_type, arr->descr,
+         PyArray_NewFromDescr(Py_TYPE(arr), arr->descr,
                               newaxis_count, dimensions,
                               NULL, arr->data,
                               arr->flags,
@@ -519,7 +521,7 @@ array_subscript_simple(PyArrayObject *self, PyObject *op)
     /* This will only work if new array will be a view */
     Py_INCREF(self->descr);
     if ((other = (PyArrayObject *)
-         PyArray_NewFromDescr(self->ob_type, self->descr,
+         PyArray_NewFromDescr(Py_TYPE(self), self->descr,
                               nd, dimensions,
                               strides, self->data+offset,
                               self->flags,
@@ -541,6 +543,8 @@ array_subscript(PyArrayObject *self, PyObject *op)
     PyObject *obj;
 
     if (PyString_Check(op) || PyUnicode_Check(op)) {
+        PyObject *temp;
+
         if (self->descr->names) {
             obj = PyDict_GetItem(self->descr->fields, op);
             if (obj != NULL) {
@@ -555,9 +559,16 @@ array_subscript(PyArrayObject *self, PyObject *op)
             }
         }
 
+        temp = op;
+        if (PyUnicode_Check(op)) {
+            temp = PyUnicode_AsUnicodeEscapeString(op);
+        }
         PyErr_Format(PyExc_ValueError,
                      "field named %s not found.",
-                     PyString_AsString(op));
+                     PyBytes_AsString(temp));
+        if (temp != op) {
+            Py_DECREF(temp);
+        }
         return NULL;
     }
 
@@ -620,7 +631,7 @@ array_subscript(PyArrayObject *self, PyObject *op)
             else {
                 intp oned = 0;
                 Py_INCREF(self->descr);
-                return PyArray_NewFromDescr(self->ob_type,
+                return PyArray_NewFromDescr(Py_TYPE(self),
                                             self->descr,
                                             1, &oned,
                                             NULL, NULL,
@@ -698,6 +709,12 @@ array_ass_sub_simple(PyArrayObject *self, PyObject *index, PyObject *op)
     }
     else {
         PyObject *tmp0;
+
+        /*
+         * Note: this code path should never be reached with an index that
+         *       produces scalars -- those are handled earlier in array_ass_sub
+         */
+
         tmp0 = PyObject_GetItem((PyObject *)self, index);
         if (tmp0 == NULL) {
             return -1;
@@ -830,9 +847,8 @@ array_ass_sub(PyArrayObject *self, PyObject *index, PyObject *op)
         return -1;
     }
 
-    /* optimization for integer-tuple */
-    if (self->nd > 1 &&
-        (PyTuple_Check(index) && (PyTuple_GET_SIZE(index) == self->nd))
+    /* Integer-tuple */
+    if (PyTuple_Check(index) && (PyTuple_GET_SIZE(index) == self->nd)
         && (_tuple_of_integers(index, vals, self->nd) >= 0)) {
         int i;
         char *item;
@@ -1355,7 +1371,7 @@ PyArray_MapIterBind(PyArrayMapIterObject *mit, PyArrayObject *arr)
                 PyErr_Format(PyExc_ValueError,
                              "unexpected object "       \
                              "(%s) in selection position %d",
-                             obj->ob_type->tp_name, i);
+                             Py_TYPE(obj)->tp_name, i);
                 goto fail;
             }
             else {
