@@ -1,4 +1,5 @@
 #define PY_SSIZE_T_CLEAN
+#include <stdarg.h>
 #include <Python.h>
 #include "structmember.h"
 
@@ -9,12 +10,40 @@
 
 #include "npy_config.h"
 
+#include "numpy/npy_3kcompat.h"
+
 #include "common.h"
 #include "ctors.h"
 #include "calculation.h"
 
 #include "methods.h"
 
+
+/* NpyArg_ParseKeywords
+ *
+ * Utility function that provides the keyword parsing functionality of
+ * PyArg_ParseTupleAndKeywords without having to have an args argument.
+ *
+ */
+static int
+NpyArg_ParseKeywords(PyObject *keys, const char *format, char **kwlist, ...)
+{
+    PyObject *args = PyTuple_New(0);
+    int ret;
+    va_list va;
+
+    if (args == NULL) {
+        PyErr_SetString(PyExc_RuntimeError,
+                "Failed to allocate new tuple");
+        return 0;
+    }
+    va_start(va, kwlist);
+    ret = PyArg_VaParseTupleAndKeywords(args, keys, format, kwlist, va);
+    va_end(va);
+    Py_DECREF(args);
+    return ret;
+}
+
 /* Should only be used if x is known to be an nd-array */
 #define _ARET(x) PyArray_Return((PyArrayObject *)(x))
 
@@ -71,25 +100,17 @@ array_put(PyArrayObject *self, PyObject *args, PyObject *kwds)
 static PyObject *
 array_reshape(PyArrayObject *self, PyObject *args, PyObject *kwds)
 {
+    static char *keywords[] = {"order", NULL};
     PyArray_Dims newshape;
     PyObject *ret;
     PyArray_ORDER order = PyArray_CORDER;
-    int n;
+    Py_ssize_t n = PyTuple_Size(args);
 
-    if (kwds != NULL) {
-        PyObject *ref;
-        ref = PyDict_GetItemString(kwds, "order");
-        if (ref == NULL) {
-            PyErr_SetString(PyExc_TypeError,
-                            "invalid keyword argument");
-            return NULL;
-        }
-        if ((PyArray_OrderConverter(ref, &order) == PY_FAIL)) {
-            return NULL;
-        }
+    if (!NpyArg_ParseKeywords(kwds, "|O&", keywords,
+                PyArray_OrderConverter, &order)) {
+        return NULL;
     }
 
-    n = PyTuple_Size(args);
     if (n <= 1) {
         if (PyTuple_GET_ITEM(args, 0) == Py_None) {
             return PyArray_View(self, NULL, NULL);
@@ -289,7 +310,7 @@ PyArray_GetField(PyArrayObject *self, PyArray_Descr *typed, int offset)
         Py_DECREF(typed);
         return NULL;
     }
-    ret = PyArray_NewFromDescr(self->ob_type,
+    ret = PyArray_NewFromDescr(Py_TYPE(self),
                                typed,
                                self->nd, self->dimensions,
                                self->strides,
@@ -342,7 +363,7 @@ PyArray_SetField(PyArrayObject *self, PyArray_Descr *dtype,
         Py_DECREF(dtype);
         return -1;
     }
-    ret = PyArray_NewFromDescr(self->ob_type,
+    ret = PyArray_NewFromDescr(Py_TYPE(self),
                                dtype, self->nd, self->dimensions,
                                self->strides, self->data + offset,
                                self->flags, (PyObject *)self);
@@ -475,7 +496,7 @@ array_tostring(PyArrayObject *self, PyObject *args, PyObject *kwds)
 static PyObject *
 array_tofile(PyArrayObject *self, PyObject *args, PyObject *kwds)
 {
-    int ret;
+    int ret, ret2;
     PyObject *file;
     FILE *fd;
     char *sep = "";
@@ -487,9 +508,8 @@ array_tofile(PyArrayObject *self, PyObject *args, PyObject *kwds)
         return NULL;
     }
 
-    if (PyString_Check(file) || PyUnicode_Check(file)) {
-        file = PyObject_CallFunction((PyObject *)&PyFile_Type,
-                                     "Os", file, "wb");
+    if (PyBytes_Check(file) || PyUnicode_Check(file)) {
+        file = npy_PyFile_OpenFile(file, "wb");
         if (file == NULL) {
             return NULL;
         }
@@ -497,7 +517,7 @@ array_tofile(PyArrayObject *self, PyObject *args, PyObject *kwds)
     else {
         Py_INCREF(file);
     }
-    fd = PyFile_AsFile(file);
+    fd = npy_PyFile_Dup(file, "wb");
     if (fd == NULL) {
         PyErr_SetString(PyExc_IOError, "first argument must be a " \
                         "string or open file");
@@ -505,8 +525,9 @@ array_tofile(PyArrayObject *self, PyObject *args, PyObject *kwds)
         return NULL;
     }
     ret = PyArray_ToFile(self, fd, sep, format);
+    ret2 = npy_PyFile_DupClose(file, fd);
     Py_DECREF(file);
-    if (ret < 0) {
+    if (ret < 0 || ret2 < 0) {
         return NULL;
     }
     Py_INCREF(Py_None);
@@ -776,9 +797,9 @@ array_wraparray(PyArrayObject *self, PyObject *args)
         return NULL;
     }
 
-    if (self->ob_type != arr->ob_type){
+    if (Py_TYPE(self) != Py_TYPE(arr)){
         Py_INCREF(PyArray_DESCR(arr));
-        ret = PyArray_NewFromDescr(self->ob_type,
+        ret = PyArray_NewFromDescr(Py_TYPE(self),
                                    PyArray_DESCR(arr),
                                    PyArray_NDIM(arr),
                                    PyArray_DIMS(arr),
@@ -816,8 +837,14 @@ array_preparearray(PyArrayObject *self, PyObject *args)
         return NULL;
     }
 
+    if (Py_TYPE(self) == Py_TYPE(arr)) {
+        /* No need to create a new view */
+        Py_INCREF(arr);
+        return arr;
+    }
+
     Py_INCREF(PyArray_DESCR(arr));
-    ret = PyArray_NewFromDescr(self->ob_type,
+    ret = PyArray_NewFromDescr(Py_TYPE(self),
                                PyArray_DESCR(arr),
                                PyArray_NDIM(arr),
                                PyArray_DIMS(arr),
@@ -849,7 +876,7 @@ array_getarray(PyArrayObject *self, PyObject *args)
         PyObject *new;
         PyTypeObject *subtype = &PyArray_Type;
 
-        if (!PyType_IsSubtype(self->ob_type, &PyArray_Type)) {
+        if (!PyType_IsSubtype(Py_TYPE(self), &PyArray_Type)) {
             subtype = &PyArray_Type;
         }
 
@@ -896,52 +923,41 @@ array_copy(PyArrayObject *self, PyObject *args)
     return PyArray_NewCopy(self, fortran);
 }
 
-
+#include <stdio.h>
 static PyObject *
 array_resize(PyArrayObject *self, PyObject *args, PyObject *kwds)
 {
-    PyArray_Dims newshape;
-    PyObject *ret;
-    int n;
+    static char *kwlist[] = {"refcheck", NULL};
+    Py_ssize_t size = PyTuple_Size(args);
     int refcheck = 1;
-    PyArray_ORDER fortran = PyArray_ANYORDER;
+    PyArray_Dims newshape;
+    PyObject *ret, *obj;
 
-    if (kwds != NULL) {
-        PyObject *ref;
-        ref = PyDict_GetItemString(kwds, "refcheck");
-        if (ref) {
-            refcheck = PyInt_AsLong(ref);
-            if (refcheck==-1 && PyErr_Occurred()) {
-                return NULL;
-            }
-        }
-        ref = PyDict_GetItemString(kwds, "order");
-        if (ref != NULL ||
-            (PyArray_OrderConverter(ref, &fortran) == PY_FAIL)) {
-            return NULL;
-        }
+
+    if (!NpyArg_ParseKeywords(kwds, "|i", kwlist,  &refcheck)) {
+        return NULL;
     }
-    n = PyTuple_Size(args);
-    if (n <= 1) {
-        if (PyTuple_GET_ITEM(args, 0) == Py_None) {
+
+    if (size == 0) {
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
+    else if (size == 1) {
+        obj = PyTuple_GET_ITEM(args, 0);
+        if (obj == Py_None) {
             Py_INCREF(Py_None);
             return Py_None;
         }
-        if (!PyArg_ParseTuple(args, "O&", PyArray_IntpConverter,
-                              &newshape)) {
-            return NULL;
-        }
+        args = obj;
     }
-    else {
-        if (!PyArray_IntpConverter(args, &newshape)) {
-            if (!PyErr_Occurred()) {
-                PyErr_SetString(PyExc_TypeError,
-                                "invalid shape");
-            }
-            return NULL;
+    if (!PyArray_IntpConverter(args, &newshape)) {
+        if (!PyErr_Occurred()) {
+            PyErr_SetString(PyExc_TypeError, "invalid shape");
         }
+        return NULL;
     }
-    ret = PyArray_Resize(self, &newshape, refcheck, fortran);
+
+    ret = PyArray_Resize(self, &newshape, refcheck, PyArray_CORDER);
     PyDimMem_FREE(newshape.ptr);
     if (ret == NULL) {
         return NULL;
@@ -968,12 +984,12 @@ array_repeat(PyArrayObject *self, PyObject *args, PyObject *kwds) {
 static PyObject *
 array_choose(PyArrayObject *self, PyObject *args, PyObject *kwds)
 {
+    static char *keywords[] = {"out", "mode", NULL};
     PyObject *choices;
-    int n;
     PyArrayObject *out = NULL;
     NPY_CLIPMODE clipmode = NPY_RAISE;
+    Py_ssize_t n = PyTuple_Size(args);
 
-    n = PyTuple_Size(args);
     if (n <= 1) {
         if (!PyArg_ParseTuple(args, "O", &choices)) {
             return NULL;
@@ -982,15 +998,11 @@ array_choose(PyArrayObject *self, PyObject *args, PyObject *kwds)
     else {
         choices = args;
     }
-    if (kwds && PyDict_Check(kwds)) {
-        if (PyArray_OutputConverter(PyDict_GetItemString(kwds, "out"),
-                                    &out) == PY_FAIL) {
-            return NULL;
-        }
-        if (PyArray_ClipmodeConverter(PyDict_GetItemString(kwds, "mode"),
-                                      &clipmode) == PY_FAIL) {
-            return NULL;
-        }
+
+    if (!NpyArg_ParseKeywords(kwds, "|O&O&", keywords,
+                PyArray_OutputConverter, &out,
+                PyArray_ClipmodeConverter, &clipmode)) {
+        return NULL;
     }
 
     return _ARET(PyArray_Choose(self, choices, out, clipmode));
@@ -1274,7 +1286,7 @@ array_reduce(PyArrayObject *self, PyObject *NPY_UNUSED(args))
     PyTuple_SET_ITEM(ret, 0, obj);
     PyTuple_SET_ITEM(ret, 1,
                      Py_BuildValue("ONc",
-                                   (PyObject *)self->ob_type,
+                                   (PyObject *)Py_TYPE(self),
                                    Py_BuildValue("(N)",
                                                  PyInt_FromLong(0)),
                                    /* dummy data-type */
@@ -1336,6 +1348,7 @@ array_setstate(PyArrayObject *self, PyObject *args)
     Py_ssize_t len;
     intp size, dimensions[MAX_DIMS];
     int nd;
+    int incref_base = 1;
 
     /* This will free any memory associated with a and
        use the string in setstate as the (writeable) memory.
@@ -1387,13 +1400,23 @@ array_setstate(PyArrayObject *self, PyObject *args)
         }
     }
     else {
-        if (!PyString_Check(rawdata)) {
+#if defined(NPY_PY3K)
+        /* Backward compatibility with Python 2 Numpy pickles */
+        if (PyUnicode_Check(rawdata)) {
+            PyObject *tmp;
+            tmp = PyUnicode_AsLatin1String(rawdata);
+            rawdata = tmp;
+            incref_base = 0;
+        }
+#endif
+
+        if (!PyBytes_Check(rawdata)) {
             PyErr_SetString(PyExc_TypeError,
                             "pickle not returning string");
             return NULL;
         }
 
-        if (PyString_AsStringAndSize(rawdata, &datastr, &len))
+        if (PyBytes_AsStringAndSize(rawdata, &datastr, &len))
             return NULL;
 
         if ((len != (self->descr->elsize * size))) {
@@ -1471,7 +1494,9 @@ array_setstate(PyArrayObject *self, PyObject *args)
         }
         else {
             self->base = rawdata;
-            Py_INCREF(self->base);
+            if (incref_base) {
+                Py_INCREF(self->base);
+            }
         }
     }
     else {
@@ -1510,12 +1535,16 @@ PyArray_Dump(PyObject *self, PyObject *file, int protocol)
         protocol = 2;
     }
 
+#if defined(NPY_PY3K)
+    cpick = PyImport_ImportModule("pickle");
+#else
     cpick = PyImport_ImportModule("cPickle");
+#endif
     if (cpick == NULL) {
         return -1;
     }
-    if PyString_Check(file) {
-        file = PyFile_FromString(PyString_AS_STRING(file), "wb");
+    if (PyBytes_Check(file) || PyUnicode_Check(file)) {
+        file = npy_PyFile_OpenFile(file, "wb");
         if (file == NULL) {
             return -1;
         }
@@ -1542,7 +1571,11 @@ PyArray_Dumps(PyObject *self, int protocol)
     if (protocol < 0) {
         protocol = 2;
     }
+#if defined(NPY_PY3K)
+    cpick = PyImport_ImportModule("pickle");
+#else
     cpick = PyImport_ImportModule("cPickle");
+#endif
     if (cpick == NULL) {
         return NULL;
     }
@@ -1584,11 +1617,10 @@ static PyObject *
 array_transpose(PyArrayObject *self, PyObject *args)
 {
     PyObject *shape = Py_None;
-    int n;
+    Py_ssize_t n = PyTuple_Size(args);
     PyArray_Dims permute;
     PyObject *ret;
 
-    n = PyTuple_Size(args);
     if (n > 1) {
         shape = args;
     }
@@ -1753,6 +1785,29 @@ array_cumprod(PyArrayObject *self, PyObject *args, PyObject *kwds)
 }
 
 
+static PyObject *
+array_dot(PyArrayObject *self, PyObject *args, PyObject *kwds)
+{
+    PyObject *b;
+    static PyObject *numpycore = NULL;
+
+    if (!PyArg_ParseTuple(args, "O", &b)) {
+        return NULL;
+    }
+
+    /* Since blas-dot is exposed only on the Python side, we need to grab it
+     * from there */
+    if (numpycore == NULL) {
+        numpycore = PyImport_ImportModule("numpy.core");
+        if (numpycore == NULL) {
+            return NULL;
+        }
+    }
+
+    return PyObject_CallMethod(numpycore, "dot", "OO", self, b);
+}
+
+
 static PyObject *
 array_any(PyArrayObject *self, PyObject *args, PyObject *kwds)
 {
@@ -2072,130 +2127,192 @@ array_newbyteorder(PyArrayObject *self, PyObject *args)
 NPY_NO_EXPORT PyMethodDef array_methods[] = {
 
     /* for subtypes */
-    {"__array__", (PyCFunction)array_getarray,
-         METH_VARARGS, NULL},
-    {"__array_prepare__", (PyCFunction)array_preparearray,
-         METH_VARARGS, NULL},
-    {"__array_wrap__", (PyCFunction)array_wraparray,
-         METH_VARARGS, NULL},
+    {"__array__",
+        (PyCFunction)array_getarray,
+        METH_VARARGS, NULL},
+    {"__array_prepare__",
+        (PyCFunction)array_preparearray,
+        METH_VARARGS, NULL},
+    {"__array_wrap__",
+        (PyCFunction)array_wraparray,
+        METH_VARARGS, NULL},
 
     /* for the copy module */
-    {"__copy__", (PyCFunction)array_copy,
-         METH_VARARGS, NULL},
-    {"__deepcopy__", (PyCFunction)array_deepcopy,
-         METH_VARARGS, NULL},
+    {"__copy__",
+        (PyCFunction)array_copy,
+        METH_VARARGS, NULL},
+    {"__deepcopy__",
+        (PyCFunction)array_deepcopy,
+        METH_VARARGS, NULL},
 
     /* for Pickling */
-    {"__reduce__", (PyCFunction) array_reduce,
-         METH_VARARGS, NULL},
-    {"__setstate__", (PyCFunction) array_setstate,
-         METH_VARARGS, NULL},
-    {"dumps", (PyCFunction) array_dumps,
-         METH_VARARGS, NULL},
-    {"dump", (PyCFunction) array_dump,
-         METH_VARARGS, NULL},
+    {"__reduce__",
+        (PyCFunction) array_reduce,
+        METH_VARARGS, NULL},
+    {"__setstate__",
+        (PyCFunction) array_setstate,
+        METH_VARARGS, NULL},
+    {"dumps",
+        (PyCFunction) array_dumps,
+        METH_VARARGS, NULL},
+    {"dump",
+        (PyCFunction) array_dump,
+        METH_VARARGS, NULL},
 
     /* Original and Extended methods added 2005 */
-    {"all", (PyCFunction)array_all,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"any", (PyCFunction)array_any,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"argmax", (PyCFunction)array_argmax,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"argmin", (PyCFunction)array_argmin,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"argsort", (PyCFunction)array_argsort,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"astype", (PyCFunction)array_cast,
-         METH_VARARGS, NULL},
-    {"byteswap", (PyCFunction)array_byteswap,
-         METH_VARARGS, NULL},
-    {"choose", (PyCFunction)array_choose,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"clip", (PyCFunction)array_clip,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"compress", (PyCFunction)array_compress,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"conj", (PyCFunction)array_conjugate,
-         METH_VARARGS, NULL},
-    {"conjugate", (PyCFunction)array_conjugate,
-         METH_VARARGS, NULL},
-    {"copy", (PyCFunction)array_copy,
-         METH_VARARGS, NULL},
-    {"cumprod", (PyCFunction)array_cumprod,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"cumsum", (PyCFunction)array_cumsum,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"diagonal", (PyCFunction)array_diagonal,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"fill", (PyCFunction)array_fill,
-         METH_VARARGS, NULL},
-    {"flatten", (PyCFunction)array_flatten,
-         METH_VARARGS, NULL},
-    {"getfield", (PyCFunction)array_getfield,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"item", (PyCFunction)array_toscalar,
-         METH_VARARGS, NULL},
-    {"itemset", (PyCFunction) array_setscalar,
-         METH_VARARGS, NULL},
-    {"max", (PyCFunction)array_max,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"mean", (PyCFunction)array_mean,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"min", (PyCFunction)array_min,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"newbyteorder", (PyCFunction)array_newbyteorder,
-         METH_VARARGS, NULL},
-    {"nonzero", (PyCFunction)array_nonzero,
-         METH_VARARGS, NULL},
-    {"prod", (PyCFunction)array_prod,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"ptp", (PyCFunction)array_ptp,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"put", (PyCFunction)array_put,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"ravel", (PyCFunction)array_ravel,
-         METH_VARARGS, NULL},
-    {"repeat", (PyCFunction)array_repeat,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"reshape", (PyCFunction)array_reshape,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"resize", (PyCFunction)array_resize,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"round", (PyCFunction)array_round,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"searchsorted", (PyCFunction)array_searchsorted,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"setfield", (PyCFunction)array_setfield,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"setflags", (PyCFunction)array_setflags,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"sort", (PyCFunction)array_sort,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"squeeze", (PyCFunction)array_squeeze,
-         METH_VARARGS, NULL},
-    {"std", (PyCFunction)array_stddev,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"sum", (PyCFunction)array_sum,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"swapaxes", (PyCFunction)array_swapaxes,
-         METH_VARARGS, NULL},
-    {"take", (PyCFunction)array_take,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"tofile", (PyCFunction)array_tofile,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"tolist", (PyCFunction)array_tolist,
-         METH_VARARGS, NULL},
-    {"tostring", (PyCFunction)array_tostring,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"trace", (PyCFunction)array_trace,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"transpose", (PyCFunction)array_transpose,
-         METH_VARARGS, NULL},
-    {"var", (PyCFunction)array_variance,
-         METH_VARARGS | METH_KEYWORDS, NULL},
-    {"view", (PyCFunction)array_view,
-         METH_VARARGS | METH_KEYWORDS, NULL},
+    {"all",
+        (PyCFunction)array_all,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"any",
+        (PyCFunction)array_any,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"argmax",
+        (PyCFunction)array_argmax,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"argmin",
+        (PyCFunction)array_argmin,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"argsort",
+        (PyCFunction)array_argsort,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"astype",
+        (PyCFunction)array_cast,
+        METH_VARARGS, NULL},
+    {"byteswap",
+        (PyCFunction)array_byteswap,
+        METH_VARARGS, NULL},
+    {"choose",
+        (PyCFunction)array_choose,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"clip",
+        (PyCFunction)array_clip,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"compress",
+        (PyCFunction)array_compress,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"conj",
+        (PyCFunction)array_conjugate,
+        METH_VARARGS, NULL},
+    {"conjugate",
+        (PyCFunction)array_conjugate,
+        METH_VARARGS, NULL},
+    {"copy",
+        (PyCFunction)array_copy,
+        METH_VARARGS, NULL},
+    {"cumprod",
+        (PyCFunction)array_cumprod,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"cumsum",
+        (PyCFunction)array_cumsum,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"diagonal",
+        (PyCFunction)array_diagonal,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"dot",
+        (PyCFunction)array_dot,
+        METH_VARARGS, NULL},
+    {"fill",
+        (PyCFunction)array_fill,
+        METH_VARARGS, NULL},
+    {"flatten",
+        (PyCFunction)array_flatten,
+        METH_VARARGS, NULL},
+    {"getfield",
+        (PyCFunction)array_getfield,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"item",
+        (PyCFunction)array_toscalar,
+        METH_VARARGS, NULL},
+    {"itemset",
+        (PyCFunction) array_setscalar,
+        METH_VARARGS, NULL},
+    {"max",
+        (PyCFunction)array_max,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"mean",
+        (PyCFunction)array_mean,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"min",
+        (PyCFunction)array_min,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"newbyteorder",
+        (PyCFunction)array_newbyteorder,
+        METH_VARARGS, NULL},
+    {"nonzero",
+        (PyCFunction)array_nonzero,
+        METH_VARARGS, NULL},
+    {"prod",
+        (PyCFunction)array_prod,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"ptp",
+        (PyCFunction)array_ptp,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"put",
+        (PyCFunction)array_put,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"ravel",
+        (PyCFunction)array_ravel,
+        METH_VARARGS, NULL},
+    {"repeat",
+        (PyCFunction)array_repeat,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"reshape",
+        (PyCFunction)array_reshape,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"resize",
+        (PyCFunction)array_resize,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"round",
+        (PyCFunction)array_round,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"searchsorted",
+        (PyCFunction)array_searchsorted,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"setfield",
+        (PyCFunction)array_setfield,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"setflags",
+        (PyCFunction)array_setflags,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"sort",
+        (PyCFunction)array_sort,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"squeeze",
+        (PyCFunction)array_squeeze,
+        METH_VARARGS, NULL},
+    {"std",
+        (PyCFunction)array_stddev,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"sum",
+        (PyCFunction)array_sum,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"swapaxes",
+        (PyCFunction)array_swapaxes,
+        METH_VARARGS, NULL},
+    {"take",
+        (PyCFunction)array_take,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"tofile",
+        (PyCFunction)array_tofile,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"tolist",
+        (PyCFunction)array_tolist,
+        METH_VARARGS, NULL},
+    {"tostring",
+        (PyCFunction)array_tostring,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"trace",
+        (PyCFunction)array_trace,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"transpose",
+        (PyCFunction)array_transpose,
+        METH_VARARGS, NULL},
+    {"var",
+        (PyCFunction)array_variance,
+        METH_VARARGS | METH_KEYWORDS, NULL},
+    {"view",
+        (PyCFunction)array_view,
+        METH_VARARGS | METH_KEYWORDS, NULL},
     {NULL, NULL, 0, NULL}           /* sentinel */
 };
 
