@@ -1,8 +1,11 @@
+import sys
+from decimal import Decimal
+
+import numpy as np
 from numpy.core import *
 from numpy.random import rand, randint, randn
 from numpy.testing import *
 from numpy.core.multiarray import dot as dot_
-import sys
 
 class Vec:
     def __init__(self,sequence=None):
@@ -23,15 +26,9 @@ class Vec:
         return out
     def __rmul__(self,other):
         return self*other
-    def __abs__(self):
-        out=Vec()
-        out.array=abs(self.array)
-        return out
-    def __repr__(self):
-        return "Vec("+repr(self.array.tolist())+")"
-    __str__=__repr__
 
-class TestDot(NumpyTestCase):
+
+class TestDot(TestCase):
     def setUp(self):
         self.A = rand(10,8)
         self.b1 = rand(8,1)
@@ -40,87 +37,83 @@ class TestDot(NumpyTestCase):
         self.b4 = rand(10)
         self.N = 14
 
-    def check_matmat(self):
+    def test_matmat(self):
         A = self.A
         c1 = dot(A.transpose(), A)
         c2 = dot_(A.transpose(), A)
         assert_almost_equal(c1, c2, decimal=self.N)
 
-    def check_matvec(self):
+    def test_matvec(self):
         A, b1 = self.A, self.b1
         c1 = dot(A, b1)
         c2 = dot_(A, b1)
         assert_almost_equal(c1, c2, decimal=self.N)
 
-    def check_matvec2(self):
+    def test_matvec2(self):
         A, b2 = self.A, self.b2
         c1 = dot(A, b2)
         c2 = dot_(A, b2)
         assert_almost_equal(c1, c2, decimal=self.N)
 
-    def check_vecmat(self):
+    def test_vecmat(self):
         A, b4 = self.A, self.b4
         c1 = dot(b4, A)
         c2 = dot_(b4, A)
         assert_almost_equal(c1, c2, decimal=self.N)
 
-    def check_vecmat2(self):
+    def test_vecmat2(self):
         b3, A = self.b3, self.A
         c1 = dot(b3, A.transpose())
         c2 = dot_(b3, A.transpose())
         assert_almost_equal(c1, c2, decimal=self.N)
 
-    def check_vecmat3(self):
+    def test_vecmat3(self):
         A, b4 = self.A, self.b4
         c1 = dot(A.transpose(),b4)
         c2 = dot_(A.transpose(),b4)
         assert_almost_equal(c1, c2, decimal=self.N)
 
-    def check_vecvecouter(self):
+    def test_vecvecouter(self):
         b1, b3 = self.b1, self.b3
         c1 = dot(b1, b3)
         c2 = dot_(b1, b3)
         assert_almost_equal(c1, c2, decimal=self.N)
 
-    def check_vecvecinner(self):
+    def test_vecvecinner(self):
         b1, b3 = self.b1, self.b3
         c1 = dot(b3, b1)
         c2 = dot_(b3, b1)
         assert_almost_equal(c1, c2, decimal=self.N)
 
-    def check_matscalar(self):
-        b1 = matrix(ones((3,3),dtype=complex))
-        assert_equal(b1*1.0, b1)
-
-    def check_columnvect(self):
+    def test_columnvect1(self):
         b1 = ones((3,1))
         b2 = [5.3]
         c1 = dot(b1,b2)
         c2 = dot_(b1,b2)
         assert_almost_equal(c1, c2, decimal=self.N)
 
-    def check_columnvect(self):
+    def test_columnvect2(self):
         b1 = ones((3,1)).transpose()
         b2 = [6.2]
         c1 = dot(b2,b1)
         c2 = dot_(b2,b1)
         assert_almost_equal(c1, c2, decimal=self.N)
 
-    def check_vecscalar(self):
+    def test_vecscalar(self):
         b1 = rand(1,1)
         b2 = rand(1,8)
         c1 = dot(b1,b2)
         c2 = dot_(b1,b2)
         assert_almost_equal(c1, c2, decimal=self.N)
 
-    def check_vecscalar2(self):
+    def test_vecscalar2(self):
         b1 = rand(8,1)
         b2 = rand(1,1)
         c1 = dot(b1,b2)
         c2 = dot_(b1,b2)
         assert_almost_equal(c1, c2, decimal=self.N)
 
-    def check_all(self):
+    def test_all(self):
         dims = [(),(1,),(1,1)]
         for dim1 in dims:
             for dim2 in dims:
@@ -131,7 +124,7 @@ class TestDot(NumpyTestCase):
                 assert (c1.shape == c2.shape)
                 assert_almost_equal(c1, c2, decimal=self.N)
 
-    def check_vecobject(self):
+    def test_vecobject(self):
         U_non_cont = transpose([[1.,1.],[1.,2.]])
         U_cont = ascontiguousarray(U_non_cont)
         x = array([Vec([1.,0.]),Vec([0.,1.])])
@@ -141,65 +134,124 @@ class TestDot(NumpyTestCase):
         assert_equal(zeros[1].array, zeros_test[1].array)
 
 
-class TestBoolScalar(NumpyTestCase):
+class TestResize(TestCase):
+    def test_copies(self):
+        A = array([[1,2],[3,4]])
+        Ar1 = array([[1,2,3,4],[1,2,3,4]])
+        assert_equal(resize(A, (2,4)), Ar1)
+
+        Ar2 = array([[1,2],[3,4],[1,2],[3,4]])
+        assert_equal(resize(A, (4,2)), Ar2)
+
+        Ar3 = array([[1,2,3],[4,1,2],[3,4,1],[2,3,4]])
+        assert_equal(resize(A, (4,3)), Ar3)
+
+    def test_zeroresize(self):
+        A = array([[1,2],[3,4]])
+        Ar = resize(A, (0,))
+        assert_equal(Ar, array([]))
+
+
+class TestNonarrayArgs(TestCase):
+    # check that non-array arguments to functions wrap them in arrays
+    def test_squeeze(self):
+        A = [[[1,1,1],[2,2,2],[3,3,3]]]
+        assert squeeze(A).shape == (3,3)
+
+    def test_cumproduct(self):
+        A = [[1,2,3],[4,5,6]]
+        assert all(cumproduct(A) == array([1,2,6,24,120,720]))
+
+    def test_size(self):
+        A = [[1,2,3],[4,5,6]]
+        assert size(A) == 6
+        assert size(A,0) == 2
+        assert size(A,1) == 3
+
+    def test_mean(self):
+        A = [[1,2,3],[4,5,6]]
+        assert mean(A) == 3.5
+        assert all(mean(A,0) == array([2.5,3.5,4.5]))
+        assert all(mean(A,1) == array([2.,5.]))
+
+    def test_std(self):
+        A = [[1,2,3],[4,5,6]]
+        assert_almost_equal(std(A), 1.707825127659933)
+        assert_almost_equal(std(A,0), array([1.5, 1.5, 1.5]))
+        assert_almost_equal(std(A,1), array([0.81649658, 0.81649658]))
+
+    def test_var(self):
+        A = [[1,2,3],[4,5,6]]
+        assert_almost_equal(var(A), 2.9166666666666665)
+        assert_almost_equal(var(A,0), array([2.25, 2.25, 2.25]))
+        assert_almost_equal(var(A,1), array([0.66666667, 0.66666667]))
+
+
+class TestBoolScalar(TestCase):
     def test_logical(self):
         f = False_
         t = True_
         s = "xyz"
-        self.failUnless((t and s) is s)
-        self.failUnless((f and s) is f)
+        self.assertTrue((t and s) is s)
+        self.assertTrue((f and s) is f)
 
     def test_bitwise_or(self):
         f = False_
         t = True_
-        self.failUnless((t | t) is t)
-        self.failUnless((f | t) is t)
-        self.failUnless((t | f) is t)
-        self.failUnless((f | f) is f)
+        self.assertTrue((t | t) is t)
+        self.assertTrue((f | t) is t)
+        self.assertTrue((t | f) is t)
+        self.assertTrue((f | f) is f)
 
     def test_bitwise_and(self):
         f = False_
         t = True_
-        self.failUnless((t & t) is t)
-        self.failUnless((f & t) is f)
-        self.failUnless((t & f) is f)
-        self.failUnless((f & f) is f)
+        self.assertTrue((t & t) is t)
+        self.assertTrue((f & t) is f)
+        self.assertTrue((t & f) is f)
+        self.assertTrue((f & f) is f)
 
     def test_bitwise_xor(self):
         f = False_
         t = True_
-        self.failUnless((t ^ t) is f)
-        self.failUnless((f ^ t) is t)
-        self.failUnless((t ^ f) is t)
-        self.failUnless((f ^ f) is f)
+        self.assertTrue((t ^ t) is f)
+        self.assertTrue((f ^ t) is t)
+        self.assertTrue((t ^ f) is t)
+        self.assertTrue((f ^ f) is f)
 
 
-class TestSeterr(NumpyTestCase):
+class TestSeterr(TestCase):
     def test_set(self):
         err = seterr()
-        old = seterr(divide='warn')
-        self.failUnless(err == old)
-        new = seterr()
-        self.failUnless(new['divide'] == 'warn')
-        seterr(over='raise')
-        self.failUnless(geterr()['over'] == 'raise')
-        self.failUnless(new['divide'] == 'warn')
-        seterr(**old)
-        self.failUnless(geterr() == old)
-    def test_divideerr(self):
-        seterr(divide='raise')
         try:
+            old = seterr(divide='warn')
+            self.assertTrue(err == old)
+            new = seterr()
+            self.assertTrue(new['divide'] == 'warn')
+            seterr(over='raise')
+            self.assertTrue(geterr()['over'] == 'raise')
+            self.assertTrue(new['divide'] == 'warn')
+            seterr(**old)
+            self.assertTrue(geterr() == old)
+        finally:
+            seterr(**err)
+
+    def test_divide_err(self):
+        err = seterr(divide='raise')
+        try:
+            try:
+                array([1.]) / array([0.])
+            except FloatingPointError:
+                pass
+            else:
+                self.fail()
+            seterr(divide='ignore')
             array([1.]) / array([0.])
-        except FloatingPointError:
-            pass
-        else:
-            self.fail()
-        seterr(divide='ignore')
-        array([1.]) / array([0.])
+        finally:
+            seterr(**err)
 
 
-class TestFromiter(NumpyTestCase):
-
+class TestFromiter(TestCase):
     def makegen(self):
         for x in xrange(24):
             yield x**2
@@ -208,16 +260,16 @@ class TestFromiter(NumpyTestCase):
         ai32 = fromiter(self.makegen(), int32)
         ai64 = fromiter(self.makegen(), int64)
         af = fromiter(self.makegen(), float)
-        self.failUnless(ai32.dtype == dtype(int32))
-        self.failUnless(ai64.dtype == dtype(int64))
-        self.failUnless(af.dtype == dtype(float))
+        self.assertTrue(ai32.dtype == dtype(int32))
+        self.assertTrue(ai64.dtype == dtype(int64))
+        self.assertTrue(af.dtype == dtype(float))
 
     def test_lengths(self):
         expected = array(list(self.makegen()))
         a = fromiter(self.makegen(), int)
         a20 = fromiter(self.makegen(), int, 20)
-        self.failUnless(len(a) == len(expected))
-        self.failUnless(len(a20) == 20)
+        self.assertTrue(len(a) == len(expected))
+        self.assertTrue(len(a20) == 20)
         try:
             fromiter(self.makegen(), int, len(expected) + 10)
         except ValueError:
@@ -229,10 +281,11 @@ class TestFromiter(NumpyTestCase):
         expected = array(list(self.makegen()))
         a = fromiter(self.makegen(), int)
         a20 = fromiter(self.makegen(), int, 20)
-        self.failUnless(alltrue(a == expected,axis=0))
-        self.failUnless(alltrue(a20 == expected[:20],axis=0))
+        self.assertTrue(alltrue(a == expected,axis=0))
+        self.assertTrue(alltrue(a20 == expected[:20],axis=0))
 
-class TestIndex(NumpyTestCase):
+
+class TestIndex(TestCase):
     def test_boolean(self):
         a = rand(3,5,8)
         V = rand(5,8)
@@ -241,7 +294,8 @@ class TestIndex(NumpyTestCase):
         V[g1,g2] = -V[g1,g2]
         assert (array([a[0][V>0],a[1][V>0],a[2][V>0]]) == a[:,V>0]).all()
 
-class TestBinaryRepr(NumpyTestCase):
+
+class TestBinaryRepr(TestCase):
     def test_zero(self):
         assert_equal(binary_repr(0),'0')
 
@@ -252,7 +306,22 @@ class TestBinaryRepr(NumpyTestCase):
         assert_equal(binary_repr(-1), '-1')
         assert_equal(binary_repr(-1, width=8), '11111111')
 
-class TestArrayComparisons(NumpyTestCase):
+class TestBaseRepr(TestCase):
+    def test_base3(self):
+        assert_equal(base_repr(3**5, 3), '100000')
+
+    def test_positive(self):
+        assert_equal(base_repr(12, 10), '12')
+        assert_equal(base_repr(12, 10, 4), '000012')
+        assert_equal(base_repr(12, 4), '30')
+        assert_equal(base_repr(3731624803700888, 36), '10QR0ROFCEW')
+
+    def test_negative(self):
+        assert_equal(base_repr(-12, 10), '-12')
+        assert_equal(base_repr(-12, 10, 4), '-000012')
+        assert_equal(base_repr(-12, 4), '-30')
+
+class TestArrayComparisons(TestCase):
     def test_array_equal(self):
         res = array_equal(array([1,2]), array([1,2]))
         assert res
@@ -306,7 +375,7 @@ def assert_array_strict_equal(x, y):
     assert x.dtype.isnative == y.dtype.isnative
 
 
-class TestClip(NumpyTestCase):
+class TestClip(TestCase):
     def setUp(self):
         self.nr = 5
         self.nc = 3
@@ -333,17 +402,16 @@ class TestClip(NumpyTestCase):
         return (randn(n, m)).astype(float32)
 
     def _neg_byteorder(self, a):
-        import sys
-        a   = asarray(a)
+        a = asarray(a)
         if sys.byteorder == 'little':
-            a   = a.astype(a.dtype.newbyteorder('>'))
+            a = a.astype(a.dtype.newbyteorder('>'))
         else:
-            a   = a.astype(a.dtype.newbyteorder('<'))
+            a = a.astype(a.dtype.newbyteorder('<'))
         return a
 
     def _generate_non_native_data(self, n, m):
-        data    = randn(n, m)
-        data    = self._neg_byteorder(data)
+        data = randn(n, m)
+        data = self._neg_byteorder(data)
         assert not data.dtype.isnative
         return data
 
@@ -555,7 +623,7 @@ class TestClip(NumpyTestCase):
         ac  = self.clip(a,m,M)
         assert_array_strict_equal(ac, act)
 
-    def test_type_cast_04(self):
+    def test_type_cast_05(self):
         "Test native int32 with double arrays min/max."
         a   = self._generate_int_data(self.nr, self.nc)
         m   = -0.5
@@ -564,7 +632,7 @@ class TestClip(NumpyTestCase):
         act = self.clip(a, m * zeros(a.shape), M)
         assert_array_strict_equal(ac, act)
 
-    def test_type_cast_05(self):
+    def test_type_cast_06(self):
         "Test native with NON native scalar min/max."
         a   = self._generate_data(self.nr, self.nc)
         m   = 0.5
@@ -574,7 +642,7 @@ class TestClip(NumpyTestCase):
         ac  = self.fastclip(a, m_s, M)
         assert_array_strict_equal(ac, act)
 
-    def test_type_cast_06(self):
+    def test_type_cast_07(self):
         "Test NON native with native array min/max."
         a   = self._generate_data(self.nr, self.nc)
         m   = -0.5 * ones(a.shape)
@@ -585,7 +653,7 @@ class TestClip(NumpyTestCase):
         ac  = self.fastclip(a_s, m, M)
         assert_array_strict_equal(ac, act)
 
-    def test_type_cast_07(self):
+    def test_type_cast_08(self):
         "Test NON native with native scalar min/max."
         a   = self._generate_data(self.nr, self.nc)
         m   = -0.5
@@ -596,7 +664,7 @@ class TestClip(NumpyTestCase):
         act = a_s.clip(m, M)
         assert_array_strict_equal(ac, act)
 
-    def test_type_cast_08(self):
+    def test_type_cast_09(self):
         "Test native with NON native array min/max."
         a   = self._generate_data(self.nr, self.nc)
         m   = -0.5 * ones(a.shape)
@@ -607,7 +675,7 @@ class TestClip(NumpyTestCase):
         act = self.clip(a, m_s, M)
         assert_array_strict_equal(ac, act)
 
-    def test_type_cast_09(self):
+    def test_type_cast_10(self):
         """Test native int32 with float min/max and float out for output argument."""
         a   = self._generate_int_data(self.nr, self.nc)
         b   = zeros(a.shape, dtype = float32)
@@ -617,7 +685,7 @@ class TestClip(NumpyTestCase):
         ac  = self.fastclip(a, m , M, out = b)
         assert_array_strict_equal(ac, act)
 
-    def test_type_cast_10(self):
+    def test_type_cast_11(self):
         "Test non native with native scalar, min/max, out non native"
         a   = self._generate_non_native_data(self.nr, self.nc)
         b   = a.copy()
@@ -629,7 +697,7 @@ class TestClip(NumpyTestCase):
         self.clip(a, m, M, out = bt)
         assert_array_strict_equal(b, bt)
 
-    def test_type_cast_11(self):
+    def test_type_cast_12(self):
         "Test native int32 input and min/max and float out"
         a   = self._generate_int_data(self.nr, self.nc)
         b   = zeros(a.shape, dtype = float32)
@@ -727,7 +795,7 @@ class TestClip(NumpyTestCase):
         self.assert_(a2 is a)
 
 
-class test_allclose_inf(ParametricTestCase):
+class test_allclose_inf(TestCase):
     rtol = 1e-5
     atol = 1e-8
 
@@ -737,7 +805,7 @@ class test_allclose_inf(ParametricTestCase):
     def tst_not_allclose(self,x,y):
         assert not allclose(x,y), "%s and %s shouldn't be close" % (x,y)
 
-    def testip_allclose(self):
+    def test_ip_allclose(self):
         """Parametric test factory."""
         arr = array([100,1000])
         aran = arange(125).reshape((5,5,5))
@@ -755,7 +823,7 @@ class test_allclose_inf(ParametricTestCase):
         for (x,y) in data:
             yield (self.tst_allclose,x,y)
 
-    def testip_not_allclose(self):
+    def test_ip_not_allclose(self):
         """Parametric test factory."""
         aran = arange(125).reshape((5,5,5))
 
@@ -783,7 +851,8 @@ class test_allclose_inf(ParametricTestCase):
         assert_array_equal(x,array([inf,1]))
         assert_array_equal(y,array([0,inf]))
 
-class TestStdVar(NumpyTestCase):
+
+class TestStdVar(TestCase):
     def setUp(self):
         self.A = array([1,-1,1,-1])
         self.real_var = 1
@@ -791,25 +860,132 @@ class TestStdVar(NumpyTestCase):
     def test_basic(self):
         assert_almost_equal(var(self.A),self.real_var)
         assert_almost_equal(std(self.A)**2,self.real_var)
-    def test_ddof1(self):
-        assert_almost_equal(var(self.A,ddof=1),self.real_var*len(self.A)/float(len(self.A)-1))
-        assert_almost_equal(std(self.A,ddof=1)**2,self.real_var*len(self.A)/float(len(self.A)-1))
-    def test_ddof2(self):
-        assert_almost_equal(var(self.A,ddof=2),self.real_var*len(self.A)/float(len(self.A)-2))
-        assert_almost_equal(std(self.A,ddof=2)**2,self.real_var*len(self.A)/float(len(self.A)-2))
 
-class TestStdVarComplex(NumpyTestCase):
+    def test_ddof1(self):
+        assert_almost_equal(var(self.A,ddof=1),
+                            self.real_var*len(self.A)/float(len(self.A)-1))
+        assert_almost_equal(std(self.A,ddof=1)**2,
+                            self.real_var*len(self.A)/float(len(self.A)-1))
+
+    def test_ddof2(self):
+        assert_almost_equal(var(self.A,ddof=2),
+                            self.real_var*len(self.A)/float(len(self.A)-2))
+        assert_almost_equal(std(self.A,ddof=2)**2,
+                            self.real_var*len(self.A)/float(len(self.A)-2))
+
+
+class TestStdVarComplex(TestCase):
     def test_basic(self):
         A = array([1,1.j,-1,-1.j])
         real_var = 1
         assert_almost_equal(var(A),real_var)
         assert_almost_equal(std(A)**2,real_var)
 
-import sys
-if sys.version_info[:2] >= (2, 5):
-    set_local_path()
-    from test_errstate import *
-    restore_path()
 
-if __name__ == '__main__':
-    NumpyTest().run()
+class TestLikeFuncs(TestCase):
+    '''Test zeros_like and empty_like'''
+
+    def setUp(self):
+        self.data = [(array([[1,2,3],[4,5,6]],dtype=int32), (2,3), int32),
+                     (array([[1,2,3],[4,5,6]],dtype=float32), (2,3), float32),
+                     ]
+
+    def test_zeros_like(self):
+        for d, dshape, dtype in self.data:
+            dz = zeros_like(d)
+            assert dz.shape == dshape
+            assert dz.dtype.type == dtype
+            assert all(abs(dz) == 0)
+
+    def test_empty_like(self):
+        for d, dshape, dtype in self.data:
+            dz = zeros_like(d)
+            assert dz.shape == dshape
+            assert dz.dtype.type == dtype
+
+class _TestCorrelate(TestCase):
+    def _setup(self, dt):
+        self.x = np.array([1, 2, 3, 4, 5], dtype=dt)
+        self.y = np.array([-1, -2, -3], dtype=dt)
+        self.z1 = np.array([ -3.,  -8., -14., -20., -26., -14.,  -5.], dtype=dt)
+        self.z2 = np.array([ -5.,  -14., -26., -20., -14., -8.,  -3.], dtype=dt)
+
+    def test_float(self):
+        self._setup(np.float)
+        z = np.correlate(self.x, self.y, 'full', old_behavior=self.old_behavior)
+        assert_array_almost_equal(z, self.z1)
+        z = np.correlate(self.y, self.x, 'full', old_behavior=self.old_behavior)
+        assert_array_almost_equal(z, self.z2)
+
+    def test_object(self):
+        self._setup(Decimal)
+        z = np.correlate(self.x, self.y, 'full', old_behavior=self.old_behavior)
+        assert_array_almost_equal(z, self.z1)
+        z = np.correlate(self.y, self.x, 'full', old_behavior=self.old_behavior)
+        assert_array_almost_equal(z, self.z2)
+
+class TestCorrelate(_TestCorrelate):
+    old_behavior = True
+    def _setup(self, dt):
+        # correlate uses an unconventional definition so that correlate(a, b)
+        # == correlate(b, a), so force the corresponding outputs to be the same
+        # as well
+        _TestCorrelate._setup(self, dt)
+        self.z2 = self.z1
+
+    @dec.deprecated()
+    def test_complex(self):
+        x = np.array([1, 2, 3, 4+1j], dtype=np.complex)
+        y = np.array([-1, -2j, 3+1j], dtype=np.complex)
+        r_z = np.array([3+1j, 6, 8-1j, 9+1j, -1-8j, -4-1j], dtype=np.complex)
+        z = np.correlate(x, y, 'full', old_behavior=self.old_behavior)
+        assert_array_almost_equal(z, r_z)
+
+    @dec.deprecated()
+    def test_float(self):
+        _TestCorrelate.test_float(self)
+
+    @dec.deprecated()
+    def test_object(self):
+        _TestCorrelate.test_object(self)
+
+class TestCorrelateNew(_TestCorrelate):
+    old_behavior = False
+    def test_complex(self):
+        x = np.array([1, 2, 3, 4+1j], dtype=np.complex)
+        y = np.array([-1, -2j, 3+1j], dtype=np.complex)
+        r_z = np.array([3-1j, 6, 8+1j, 11+5j, -5+8j, -4-1j], dtype=np.complex)
+        #z = np.acorrelate(x, y, 'full')
+        #assert_array_almost_equal(z, r_z)
+
+        r_z = r_z[::-1].conjugate()
+        z = np.correlate(y, x, 'full', old_behavior=self.old_behavior)
+        assert_array_almost_equal(z, r_z)
+
+class TestArgwhere:
+    def test_2D(self):
+        x = np.arange(6).reshape((2, 3))
+        assert_array_equal(np.argwhere(x > 1),
+                           [[0, 2],
+                            [1, 0],
+                            [1, 1],
+                            [1, 2]])
+
+    def test_list(self):
+        assert_equal(np.argwhere([4, 0, 2, 1, 3]), [[0], [2], [3], [4]])
+
+class TestStringFunction:
+    def test_set_string_function(self):
+        a = np.array([1])
+        np.set_string_function(lambda x: "FOO", repr=True)
+        assert_equal(repr(a), "FOO")
+        np.set_string_function(None, repr=True)
+        assert_equal(repr(a), "array([1])")
+
+        np.set_string_function(lambda x: "FOO", repr=False)
+        assert_equal(str(a), "FOO")
+        np.set_string_function(None, repr=False)
+        assert_equal(str(a), "[1]")
+
+if __name__ == "__main__":
+    run_module_suite()
