@@ -1,5 +1,6 @@
 import sys
 from numpy.testing import *
+from numpy.compat import asbytes, asunicode
 import numpy as np
 
 # This is the structure of the table used for plain objects:
@@ -57,8 +58,8 @@ NbufferT = [
     # x     Info                                                color info        y                  z
     #       value y2 Info2                            name z2         Name Value
     #                name   value    y3       z3
-    ([3,2], (6j, 6., ('nn', [6j,4j], [6.,4.], [1,2]), 'NN', True), 'cc', ('NN', 6j), [[6.,4.],[6.,4.]], 8),
-    ([4,3], (7j, 7., ('oo', [7j,5j], [7.,5.], [2,1]), 'OO', False), 'dd', ('OO', 7j), [[7.,5.],[7.,5.]], 9),
+    ([3,2], (6j, 6., (asbytes('nn'), [6j,4j], [6.,4.], [1,2]), asbytes('NN'), True), asbytes('cc'), (asunicode('NN'), 6j), [[6.,4.],[6.,4.]], 8),
+    ([4,3], (7j, 7., (asbytes('oo'), [7j,5j], [7.,5.], [2,1]), asbytes('OO'), False), asbytes('dd'), (asunicode('OO'), 7j), [[7.,5.],[7.,5.]], 9),
     ]
 
 
@@ -77,7 +78,7 @@ def normalize_descr(descr):
                     dtype = "|" + dtype
                 else:
                     dtype = byteorder + dtype
-            if len(item) > 2 and item[2] > 1:
+            if len(item) > 2 and np.prod(item[2]) > 1:
                 nitem = (item[0], dtype, item[2])
             else:
                 nitem = (item[0], dtype)
@@ -307,7 +308,10 @@ class read_values_nested(object):
         h = np.array(self._buffer, dtype=self._descr)
         self.assert_(h.dtype['Info']['value'].name == 'complex128')
         self.assert_(h.dtype['Info']['y2'].name == 'float64')
-        self.assert_(h.dtype['info']['Name'].name == 'unicode256')
+        if sys.version_info[0] >= 3:
+            self.assert_(h.dtype['info']['Name'].name == 'str256')
+        else:
+            self.assert_(h.dtype['info']['Name'].name == 'unicode256')
         self.assert_(h.dtype['info']['Value'].name == 'complex128')
 
     def test_nested2_descriptor(self):
@@ -359,7 +363,7 @@ class TestMultipleFields(TestCase):
     def _bad_call(self):
         return self.ary['f0','f1']
     def test_no_tuple(self):
-        self.failUnlessRaises(ValueError, self._bad_call)
+        self.assertRaises(ValueError, self._bad_call)
     def test_return(self):
         res = self.ary[['f0','f2']].tolist()
         assert(res == [(1,3), (5,7)])        
