@@ -16,7 +16,8 @@ from numpy.distutils.exec_command import exec_command
 from numpy.distutils.system_info import combine_paths
 from numpy.distutils.misc_util import filter_sources, has_f_sources, \
      has_cxx_sources, get_ext_source_files, \
-     get_numpy_include_dirs, is_sequence
+     get_numpy_include_dirs, is_sequence, get_build_architecture, \
+     msvc_version
 from numpy.distutils.command.config_compiler import show_fortran_compilers
 
 try:
@@ -56,8 +57,21 @@ class build_ext (old_build_ext):
         self.run_command('build_src')
 
         if self.distribution.has_c_libraries():
-            self.run_command('build_clib')
-            build_clib = self.get_finalized_command('build_clib')
+            if self.inplace:
+                if self.distribution.have_run.get('build_clib'):
+                    log.warn('build_clib already run, it is too late to ' \
+                            'ensure in-place build of build_clib')
+                    build_clib = self.distribution.get_command_obj('build_clib')
+                else:
+                    build_clib = self.distribution.get_command_obj('build_clib')
+                    build_clib.inplace = 1
+                    build_clib.ensure_finalized()
+                    build_clib.run()
+                    self.distribution.have_run['build_clib'] = 1
+
+            else:
+                self.run_command('build_clib')
+                build_clib = self.get_finalized_command('build_clib')
             self.library_dirs.append(build_clib.build_clib)
         else:
             build_clib = None
@@ -84,14 +98,14 @@ class build_ext (old_build_ext):
         clibs = {}
         if build_clib is not None:
             for libname,build_info in build_clib.libraries or []:
-                if clibs.has_key(libname):
+                if libname in clibs and clibs[libname] != build_info:
                     log.warn('library %r defined more than once,'\
-                             ' overwriting build_info %r with %r.' \
-                             % (libname, clibs[libname], build_info))
+                             ' overwriting build_info\n%s... \nwith\n%s...' \
+                             % (libname, repr(clibs[libname])[:300], repr(build_info)[:300]))
                 clibs[libname] = build_info
         # .. and distribution libraries:
         for libname,build_info in self.distribution.libraries or []:
-            if clibs.has_key(libname):
+            if libname in clibs:
                 # build_clib libraries have a precedence before distribution ones
                 continue
             clibs[libname] = build_info
@@ -105,13 +119,14 @@ class build_ext (old_build_ext):
             c_lib_dirs = []
             macros = []
             for libname in ext.libraries:
-                if clibs.has_key(libname):
+                if libname in clibs:
                     binfo = clibs[libname]
                     c_libs += binfo.get('libraries',[])
                     c_lib_dirs += binfo.get('library_dirs',[])
                     for m in binfo.get('macros',[]):
                         if m not in macros:
                             macros.append(m)
+
                 for l in clibs.get(libname,{}).get('source_languages',[]):
                     ext_languages.add(l)
             if c_libs:
@@ -169,38 +184,46 @@ class build_ext (old_build_ext):
 
         # Initialize Fortran 77 compiler:
         if need_f77_compiler:
+            ctype = self.fcompiler
             self._f77_compiler = new_fcompiler(compiler=self.fcompiler,
                                                verbose=self.verbose,
                                                dry_run=self.dry_run,
                                                force=self.force,
-                                               requiref90=False)
+                                               requiref90=False,
+                                               c_compiler=self.compiler)
             fcompiler = self._f77_compiler
-            if fcompiler.get_version():
+            if fcompiler:
+                ctype = fcompiler.compiler_type
                 fcompiler.customize(self.distribution)
+            if fcompiler and fcompiler.get_version():
                 fcompiler.customize_cmd(self)
                 fcompiler.show_customization()
             else:
                 self.warn('f77_compiler=%s is not available.' %
-                          (fcompiler.compiler_type))
+                          (ctype))
                 self._f77_compiler = None
         else:
             self._f77_compiler = None
 
         # Initialize Fortran 90 compiler:
         if need_f90_compiler:
+            ctype = self.fcompiler
             self._f90_compiler = new_fcompiler(compiler=self.fcompiler,
                                                verbose=self.verbose,
                                                dry_run=self.dry_run,
                                                force=self.force,
-                                               requiref90=True)
+                                               requiref90=True,
+                                               c_compiler = self.compiler)
             fcompiler = self._f90_compiler
-            if fcompiler.get_version():
+            if fcompiler:
+                ctype = fcompiler.compiler_type
                 fcompiler.customize(self.distribution)
+            if fcompiler and fcompiler.get_version():
                 fcompiler.customize_cmd(self)
                 fcompiler.show_customization()
             else:
                 self.warn('f90_compiler=%s is not available.' %
-                          (fcompiler.compiler_type))
+                          (ctype))
                 self._f90_compiler = None
         else:
             self._f90_compiler = None
@@ -208,6 +231,12 @@ class build_ext (old_build_ext):
         # Build extensions
         self.build_extensions()
 
+        # Make sure that scons based extensions are complete.
+        if self.inplace:
+            cmd = self.reinitialize_command('scons')
+            cmd.inplace = 1
+        self.run_command('scons')
+
     def swig_sources(self, sources):
         # Do nothing. Swig sources have beed handled in build_src command.
         return sources
@@ -274,11 +303,11 @@ class build_ext (old_build_ext):
 
         # check for the availability of required compilers
         if cxx_sources and cxx_compiler is None:
-            raise DistutilsError, "extension %r has C++ sources" \
-                  "but no C++ compiler found" % (ext.name)
+            raise DistutilsError("extension %r has C++ sources" \
+                  "but no C++ compiler found" % (ext.name))
         if (f_sources or fmodule_sources) and fcompiler is None:
-            raise DistutilsError, "extension %r has Fortran sources " \
-                  "but no Fortran compiler found" % (ext.name)
+            raise DistutilsError("extension %r has Fortran sources " \
+                  "but no Fortran compiler found" % (ext.name))
         if ext.language in ['f77','f90'] and fcompiler is None:
             self.warn("extension %r has Fortran libraries " \
                   "but no Fortran linker found, using default linker" % (ext.name))
@@ -372,6 +401,7 @@ class build_ext (old_build_ext):
             # expand libraries with fcompiler libraries as we are
             # not using fcompiler linker
             self._libs_with_msvc_and_fortran(fcompiler, libraries, library_dirs)
+
         elif ext.language in ['f77','f90'] and fcompiler is not None:
             linker = fcompiler.link_shared_object
         if ext.language=='c++' and cxx_compiler is not None:
@@ -391,6 +421,14 @@ class build_ext (old_build_ext):
                debug=self.debug,
                build_temp=self.build_temp,**kws)
 
+    def _add_dummy_mingwex_sym(self, c_sources):
+        build_src = self.get_finalized_command("build_src").build_src
+        build_clib = self.get_finalized_command("build_clib").build_clib
+        objects = self.compiler.compile([os.path.join(build_src,
+                "gfortran_vs2003_hack.c")],
+                output_dir=self.build_temp)
+        self.compiler.create_static_lib(objects, "_gfortran_workaround", output_dir=build_clib, debug=self.debug)
+
     def _libs_with_msvc_and_fortran(self, fcompiler, c_libraries,
                                     c_library_dirs):
         if fcompiler is None: return
