@@ -19,12 +19,14 @@ __all__ = ['FCompiler','new_fcompiler','show_fcompilers',
 import os
 import sys
 import re
-import new
+import types
 try:
     set
 except NameError:
     from sets import Set as set
 
+from numpy.compat import open_latin1
+
 from distutils.sysconfig import get_config_var, get_python_lib
 from distutils.fancy_getopt import FancyGetopt
 from distutils.errors import DistutilsModuleError, \
@@ -36,6 +38,7 @@ from numpy.distutils import log
 from numpy.distutils.misc_util import is_string, all_strings, is_sequence, make_temp_file
 from numpy.distutils.environment import EnvironmentConfig
 from numpy.distutils.exec_command import find_executable
+from numpy.distutils.compat import get_exception
 
 __metaclass__ = type
 
@@ -129,7 +132,7 @@ class FCompiler(CCompiler):
         arch = ('flags.arch', 'FARCH', 'arch', flaglist),
         arch_f77 = ('flags.arch_f77', None, None, flaglist),
         arch_f90 = ('flags.arch_f90', None, None, flaglist),
-        debug = ('flags.debug', 'FDEBUG', 'fdebug', None, flaglist),
+        debug = ('flags.debug', 'FDEBUG', 'fdebug', flaglist),
         debug_f77 = ('flags.debug_f77', None, None, flaglist),
         debug_f90 = ('flags.debug_f90', None, None, flaglist),
         flags = ('self.get_flags', 'FFLAGS', 'fflags', flaglist),
@@ -223,7 +226,8 @@ class FCompiler(CCompiler):
         self._is_customised = False
 
     def __copy__(self):
-        obj = new.instance(self.__class__, self.__dict__)
+        obj = self.__new__(self.__class__)
+        obj.__dict__.update(self.__dict__)
         obj.distutils_vars = obj.distutils_vars.clone(obj._environment_hook)
         obj.command_vars = obj.command_vars.clone(obj._environment_hook)
         obj.flag_vars = obj.flag_vars.clone(obj._environment_hook)
@@ -417,7 +421,10 @@ class FCompiler(CCompiler):
 
     def get_version(self, force=False, ok_status=[0]):
         assert self._is_customised
-        return CCompiler.get_version(self, force=force, ok_status=ok_status)
+        version = CCompiler.get_version(self, force=force, ok_status=ok_status)
+        if version is None:
+            raise CompilerNotFound()
+        return version
 
     ############################################################
 
@@ -540,7 +547,7 @@ class FCompiler(CCompiler):
                                               % (self.__class__.__name__)):
             if l[:4]=='  --':
                 l = '  ' + l[4:]
-            print l
+            print(l)
 
     ###################
 
@@ -555,14 +562,14 @@ class FCompiler(CCompiler):
             flavor = ':f90'
             compiler = self.compiler_f90
             if compiler is None:
-                raise DistutilsExecError, 'f90 not supported by %s needed for %s'\
-                      % (self.__class__.__name__,src)
+                raise DistutilsExecError('f90 not supported by %s needed for %s'\
+                      % (self.__class__.__name__,src))
         else:
             flavor = ':fix'
             compiler = self.compiler_fix
             if compiler is None:
-                raise DistutilsExecError, 'f90 (fixed) not supported by %s needed for %s'\
-                      % (self.__class__.__name__,src)
+                raise DistutilsExecError('f90 (fixed) not supported by %s needed for %s'\
+                      % (self.__class__.__name__,src))
         if self.object_switch[-1]==' ':
             o_args = [self.object_switch.strip(),obj]
         else:
@@ -583,8 +590,9 @@ class FCompiler(CCompiler):
                               src)
         try:
             self.spawn(command,display=display)
-        except DistutilsExecError, msg:
-            raise CompileError, msg
+        except DistutilsExecError:
+            msg = str(get_exception())
+            raise CompileError(msg)
 
     def module_options(self, module_dirs, module_build_dir):
         options = []
@@ -594,14 +602,14 @@ class FCompiler(CCompiler):
             else:
                 options.append(self.module_dir_switch.strip()+module_build_dir)
         else:
-            print 'XXX: module_build_dir=%r option ignored' % (module_build_dir)
-            print 'XXX: Fix module_dir_switch for ',self.__class__.__name__
+            print('XXX: module_build_dir=%r option ignored' % (module_build_dir))
+            print('XXX: Fix module_dir_switch for ',self.__class__.__name__)
         if self.module_include_switch is not None:
             for d in [module_build_dir]+module_dirs:
                 options.append('%s%s' % (self.module_include_switch, d))
         else:
-            print 'XXX: module_dirs=%r option ignored' % (module_dirs)
-            print 'XXX: Fix module_include_switch for ',self.__class__.__name__
+            print('XXX: module_dirs=%r option ignored' % (module_dirs))
+            print('XXX: Fix module_include_switch for ',self.__class__.__name__)
         return options
 
     def library_option(self, lib):
@@ -623,7 +631,7 @@ class FCompiler(CCompiler):
         if is_string(output_dir):
             output_filename = os.path.join(output_dir, output_filename)
         elif output_dir is not None:
-            raise TypeError, "'output_dir' must be a string or None"
+            raise TypeError("'output_dir' must be a string or None")
 
         if self._need_link(objects, output_filename):
             if self.library_switch[-1]==' ':
@@ -650,8 +658,9 @@ class FCompiler(CCompiler):
             command = linker + ld_args
             try:
                 self.spawn(command)
-            except DistutilsExecError, msg:
-                raise LinkError, msg
+            except DistutilsExecError:
+                msg = str(get_exception())
+                raise LinkError(msg)
         else:
             log.debug("skipping %s (up-to-date)", output_filename)
 
@@ -681,7 +690,8 @@ class FCompiler(CCompiler):
 
 _default_compilers = (
     # sys.platform mappings
-    ('win32', ('gnu','intelv','absoft','compaqv','intelev','gnu95','g95')),
+    ('win32', ('gnu','intelv','absoft','compaqv','intelev','gnu95','g95',
+               'intelvem', 'intelem')),
     ('cygwin.*', ('gnu','intelv','absoft','compaqv','intelev','gnu95','g95')),
     ('linux.*', ('gnu','intel','lahey','pg','absoft','nag','vast','compaq',
                 'intele','intelem','gnu95','g95')),
@@ -854,7 +864,8 @@ def show_fcompilers(dist=None):
             c = new_fcompiler(compiler=compiler, verbose=dist.verbose)
             c.customize(dist)
             v = c.get_version()
-        except (DistutilsModuleError, CompilerNotFound), e:
+        except (DistutilsModuleError, CompilerNotFound):
+            e = get_exception()
             log.debug("show_fcompilers: %s not found" % (compiler,))
             log.debug(repr(e))
 
@@ -881,7 +892,7 @@ def show_fcompilers(dist=None):
     if compilers_ni:
         pretty_printer = FancyGetopt(compilers_ni)
         pretty_printer.print_help("Compilers not available on this platform:")
-    print "For compiler details, run 'config_fc --verbose' setup command."
+    print("For compiler details, run 'config_fc --verbose' setup command.")
 
 
 def dummy_fortran_file():
@@ -902,7 +913,7 @@ def is_free_format(file):
     # f90 allows both fixed and free format, assuming fixed unless
     # signs of free format are detected.
     result = 0
-    f = open(file,'r')
+    f = open_latin1(file,'r')
     line = f.readline()
     n = 10000 # the number of non-comment lines to scan for hints
     if _has_f_header(line):
@@ -922,7 +933,7 @@ def is_free_format(file):
     return result
 
 def has_f90_header(src):
-    f = open(src,'r')
+    f = open_latin1(src,'r')
     line = f.readline()
     f.close()
     return _has_f90_header(line) or _has_fix_header(line)
@@ -935,7 +946,7 @@ def get_f77flags(src):
     Return a dictionary {<fcompiler type>:<f77 flags>}.
     """
     flags = {}
-    f = open(src,'r')
+    f = open_latin1(src,'r')
     i = 0
     for line in f.readlines():
         i += 1
