@@ -63,19 +63,19 @@ The file 'site.cfg' is looked for in
 
 The first one found is used to get system configuration options The
 format is that used by ConfigParser (i.e., Windows .INI style). The
-section DEFAULT has options that are the default for each section. The
+section ALL has options that are the default for each section. The
 available sections are fftw, atlas, and x11. Appropiate defaults are
 used if nothing is specified.
 
 The order of finding the locations of resources is the following:
  1. environment variable
  2. section in site.cfg
- 3. DEFAULT section in site.cfg
+ 3. ALL section in site.cfg
 Only the first complete match is returned.
 
 Example:
 ----------
-[DEFAULT]
+[ALL]
 library_dirs = /usr/lib:/usr/local/lib:/opt/lib
 include_dirs = /usr/include:/usr/local/include:/opt/include
 src_dirs = /usr/local/src:/opt/src
@@ -116,7 +116,10 @@ import re
 import copy
 import warnings
 from glob import glob
-import ConfigParser
+if sys.version_info[0] < 3:
+    from ConfigParser import SafeConfigParser, NoOptionError, ConfigParser
+else:
+    from configparser import SafeConfigParser, NoOptionError, ConfigParser
 
 from distutils.errors import DistutilsError
 from distutils.dist import Distribution
@@ -127,6 +130,54 @@ from numpy.distutils.exec_command import \
     find_executable, exec_command, get_pythonexe
 from numpy.distutils.misc_util import is_sequence, is_string
 from numpy.distutils.command.config import config as cmd_config
+from numpy.distutils.compat import get_exception
+
+if sys.version_info[0] >= 3:
+    from functools import reduce
+
+# Determine number of bits
+import platform
+_bits = {'32bit':32,'64bit':64}
+platform_bits = _bits[platform.architecture()[0]]
+
+def libpaths(paths,bits):
+    """Return a list of library paths valid on 32 or 64 bit systems.
+
+    Inputs:
+      paths : sequence
+        A sequence of strings (typically paths)
+      bits : int
+        An integer, the only valid values are 32 or 64.  A ValueError exception
+      is raised otherwise.
+
+    Examples:
+
+    Consider a list of directories
+    >>> paths = ['/usr/X11R6/lib','/usr/X11/lib','/usr/lib']
+
+    For a 32-bit platform, this is already valid:
+    >>> np.distutils.system_info.libpaths(paths,32)
+    ['/usr/X11R6/lib', '/usr/X11/lib', '/usr/lib']
+
+    On 64 bits, we prepend the '64' postfix
+    >>> np.distutils.system_info.libpaths(paths,64)
+    ['/usr/X11R6/lib64', '/usr/X11R6/lib', '/usr/X11/lib64', '/usr/X11/lib',
+    '/usr/lib64', '/usr/lib']
+    """
+    if bits not in (32, 64):
+        raise ValueError("Invalid bit size in libpaths: 32 or 64 only")
+
+    # Handle 32bit case
+    if bits==32:
+        return paths
+
+    # Handle 64bit case
+    out = []
+    for p in paths:
+        out.extend([p+'64', p])
+
+    return out
+
 
 if sys.platform == 'win32':
     default_lib_dirs = ['C:\\',
@@ -137,24 +188,16 @@ if sys.platform == 'win32':
     default_x11_lib_dirs = []
     default_x11_include_dirs = []
 else:
-    default_lib_dirs = ['/usr/local/lib', '/opt/lib', '/usr/lib',
-                        '/opt/local/lib', '/sw/lib']
+    default_lib_dirs = libpaths(['/usr/local/lib','/opt/lib','/usr/lib',
+                                 '/opt/local/lib','/sw/lib'], platform_bits)
     default_include_dirs = ['/usr/local/include',
                             '/opt/include', '/usr/include',
-                            '/opt/local/include', '/sw/include']
+                            '/opt/local/include', '/sw/include',
+                            '/usr/include/suitesparse']
     default_src_dirs = ['.','/usr/local/src', '/opt/src','/sw/src']
 
-    try:
-        platform = os.uname()
-        bit64 = platform[-1].endswith('64')
-    except:
-        bit64 = False
-
-    if bit64:
-        default_x11_lib_dirs = ['/usr/lib64']
-    else:
-        default_x11_lib_dirs = ['/usr/X11R6/lib','/usr/X11/lib','/usr/lib']
-
+    default_x11_lib_dirs = libpaths(['/usr/X11R6/lib','/usr/X11/lib',
+                                     '/usr/lib'], platform_bits)
     default_x11_include_dirs = ['/usr/X11R6/include','/usr/X11/include',
                                 '/usr/include']
 
@@ -337,7 +380,7 @@ class system_info:
 
     """ get_info() is the only public method. Don't use others.
     """
-    section = 'DEFAULT'
+    section = 'ALL'
     dir_env_var = None
     search_static_first = 0 # XXX: disabled by default, may disappear in
                             # future unless it is proved to be useful.
@@ -359,19 +402,21 @@ class system_info:
         defaults['include_dirs'] = os.pathsep.join(default_include_dirs)
         defaults['src_dirs'] = os.pathsep.join(default_src_dirs)
         defaults['search_static_first'] = str(self.search_static_first)
-        self.cp = ConfigParser.ConfigParser(defaults)
+        self.cp = ConfigParser(defaults)
         self.files = []
         self.files.extend(get_standard_file('.numpy-site.cfg'))
         self.files.extend(get_standard_file('site.cfg'))
         self.parse_config_files()
-        self.search_static_first = self.cp.getboolean(self.section,
-                                                      'search_static_first')
+        if self.section is not None:
+            self.search_static_first = self.cp.getboolean(self.section,
+                                                          'search_static_first')
         assert isinstance(self.search_static_first, int)
 
     def parse_config_files(self):
         self.cp.read(self.files)
         if not self.cp.has_section(self.section):
-            self.cp.add_section(self.section)
+            if self.section is not None:
+                self.cp.add_section(self.section)
 
     def calc_libraries_info(self):
         libs = self.get_libraries()
@@ -396,7 +441,7 @@ class system_info:
         self.saved_results[self.__class__.__name__] = info
 
     def has_info(self):
-        return self.saved_results.has_key(self.__class__.__name__)
+        return self.__class__.__name__ in self.saved_results
 
     def get_info(self,notfound_action=0):
         """ Return a dictonary with items that are compatible
@@ -413,7 +458,7 @@ class system_info:
                     if notfound_action==1:
                         warnings.warn(self.notfounderror.__doc__)
                     elif notfound_action==2:
-                        raise self.notfounderror,self.notfounderror.__doc__
+                        raise self.notfounderror(self.notfounderror.__doc__)
                     else:
                         raise ValueError(repr(notfound_action))
 
@@ -441,13 +486,13 @@ class system_info:
             if is_sequence(env_var):
                 e0 = env_var[-1]
                 for e in env_var:
-                    if os.environ.has_key(e):
+                    if e in os.environ:
                         e0 = e
                         break
                 if not env_var[0]==e0:
                     log.info('Setting %s=%s' % (env_var[0],e0))
                 env_var = e0
-        if env_var and os.environ.has_key(env_var):
+        if env_var and env_var in os.environ:
             d = os.environ[env_var]
             if d=='None':
                 log.info('Disabled %s: %s',self.__class__.__name__,'(%s is None)' \
@@ -474,12 +519,17 @@ class system_info:
                             if os.path.isdir(d1):
                                 ds2.append(d1)
                 dirs = ds2 + dirs
-        default_dirs = self.cp.get('DEFAULT', key).split(os.pathsep)
+        default_dirs = self.cp.get(self.section, key).split(os.pathsep)
         dirs.extend(default_dirs)
         ret = []
         for d in dirs:
-            if os.path.isdir(d) and d not in ret:
+            if not os.path.isdir(d):
+                warnings.warn('Specified path %s is invalid.' % d)
+                continue
+
+            if d not in ret:
                 ret.append(d)
+
         log.debug('( %s = %s )', key, ':'.join(ret))
         return ret
 
@@ -495,7 +545,7 @@ class system_info:
     def get_libs(self, key, default):
         try:
             libs = self.cp.get(self.section, key)
-        except ConfigParser.NoOptionError:
+        except NoOptionError:
             if not default:
                 return []
             if is_string(default):
@@ -518,6 +568,11 @@ class system_info:
             exts.append('.dll.a')
         if sys.platform == 'darwin':
             exts.append('.dylib')
+        # Debian and Ubuntu added a g3f suffix to shared library to deal with
+        # g77 -> gfortran ABI transition
+        # XXX: disabled, it hides more problem than it solves.
+        #if sys.platform[:5] == 'linux':
+        #    exts.append('.so.3gf')
         return exts
 
     def check_libs(self,lib_dir,libs,opt_libs =[]):
@@ -1125,6 +1180,10 @@ class lapack_src_info(system_info):
         # Lapack 3.1:
         src_dir2 = os.path.join(src_dir,'..','INSTALL')
         sources += [os.path.join(src_dir2,p+'lamch.f') for p in 'sdcz']
+        # Lapack 3.2.1:
+        sources += [os.path.join(src_dir,p+'larfp.f') for p in 'sdcz']
+        sources += [os.path.join(src_dir,'ila'+p+'lr.f') for p in 'sdcz']
+        sources += [os.path.join(src_dir,'ila'+p+'lc.f') for p in 'sdcz']
         # Should we check here actual existence of source files?
         # Yes, the file listing is different between 3.0 and 3.1
         # versions.
@@ -1146,7 +1205,7 @@ def get_atlas_version(**config):
     libraries = config.get('libraries', [])
     library_dirs = config.get('library_dirs', [])
     key = (tuple(libraries), tuple(library_dirs))
-    if _cached_atlas_version.has_key(key):
+    if key in _cached_atlas_version:
         return _cached_atlas_version[key]
     c = cmd_config(Distribution())
     atlas_version = None
@@ -1229,7 +1288,7 @@ class lapack_opt_info(system_info):
         if atlas_info:
             version_info = atlas_info.copy()
             atlas_version = get_atlas_version(**version_info)
-            if not atlas_info.has_key('define_macros'):
+            if 'define_macros' not in atlas_info:
                 atlas_info['define_macros'] = []
             if atlas_version is None:
                 atlas_info['define_macros'].append(('NO_ATLAS_INFO',2))
@@ -1328,7 +1387,7 @@ class blas_opt_info(system_info):
         if atlas_info:
             version_info = atlas_info.copy()
             atlas_version = get_atlas_version(**version_info)
-            if not atlas_info.has_key('define_macros'):
+            if 'define_macros' not in atlas_info:
                 atlas_info['define_macros'] = []
             if atlas_version is None:
                 atlas_info['define_macros'].append(('NO_ATLAS_INFO',2))
@@ -1473,6 +1532,13 @@ class _numpy_info(system_info):
                 if name=='lib':
                     break
                 prefix.append(name)
+
+            # Ask numpy for its own include path before attempting anything else
+            try:
+                include_dirs.append(getattr(module, 'get_include')())
+            except AttributeError:
+                pass
+
             include_dirs.append(distutils.sysconfig.get_python_inc(
                                         prefix=os.sep.join(prefix)))
         except ImportError:
@@ -1548,15 +1614,18 @@ class numerix_info(system_info):
             try:
                 import numpy
                 which = "numpy", "defaulted"
-            except ImportError,msg1:
+            except ImportError:
+                msg1 = str(get_exception())
                 try:
                     import Numeric
                     which = "numeric", "defaulted"
-                except ImportError,msg2:
+                except ImportError:
+                    msg2 = str(get_exception())
                     try:
                         import numarray
                         which = "numarray", "defaulted"
-                    except ImportError,msg3:
+                    except ImportError:
+                        msg3 = str(get_exception())
                         log.info(msg1)
                         log.info(msg2)
                         log.info(msg3)
@@ -1657,7 +1726,7 @@ class _pkg_config_info(system_info):
     cflags_flag = '--cflags'
 
     def get_config_exe(self):
-        if os.environ.has_key(self.config_env_var):
+        if self.config_env_var in os.environ:
             return os.environ[self.config_env_var]
         return self.default_config_exe
     def get_config_output(self, config_exe, option):
@@ -1890,7 +1959,7 @@ def dict_append(d,**kws):
         if k=='language':
             languages.append(v)
             continue
-        if d.has_key(k):
+        if k in d:
             if k in ['library_dirs','include_dirs','define_macros']:
                 [d[k].append(vv) for vv in v if vv not in d[k]]
             else:
