@@ -21,7 +21,6 @@ f2py_version = __version__.version
 import sys
 import os
 import pprint
-import shutil
 import types
 import re
 errmess=sys.stderr.write
@@ -31,11 +30,8 @@ show=pprint.pprint
 import crackfortran
 import rules
 import cb_rules
-import common_rules
 import auxfuncs
 import cfuncs
-import capi_maps
-import func2subr
 import f90mod_rules
 
 outmess = auxfuncs.outmess
@@ -69,11 +65,11 @@ Description: This program generates a Python C/API file (<modulename>module.c)
 
 Options:
 
-  --g3-numpy       Use numpy.f2py.lib tool, the 3rd generation of F2PY,
-                   with NumPy support.
   --2d-numpy       Use numpy.f2py tool with NumPy support. [DEFAULT]
   --2d-numeric     Use f2py2e tool with Numeric support.
   --2d-numarray    Use f2py2e tool with Numarray support.
+  --g3-numpy       Use 3rd generation f2py from the separate f2py package.
+                   [NOT AVAILABLE YET]
 
   -h <filename>    Write signatures of the fortran routines to file <filename>
                    and exit. You can then edit <filename> and use it instead
@@ -289,21 +285,21 @@ def callcrackfortran(files,options):
             f.close()
     return postlist
 
-def buildmodules(list):
+def buildmodules(lst):
     cfuncs.buildcfuncs()
     outmess('Building modules...\n')
     modules,mnames,isusedby=[],[],{}
-    for i in range(len(list)):
-        if '__user__' in list[i]['name']:
-            cb_rules.buildcallbacks(list[i])
+    for i in range(len(lst)):
+        if '__user__' in lst[i]['name']:
+            cb_rules.buildcallbacks(lst[i])
         else:
-            if 'use' in list[i]:
-                for u in list[i]['use'].keys():
+            if 'use' in lst[i]:
+                for u in lst[i]['use'].keys():
                     if u not in isusedby:
                         isusedby[u]=[]
-                    isusedby[u].append(list[i]['name'])
-            modules.append(list[i])
-            mnames.append(list[i]['name'])
+                    isusedby[u].append(lst[i]['name'])
+            modules.append(lst[i])
+            mnames.append(lst[i]['name'])
     ret = {}
     for i in range(len(mnames)):
         if mnames[i] in isusedby:
@@ -333,7 +329,11 @@ def run_main(comline_list):
     """Run f2py as if string.join(comline_list,' ') is used as a command line.
     In case of using -h flag, return None.
     """
-    reload(crackfortran)
+    if sys.version_info[0] >= 3:
+        import imp
+        imp.reload(crackfortran)
+    else:
+        reload(crackfortran)
     f2pydir=os.path.dirname(os.path.abspath(cfuncs.__file__))
     fobjhsrc = os.path.join(f2pydir,'src','fortranobject.h')
     fobjcsrc = os.path.join(f2pydir,'src','fortranobject.c')
@@ -395,7 +395,7 @@ def run_compile():
     """
     Do it all in one call!
     """
-    import tempfile,os,shutil
+    import tempfile
 
     i = sys.argv.index('-c')
     del sys.argv[i]
@@ -547,6 +547,7 @@ def run_compile():
     setup(ext_modules = [ext])
 
     if remove_build_dir and os.path.exists(build_dir):
+        import shutil
         outmess('Removing build directory %s\n'%(build_dir))
         shutil.rmtree(build_dir)
 
