@@ -1,6 +1,7 @@
 #include "Python.h"
 #include "structmember.h"
 #include "numpy/noprefix.h"
+#include "npy_config.h"
 
 static intp
 incr_slot_(double x, double *bins, intp lbins)
@@ -116,6 +117,11 @@ arr_bincount(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwds)
             goto fail;
     }
     len = PyArray_SIZE(lst);
+    if (len < 1) {
+        PyErr_SetString(PyExc_ValueError,
+                "The first argument cannot be empty.");
+        goto fail;
+    }
     numbers = (intp *) PyArray_DATA(lst);
     mxi = mxx(numbers, len);
     mni = mnx(numbers, len);
@@ -401,45 +407,39 @@ fail:
     return NULL;
 }
 
-/*
- * binary_search accepts three arguments: a numeric value and
- * a numeric array and its length. It assumes that the array is sorted in
- * increasing order. It returns the index of the array's
- * largest element which is <= the value. It will return -1 if
- * the value is less than the least element of the array.
- * self is not used
+/** @brief Use bisection on a sorted array to find first entry > key.
+ *
+ * Use bisection to find an index i s.t. arr[i] <= key < arr[i + 1]. If there is
+ * no such i the error returns are:
+ *     key < arr[0] -- -1
+ *     key == arr[len - 1] -- len - 1
+ *     key > arr[len - 1] -- len
+ * The array is assumed contiguous and sorted in ascending order.
+ *
+ * @param key key value.
+ * @param arr contiguous sorted array to be searched.
+ * @param len length of the array.
+ * @return index
  */
 static npy_intp
-binary_search(double dval, double dlist [], npy_intp len)
+binary_search(double key, double arr [], npy_intp len)
 {
-    npy_intp bottom , top , middle, result;
+    npy_intp imin = 0;
+    npy_intp imax = len;
 
-    if (dval < dlist [0]) {
-        result = -1;
+    if (key > arr[len - 1]) {
+        return len;
     }
-    else {
-        bottom = 0;
-        top = len - 1;
-        while (bottom < top) {
-            middle = (top + bottom) / 2;
-            if (dlist [middle] < dval) {
-                bottom = middle + 1;
-            }
-            else if (dlist [middle] > dval) {
-                top = middle - 1;
-            }
-            else {
-                return middle;
-            }
-        }
-        if (dlist [bottom] > dval) {
-            result = bottom - 1;
+    while (imin < imax) {
+        npy_intp imid = imin + ((imax - imin) >> 1);
+        if (key >= arr[imid]) {
+            imin = imid + 1;
         }
         else {
-            result = bottom;
+            imax = imid;
         }
     }
-    return result;
+    return imin - 1;
 }
 
 static PyObject *
@@ -472,8 +472,12 @@ arr_interp(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwdict)
     if (ax == NULL) {
         goto fail;
     }
-
     lenxp = axp->dimensions[0];
+    if (lenxp == 0) {
+        PyErr_SetString(PyExc_ValueError,
+                "array of sample points is empty");
+        goto fail;
+    }
     if (afp->dimensions[0] != lenxp) {
         PyErr_SetString(PyExc_ValueError,
                 "fp and xp are not of the same length.");
@@ -511,20 +515,23 @@ arr_interp(PyObject *NPY_UNUSED(self), PyObject *args, PyObject *kwdict)
         }
     }
 
-    slopes = (double *) PyDataMem_NEW((lenxp-1)*sizeof(double));
+    slopes = (double *) PyDataMem_NEW((lenxp - 1)*sizeof(double));
     for (i = 0; i < lenxp - 1; i++) {
         slopes[i] = (dy[i + 1] - dy[i])/(dx[i + 1] - dx[i]);
     }
     for (i = 0; i < lenx; i++) {
         indx = binary_search(dz[i], dx, lenxp);
-        if (indx < 0) {
+        if (indx == -1) {
             dres[i] = lval;
         }
-        else if (indx >= lenxp - 1) {
+        else if (indx == lenxp - 1) {
+            dres[i] = dy[indx];
+        }
+        else if (indx == lenxp) {
             dres[i] = rval;
         }
         else {
-            dres[i] = slopes[indx]*(dz[i]-dx[indx]) + dy[indx];
+            dres[i] = slopes[indx]*(dz[i] - dx[indx]) + dy[indx];
         }
     }
 
@@ -562,11 +569,19 @@ arr_add_docstring(PyObject *NPY_UNUSED(dummy), PyObject *args)
         Py_INCREF(Py_None);
         return Py_None;
     }
+#if defined(NPY_PY3K)
+    if (!PyArg_ParseTuple(args, "OO!", &obj, &PyUnicode_Type, &str)) {
+        return NULL;
+    }
+
+    docstr = PyBytes_AS_STRING(PyUnicode_AsUTF8String(str));
+#else
     if (!PyArg_ParseTuple(args, "OO!", &obj, &PyString_Type, &str)) {
         return NULL;
     }
 
     docstr = PyString_AS_STRING(str);
+#endif
 
 #define _TESTDOC1(typebase) (obj->ob_type == &Py##typebase##_Type)
 #define _TESTDOC2(typebase) (obj->ob_type == Py##typebase##_TypePtr)
@@ -918,12 +933,39 @@ define_types(void)
     return;
 }
 
-/* Initialization function for the module (*must* be called init<name>) */
-PyMODINIT_FUNC init_compiled_base(void) {
-    PyObject *m, *d, *s;
+#if defined(NPY_PY3K)
+static struct PyModuleDef moduledef = {
+        PyModuleDef_HEAD_INIT,
+        "_compiled_base",
+        NULL,
+        -1,
+        methods,
+        NULL,
+        NULL,
+        NULL,
+        NULL
+};
+#endif
 
-    /* Create the module and add the functions */
+#if defined(NPY_PY3K)
+#define RETVAL m
+PyObject *PyInit__compiled_base(void)
+#else
+#define RETVAL
+PyMODINIT_FUNC
+init_compiled_base(void)
+#endif
+{
+    PyObject *m, *d;
+
+#if defined(NPY_PY3K)
+    m = PyModule_Create(&moduledef);
+#else
     m = Py_InitModule("_compiled_base", methods);
+#endif
+    if (!m) {
+        return RETVAL;
+    }
 
     /* Import the array objects */
     import_array();
@@ -931,10 +973,6 @@ PyMODINIT_FUNC init_compiled_base(void) {
     /* Add some symbolic constants to the module */
     d = PyModule_GetDict(m);
 
-    s = PyString_FromString("0.5");
-    PyDict_SetItemString(d, "__version__", s);
-    Py_DECREF(s);
-
     /*
      * PyExc_Exception should catch all the standard errors that are
      * now raised instead of the string exception "numpy.lib.error".
@@ -946,5 +984,5 @@ PyMODINIT_FUNC init_compiled_base(void) {
     /* define PyGetSetDescr_Type and PyMemberDescr_Type */
     define_types();
 
-    return;
+    return RETVAL;
 }
