@@ -1,18 +1,29 @@
-import sys
-
 from numpy.testing import *
-set_package_path()
-import numpy.lib;reload(numpy.lib);reload(numpy.lib.type_check)
 from numpy.lib import *
 from numpy.core import *
-restore_path()
+from numpy.compat import asbytes
 
 def assert_all(x):
     assert(all(x)), x
 
-class test_mintypecode(NumpyTestCase):
 
-    def check_default_1(self):
+class TestCommonType(TestCase):
+    def test_basic(self):
+        ai32 = array([[1,2],[3,4]], dtype=int32)
+        af32 = array([[1,2],[3,4]], dtype=float32)
+        af64 = array([[1,2],[3,4]], dtype=float64)
+        acs = array([[1+5j,2+6j],[3+7j,4+8j]], dtype=csingle)
+        acd = array([[1+5j,2+6j],[3+7j,4+8j]], dtype=cdouble)
+        assert common_type(af32) == float32
+        assert common_type(af64) == float64
+        assert common_type(acs) == csingle
+        assert common_type(acd) == cdouble
+
+
+
+class TestMintypecode(TestCase):
+
+    def test_default_1(self):
         for itype in '1bcsuwil':
             assert_equal(mintypecode(itype),'d')
         assert_equal(mintypecode('f'),'f')
@@ -20,7 +31,7 @@ class test_mintypecode(NumpyTestCase):
         assert_equal(mintypecode('F'),'F')
         assert_equal(mintypecode('D'),'D')
 
-    def check_default_2(self):
+    def test_default_2(self):
         for itype in '1bcsuwil':
             assert_equal(mintypecode(itype+'f'),'f')
             assert_equal(mintypecode(itype+'d'),'d')
@@ -45,7 +56,7 @@ class test_mintypecode(NumpyTestCase):
         assert_equal(mintypecode('DF'),'D')
         assert_equal(mintypecode('DD'),'D')
 
-    def check_default_3(self):
+    def test_default_3(self):
         assert_equal(mintypecode('fdF'),'D')
         #assert_equal(mintypecode('fdF',savespace=1),'F')
         assert_equal(mintypecode('fdD'),'D')
@@ -59,8 +70,10 @@ class test_mintypecode(NumpyTestCase):
         #assert_equal(mintypecode('idF',savespace=1),'F')
         assert_equal(mintypecode('idD'),'D')
 
-class test_isscalar(NumpyTestCase):
-    def check_basic(self):
+
+class TestIsscalar(TestCase):
+
+    def test_basic(self):
         assert(isscalar(3))
         assert(not isscalar([3]))
         assert(not isscalar((3,)))
@@ -68,188 +81,271 @@ class test_isscalar(NumpyTestCase):
         assert(isscalar(10L))
         assert(isscalar(4.0))
 
-class test_real(NumpyTestCase):
-    def check_real(self):
+
+class TestReal(TestCase):
+
+    def test_real(self):
         y = rand(10,)
         assert_array_equal(y,real(y))
 
-    def check_cmplx(self):
+    def test_cmplx(self):
         y = rand(10,)+1j*rand(10,)
         assert_array_equal(y.real,real(y))
 
-class test_imag(NumpyTestCase):
-    def check_real(self):
+
+class TestImag(TestCase):
+
+    def test_real(self):
         y = rand(10,)
         assert_array_equal(0,imag(y))
 
-    def check_cmplx(self):
+    def test_cmplx(self):
         y = rand(10,)+1j*rand(10,)
         assert_array_equal(y.imag,imag(y))
 
-class test_iscomplex(NumpyTestCase):
-    def check_fail(self):
+
+class TestIscomplex(TestCase):
+
+    def test_fail(self):
         z = array([-1,0,1])
         res = iscomplex(z)
         assert(not sometrue(res,axis=0))
-    def check_pass(self):
+    def test_pass(self):
         z = array([-1j,1,0])
         res = iscomplex(z)
         assert_array_equal(res,[1,0,0])
 
-class test_isreal(NumpyTestCase):
-    def check_pass(self):
+
+class TestIsreal(TestCase):
+
+    def test_pass(self):
         z = array([-1,0,1j])
         res = isreal(z)
         assert_array_equal(res,[1,1,0])
-    def check_fail(self):
+    def test_fail(self):
         z = array([-1j,1,0])
         res = isreal(z)
         assert_array_equal(res,[0,1,1])
 
-class test_iscomplexobj(NumpyTestCase):
-    def check_basic(self):
+
+class TestIscomplexobj(TestCase):
+
+    def test_basic(self):
         z = array([-1,0,1])
         assert(not iscomplexobj(z))
         z = array([-1j,0,-1])
         assert(iscomplexobj(z))
 
-class test_isrealobj(NumpyTestCase):
-    def check_basic(self):
+
+
+class TestIsrealobj(TestCase):
+    def test_basic(self):
         z = array([-1,0,1])
         assert(isrealobj(z))
         z = array([-1j,0,-1])
         assert(not isrealobj(z))
 
-class test_isnan(NumpyTestCase):
-    def check_goodvalues(self):
+
+class TestIsnan(TestCase):
+
+    def test_goodvalues(self):
         z = array((-1.,0.,1.))
         res = isnan(z) == 0
         assert_all(alltrue(res,axis=0))
-    def check_posinf(self):
-        olderr = seterr(divide='ignore')
-        assert_all(isnan(array((1.,))/0.) == 0)
-        seterr(**olderr)
-    def check_neginf(self):
-        olderr = seterr(divide='ignore')
-        assert_all(isnan(array((-1.,))/0.) == 0)
-        seterr(**olderr)
-    def check_ind(self):
-        olderr = seterr(divide='ignore', invalid='ignore')
-        assert_all(isnan(array((0.,))/0.) == 1)
-        seterr(**olderr)
-    #def check_qnan(self):             log(-1) return pi*j now
-    #    assert_all(isnan(log(-1.)) == 1)
-    def check_integer(self):
-        assert_all(isnan(1) == 0)
-    def check_complex(self):
-        assert_all(isnan(1+1j) == 0)
-    def check_complex1(self):
-        olderr = seterr(divide='ignore', invalid='ignore')
-        assert_all(isnan(array(0+0j)/0.) == 1)
-        seterr(**olderr)
 
-class test_isfinite(NumpyTestCase):
-    def check_goodvalues(self):
+    def test_posinf(self):
+        olderr = seterr(divide='ignore')
+        try:
+            assert_all(isnan(array((1.,))/0.) == 0)
+        finally:
+            seterr(**olderr)
+
+    def test_neginf(self):
+        olderr = seterr(divide='ignore')
+        try:
+            assert_all(isnan(array((-1.,))/0.) == 0)
+        finally:
+            seterr(**olderr)
+
+    def test_ind(self):
+        olderr = seterr(divide='ignore', invalid='ignore')
+        try:
+            assert_all(isnan(array((0.,))/0.) == 1)
+        finally:
+            seterr(**olderr)
+
+    #def test_qnan(self):             log(-1) return pi*j now
+    #    assert_all(isnan(log(-1.)) == 1)
+
+    def test_integer(self):
+        assert_all(isnan(1) == 0)
+
+    def test_complex(self):
+        assert_all(isnan(1+1j) == 0)
+
+    def test_complex1(self):
+        olderr = seterr(divide='ignore', invalid='ignore')
+        try:
+            assert_all(isnan(array(0+0j)/0.) == 1)
+        finally:
+            seterr(**olderr)
+
+
+class TestIsfinite(TestCase):
+
+    def test_goodvalues(self):
         z = array((-1.,0.,1.))
         res = isfinite(z) == 1
         assert_all(alltrue(res,axis=0))
-    def check_posinf(self):
-        olderr = seterr(divide='ignore')
-        assert_all(isfinite(array((1.,))/0.) == 0)
-        seterr(**olderr)
-    def check_neginf(self):
-        olderr = seterr(divide='ignore')
-        assert_all(isfinite(array((-1.,))/0.) == 0)
-        seterr(**olderr)
-    def check_ind(self):
-        olderr = seterr(divide='ignore', invalid='ignore')
-        assert_all(isfinite(array((0.,))/0.) == 0)
-        seterr(**olderr)
-    #def check_qnan(self):
-    #    assert_all(isfinite(log(-1.)) == 0)
-    def check_integer(self):
-        assert_all(isfinite(1) == 1)
-    def check_complex(self):
-        assert_all(isfinite(1+1j) == 1)
-    def check_complex1(self):
-        olderr = seterr(divide='ignore', invalid='ignore')
-        assert_all(isfinite(array(1+1j)/0.) == 0)
-        seterr(**olderr)
 
-class test_isinf(NumpyTestCase):
-    def check_goodvalues(self):
+    def test_posinf(self):
+        olderr = seterr(divide='ignore', invalid='ignore')
+        try:
+            assert_all(isfinite(array((1.,))/0.) == 0)
+        finally:
+            seterr(**olderr)
+
+    def test_neginf(self):
+        olderr = seterr(divide='ignore', invalid='ignore')
+        try:
+            assert_all(isfinite(array((-1.,))/0.) == 0)
+        finally:
+            seterr(**olderr)
+
+    def test_ind(self):
+        olderr = seterr(divide='ignore', invalid='ignore')
+        try:
+            assert_all(isfinite(array((0.,))/0.) == 0)
+        finally:
+            seterr(**olderr)
+
+    #def test_qnan(self):
+    #    assert_all(isfinite(log(-1.)) == 0)
+
+    def test_integer(self):
+        assert_all(isfinite(1) == 1)
+
+    def test_complex(self):
+        assert_all(isfinite(1+1j) == 1)
+
+    def test_complex1(self):
+        olderr = seterr(divide='ignore', invalid='ignore')
+        try:
+            assert_all(isfinite(array(1+1j)/0.) == 0)
+        finally:
+            seterr(**olderr)
+
+
+class TestIsinf(TestCase):
+
+    def test_goodvalues(self):
         z = array((-1.,0.,1.))
         res = isinf(z) == 0
         assert_all(alltrue(res,axis=0))
-    def check_posinf(self):
-        olderr = seterr(divide='ignore')
-        assert_all(isinf(array((1.,))/0.) == 1)
-        seterr(**olderr)
-    def check_posinf_scalar(self):
-        olderr = seterr(divide='ignore')
-        assert_all(isinf(array(1.,)/0.) == 1)
-        seterr(**olderr)
-    def check_neginf(self):
-        olderr = seterr(divide='ignore')
-        assert_all(isinf(array((-1.,))/0.) == 1)
-        seterr(**olderr)
-    def check_neginf_scalar(self):
-        olderr = seterr(divide='ignore')
-        assert_all(isinf(array(-1.)/0.) == 1)
-        seterr(**olderr)
-    def check_ind(self):
+
+    def test_posinf(self):
         olderr = seterr(divide='ignore', invalid='ignore')
-        assert_all(isinf(array((0.,))/0.) == 0)
-        seterr(**olderr)
-    #def check_qnan(self):
+        try:
+            assert_all(isinf(array((1.,))/0.) == 1)
+        finally:
+            seterr(**olderr)
+
+    def test_posinf_scalar(self):
+        olderr = seterr(divide='ignore', invalid='ignore')
+        try:
+            assert_all(isinf(array(1.,)/0.) == 1)
+        finally:
+            seterr(**olderr)
+
+    def test_neginf(self):
+        olderr = seterr(divide='ignore', invalid='ignore')
+        try:
+            assert_all(isinf(array((-1.,))/0.) == 1)
+        finally:
+            seterr(**olderr)
+
+    def test_neginf_scalar(self):
+        olderr = seterr(divide='ignore', invalid='ignore')
+        try:
+            assert_all(isinf(array(-1.)/0.) == 1)
+        finally:
+            seterr(**olderr)
+
+    def test_ind(self):
+        olderr = seterr(divide='ignore', invalid='ignore')
+        try:
+            assert_all(isinf(array((0.,))/0.) == 0)
+        finally:
+            seterr(**olderr)
+
+    #def test_qnan(self):
     #    assert_all(isinf(log(-1.)) == 0)
     #    assert_all(isnan(log(-1.)) == 1)
 
-class test_isposinf(NumpyTestCase):
-    def check_generic(self):
+
+class TestIsposinf(TestCase):
+
+    def test_generic(self):
         olderr = seterr(divide='ignore', invalid='ignore')
-        vals = isposinf(array((-1.,0,1))/0.)
-        seterr(**olderr)
+        try:
+            vals = isposinf(array((-1.,0,1))/0.)
+        finally:
+            seterr(**olderr)
         assert(vals[0] == 0)
         assert(vals[1] == 0)
         assert(vals[2] == 1)
 
-class test_isneginf(NumpyTestCase):
-    def check_generic(self):
+
+class TestIsneginf(TestCase):
+    def test_generic(self):
         olderr = seterr(divide='ignore', invalid='ignore')
-        vals = isneginf(array((-1.,0,1))/0.)
-        seterr(**olderr)
+        try:
+            vals = isneginf(array((-1.,0,1))/0.)
+        finally:
+            seterr(**olderr)
         assert(vals[0] == 1)
         assert(vals[1] == 0)
         assert(vals[2] == 0)
 
-class test_nan_to_num(NumpyTestCase):
-    def check_generic(self):
+
+class TestNanToNum(TestCase):
+
+    def test_generic(self):
         olderr = seterr(divide='ignore', invalid='ignore')
-        vals = nan_to_num(array((-1.,0,1))/0.)
-        seterr(**olderr)
+        try:
+            vals = nan_to_num(array((-1.,0,1))/0.)
+        finally:
+            seterr(**olderr)
         assert_all(vals[0] < -1e10) and assert_all(isfinite(vals[0]))
         assert(vals[1] == 0)
         assert_all(vals[2] > 1e10) and assert_all(isfinite(vals[2]))
-    def check_integer(self):
+
+    def test_integer(self):
         vals = nan_to_num(1)
         assert_all(vals == 1)
-    def check_complex_good(self):
+
+    def test_complex_good(self):
         vals = nan_to_num(1+1j)
         assert_all(vals == 1+1j)
-    def check_complex_bad(self):
+
+    def test_complex_bad(self):
         v = 1+1j
         olderr = seterr(divide='ignore', invalid='ignore')
-        v += array(0+1.j)/0.
-        seterr(**olderr)
+        try:
+            v += array(0+1.j)/0.
+        finally:
+            seterr(**olderr)
         vals = nan_to_num(v)
         # !! This is actually (unexpectedly) zero
         assert_all(isfinite(vals))
-    def check_complex_bad2(self):
+
+    def test_complex_bad2(self):
         v = 1+1j
         olderr = seterr(divide='ignore', invalid='ignore')
-        v += array(-1+1.j)/0.
-        seterr(**olderr)
+        try:
+            v += array(-1+1.j)/0.
+        finally:
+            seterr(**olderr)
         vals = nan_to_num(v)
         assert_all(isfinite(vals))
         #assert_all(vals.imag > 1e10)  and assert_all(isfinite(vals))
@@ -259,8 +355,9 @@ class test_nan_to_num(NumpyTestCase):
         #assert_all(vals.real < -1e10) and assert_all(isfinite(vals))
 
 
-class test_real_if_close(NumpyTestCase):
-    def check_basic(self):
+class TestRealIfClose(TestCase):
+
+    def test_basic(self):
         a = rand(10)
         b = real_if_close(a+1e-15j)
         assert_all(isrealobj(b))
@@ -270,5 +367,14 @@ class test_real_if_close(NumpyTestCase):
         b = real_if_close(a+1e-7j,tol=1e-6)
         assert_all(isrealobj(b))
 
+
+class TestArrayConversion(TestCase):
+
+    def test_asfarray(self):
+        a = asfarray(array([1,2,3]))
+        assert_equal(a.__class__,ndarray)
+        assert issubdtype(a.dtype,float)
+
+
 if __name__ == "__main__":
-    NumpyTest().run()
+    run_module_suite()
