@@ -6,7 +6,7 @@ from numpy.testing import *
 from numpy import array, single, double, csingle, cdouble, dot, identity
 from numpy import multiply, atleast_2d, inf, asarray, matrix
 from numpy import linalg
-from numpy.linalg import matrix_power, norm
+from numpy.linalg import matrix_power, norm, matrix_rank
 
 def ifthen(a, b):
     return not a or b
@@ -33,6 +33,11 @@ class LinalgTestCase:
         b = array([2., 1.], dtype=double)
         self.do(a, b)
 
+    def test_double_2(self):
+        a = array([[1.,2.], [3.,4.]], dtype=double)
+        b = array([[2., 1., 4.], [3., 4., 6.]], dtype=double)
+        self.do(a, b)
+
     def test_csingle(self):
         a = array([[1.+2j,2+3j], [3+4j,4+5j]], dtype=csingle)
         b = array([2.+1j, 1.+2j], dtype=csingle)
@@ -43,6 +48,11 @@ class LinalgTestCase:
         b = array([2.+1j, 1.+2j], dtype=cdouble)
         self.do(a, b)
 
+    def test_cdouble_2(self):
+        a = array([[1.+2j,2+3j], [3+4j,4+5j]], dtype=cdouble)
+        b = array([[2.+1j, 1.+2j, 1+3j], [1-2j, 1-3j, 1-6j]], dtype=cdouble)
+        self.do(a, b)
+
     def test_empty(self):
         a = atleast_2d(array([], dtype = double))
         b = atleast_2d(array([], dtype = double))
@@ -70,6 +80,58 @@ class LinalgTestCase:
         self.do(a, b)
 
 
+class LinalgNonsquareTestCase:
+    def test_single_nsq_1(self):
+        a = array([[1.,2.,3.], [3.,4.,6.]], dtype=single)
+        b = array([2., 1.], dtype=single)
+        self.do(a, b)
+
+    def test_single_nsq_2(self):
+        a = array([[1.,2.], [3.,4.], [5.,6.]], dtype=single)
+        b = array([2., 1., 3.], dtype=single)
+        self.do(a, b)
+
+    def test_double_nsq_1(self):
+        a = array([[1.,2.,3.], [3.,4.,6.]], dtype=double)
+        b = array([2., 1.], dtype=double)
+        self.do(a, b)
+
+    def test_double_nsq_2(self):
+        a = array([[1.,2.], [3.,4.], [5.,6.]], dtype=double)
+        b = array([2., 1., 3.], dtype=double)
+        self.do(a, b)
+
+    def test_csingle_nsq_1(self):
+        a = array([[1.+1j,2.+2j,3.-3j], [3.-5j,4.+9j,6.+2j]], dtype=csingle)
+        b = array([2.+1j, 1.+2j], dtype=csingle)
+        self.do(a, b)
+
+    def test_csingle_nsq_2(self):
+        a = array([[1.+1j,2.+2j], [3.-3j,4.-9j], [5.-4j,6.+8j]], dtype=csingle)
+        b = array([2.+1j, 1.+2j, 3.-3j], dtype=csingle)
+        self.do(a, b)
+
+    def test_cdouble_nsq_1(self):
+        a = array([[1.+1j,2.+2j,3.-3j], [3.-5j,4.+9j,6.+2j]], dtype=cdouble)
+        b = array([2.+1j, 1.+2j], dtype=cdouble)
+        self.do(a, b)
+
+    def test_cdouble_nsq_2(self):
+        a = array([[1.+1j,2.+2j], [3.-3j,4.-9j], [5.-4j,6.+8j]], dtype=cdouble)
+        b = array([2.+1j, 1.+2j, 3.-3j], dtype=cdouble)
+        self.do(a, b)
+
+    def test_cdouble_nsq_1_2(self):
+        a = array([[1.+1j,2.+2j,3.-3j], [3.-5j,4.+9j,6.+2j]], dtype=cdouble)
+        b = array([[2.+1j, 1.+2j], [1-1j, 2-2j]], dtype=cdouble)
+        self.do(a, b)
+
+    def test_cdouble_nsq_2_2(self):
+        a = array([[1.+1j,2.+2j], [3.-3j,4.-9j], [5.-4j,6.+8j]], dtype=cdouble)
+        b = array([[2.+1j, 1.+2j], [1-1j, 2-2j], [1-1j, 2-2j]], dtype=cdouble)
+        self.do(a, b)
+
+
 class TestSolve(LinalgTestCase, TestCase):
     def do(self, a, b):
         x = linalg.solve(a, b)
@@ -127,20 +189,54 @@ class TestPinv(LinalgTestCase, TestCase):
 class TestDet(LinalgTestCase, TestCase):
     def do(self, a, b):
         d = linalg.det(a)
+        (s, ld) = linalg.slogdet(a)
         if asarray(a).dtype.type in (single, double):
             ad = asarray(a).astype(double)
         else:
             ad = asarray(a).astype(cdouble)
         ev = linalg.eigvals(ad)
         assert_almost_equal(d, multiply.reduce(ev))
+        assert_almost_equal(s * np.exp(ld), multiply.reduce(ev))
+        if s != 0:
+            assert_almost_equal(np.abs(s), 1)
+        else:
+            assert_equal(ld, -inf)
 
-class TestLstsq(LinalgTestCase, TestCase):
+    def test_zero(self):
+        assert_equal(linalg.det([[0.0]]), 0.0)
+        assert_equal(type(linalg.det([[0.0]])), double)
+        assert_equal(linalg.det([[0.0j]]), 0.0)
+        assert_equal(type(linalg.det([[0.0j]])), cdouble)
+
+        assert_equal(linalg.slogdet([[0.0]]), (0.0, -inf))
+        assert_equal(type(linalg.slogdet([[0.0]])[0]), double)
+        assert_equal(type(linalg.slogdet([[0.0]])[1]), double)
+        assert_equal(linalg.slogdet([[0.0j]]), (0.0j, -inf))
+        assert_equal(type(linalg.slogdet([[0.0j]])[0]), cdouble)
+        assert_equal(type(linalg.slogdet([[0.0j]])[1]), double)
+
+class TestLstsq(LinalgTestCase, LinalgNonsquareTestCase, TestCase):
     def do(self, a, b):
+        arr = np.asarray(a)
+        m, n = arr.shape
         u, s, vt = linalg.svd(a, 0)
         x, residuals, rank, sv = linalg.lstsq(a, b)
-        assert_almost_equal(b, dot(a, x))
-        assert_equal(rank, asarray(a).shape[0])
+        if m <= n:
+            assert_almost_equal(b, dot(a, x))
+            assert_equal(rank, m)
+        else:
+            assert_equal(rank, n)
         assert_almost_equal(sv, sv.__array_wrap__(s))
+        if rank == n and m > n:
+            expect_resids = (np.asarray(abs(np.dot(a, x) - b))**2).sum(axis=0)
+            expect_resids = np.asarray(expect_resids)
+            if len(np.asarray(b).shape) == 1:
+                expect_resids.shape = (1,)
+                assert_equal(residuals.shape, expect_resids.shape)
+        else:
+            expect_resids = type(x)([])
+        assert_almost_equal(residuals, expect_resids)
+        assert_(np.issubdtype(residuals.dtype, np.floating))
         assert imply(isinstance(b, matrix), isinstance(x, matrix))
         assert imply(isinstance(b, matrix), isinstance(residuals, matrix))
 
@@ -315,5 +411,25 @@ class TestNormSingle(_TestNorm):
     dt = np.float32
     dec = 6
 
+
+def test_matrix_rank():
+    # Full rank matrix
+    yield assert_equal, 4, matrix_rank(np.eye(4))
+    # rank deficient matrix
+    I=np.eye(4); I[-1,-1] = 0. 
+    yield assert_equal, matrix_rank(I), 3
+    # All zeros - zero rank
+    yield assert_equal, matrix_rank(np.zeros((4,4))), 0
+    # 1 dimension - rank 1 unless all 0
+    yield assert_equal, matrix_rank([1, 0, 0, 0]), 1
+    yield assert_equal, matrix_rank(np.zeros((4,))), 0
+    # accepts array-like
+    yield assert_equal, matrix_rank([1]), 1
+    # greater than 2 dimensions raises error
+    yield assert_raises, TypeError, matrix_rank, np.zeros((2,2,2))
+    # works on scalar
+    yield assert_equal, matrix_rank(1), 1
+    
+    
 if __name__ == "__main__":
     run_module_suite()
