@@ -1,50 +1,60 @@
-"""Functions for dealing with Chebyshev series.
+"""
+Objects for dealing with Chebyshev series.
 
-This module provide s a number of functions that are useful in dealing with
-Chebyshev series as well as a ``Chebyshev`` class that encapsuletes the usual
-arithmetic operations. All the Chebyshev series are assumed to be ordered
-from low to high, thus ``array([1,2,3])`` will be treated as the series
-``T_0 + 2*T_1 + 3*T_2``
+This module provides a number of objects (mostly functions) useful for
+dealing with Chebyshev series, including a `Chebyshev` class that
+encapsulates the usual arithmetic operations.  (General information
+on how this module represents and works with such polynomials is in the
+docstring for its "parent" sub-package, `numpy.polynomial`).
 
 Constants
 ---------
-- chebdomain -- Chebyshev series default domain
-- chebzero -- Chebyshev series that evaluates to 0.
-- chebone -- Chebyshev series that evaluates to 1.
-- chebx -- Chebyshev series of the identity map (x).
+- `chebdomain` -- Chebyshev series default domain, [-1,1].
+- `chebzero` -- (Coefficients of the) Chebyshev series that evaluates
+  identically to 0.
+- `chebone` -- (Coefficients of the) Chebyshev series that evaluates
+  identically to 1.
+- `chebx` -- (Coefficients of the) Chebyshev series for the identity map,
+  ``f(x) = x``.
 
 Arithmetic
 ----------
-- chebadd -- add a Chebyshev series to another.
-- chebsub -- subtract a Chebyshev series from another.
-- chebmul -- multiply a Chebyshev series by another
-- chebdiv -- divide one Chebyshev series by another.
-- chebval -- evaluate a Chebyshev series at given points.
+- `chebadd` -- add two Chebyshev series.
+- `chebsub` -- subtract one Chebyshev series from another.
+- `chebmul` -- multiply two Chebyshev series.
+- `chebdiv` -- divide one Chebyshev series by another.
+- `chebval` -- evaluate a Chebyshev series at given points.
 
 Calculus
 --------
-- chebder -- differentiate a Chebyshev series.
-- chebint -- integrate a Chebyshev series.
+- `chebder` -- differentiate a Chebyshev series.
+- `chebint` -- integrate a Chebyshev series.
 
 Misc Functions
 --------------
-- chebfromroots -- create a Chebyshev series with specified roots.
-- chebroots -- find the roots of a Chebyshev series.
-- chebvander -- Vandermode like matrix for Chebyshev polynomials.
-- chebfit -- least squares fit returning a Chebyshev series.
-- chebtrim -- trim leading coefficients from a Chebyshev series.
-- chebline -- Chebyshev series of given straight line
-- cheb2poly -- convert a Chebyshev series to a polynomial.
-- poly2cheb -- convert a polynomial to a Chebyshev series.
+- `chebfromroots` -- create a Chebyshev series with specified roots.
+- `chebroots` -- find the roots of a Chebyshev series.
+- `chebvander` -- Vandermonde-like matrix for Chebyshev polynomials.
+- `chebfit` -- least-squares fit returning a Chebyshev series.
+- `chebpts1` -- Chebyshev points of the first kind.
+- `chebpts2` -- Chebyshev points of the second kind.
+- `chebtrim` -- trim leading coefficients from a Chebyshev series.
+- `chebline` -- Chebyshev series of given straight line.
+- `cheb2poly` -- convert a Chebyshev series to a polynomial.
+- `poly2cheb` -- convert a polynomial to a Chebyshev series.
 
 Classes
 -------
-- Chebyshev -- Chebyshev series class.
+- `Chebyshev` -- A Chebyshev series class.
+
+See also
+--------
+`numpy.polynomial`
 
 Notes
 -----
 The implementations of multiplication, division, integration, and
-differentiation use the algebraic identities:
+differentiation use the algebraic identities [1]_:
 
 .. math ::
     T_n(x) = \\frac{z^n + z^{-n}}{2} \\\\
@@ -55,22 +65,29 @@ where
 .. math :: x = \\frac{z + z^{-1}}{2}.
 
 These identities allow a Chebyshev series to be expressed as a finite,
-symmetric Laurent series. These sorts of Laurent series are referred to as
-z-series in this module.
+symmetric Laurent series.  In this module, this sort of Laurent series
+is referred to as a "z-series."
+
+References
+----------
+.. [1] A. T. Benjamin, et al., "Combinatorial Trigonometry with Chebyshev
+  Polynomials," *Journal of Statistical Planning and Inference 14*, 2008
+  (preprint: http://www.math.hmc.edu/~benjamin/papers/CombTrig.pdf, pg. 4)
 
 """
 from __future__ import division
 
 __all__ = ['chebzero', 'chebone', 'chebx', 'chebdomain', 'chebline',
-        'chebadd', 'chebsub', 'chebmul', 'chebdiv', 'chebval', 'chebder',
-        'chebint', 'cheb2poly', 'poly2cheb', 'chebfromroots', 'chebvander',
-        'chebfit', 'chebtrim', 'chebroots', 'Chebyshev']
+        'chebadd', 'chebsub', 'chebmulx', 'chebmul', 'chebdiv', 'chebval',
+        'chebder', 'chebint', 'cheb2poly', 'poly2cheb', 'chebfromroots',
+        'chebvander', 'chebfit', 'chebtrim', 'chebroots', 'chebpts1',
+        'chebpts2', 'Chebyshev']
 
 import numpy as np
 import numpy.linalg as la
 import polyutils as pu
+import warnings
 from polytemplate import polytemplate
-from polyutils import RankWarning, PolyError, PolyDomainError
 
 chebtrim = pu.trimcoef
 
@@ -289,20 +306,22 @@ def _zseries_int(zs) :
 
 
 def poly2cheb(pol) :
-    """Convert a polynomial to a Chebyshev series.
+    """
+    Convert a polynomial to a Chebyshev series.
 
-    Convert a series containing polynomial coefficients ordered by degree
-    from low to high to an equivalent Chebyshev series ordered from low to
-    high.
+    Convert an array representing the coefficients of a polynomial (relative
+    to the "standard" basis) ordered from lowest degree to highest, to an
+    array of the coefficients of the equivalent Chebyshev series, ordered
+    from lowest to highest degree.
 
-    Inputs
-    ------
+    Parameters
+    ----------
     pol : array_like
-        1-d array containing the polynomial coeffients
+        1-d array containing the polynomial coefficients
 
     Returns
     -------
-    cseries : ndarray
+    cs : ndarray
         1-d array containing the coefficients of the equivalent Chebyshev
         series.
 
@@ -310,50 +329,91 @@ def poly2cheb(pol) :
     --------
     cheb2poly
 
+    Notes
+    -----
+    The easy way to do conversions between polynomial basis sets
+    is to use the convert method of a class instance.
+
+    Examples
+    --------
+    >>> from numpy import polynomial as P
+    >>> p = P.Polynomial(range(4))
+    >>> p
+    Polynomial([ 0.,  1.,  2.,  3.], [-1.,  1.])
+    >>> c = p.convert(kind=P.Chebyshev)
+    >>> c
+    Chebyshev([ 1.  ,  3.25,  1.  ,  0.75], [-1.,  1.])
+    >>> P.poly2cheb(range(4))
+    array([ 1.  ,  3.25,  1.  ,  0.75])
+
     """
     [pol] = pu.as_series([pol])
-    pol = pol[::-1]
-    zs = pol[:1].copy()
-    x = np.array([.5, 0, .5], dtype=pol.dtype)
-    for i in range(1, len(pol)) :
-        zs = _zseries_mul(zs, x)
-        zs[i] += pol[i]
-    return _zseries_to_cseries(zs)
+    deg = len(pol) - 1
+    res = 0
+    for i in range(deg, -1, -1) :
+        res = chebadd(chebmulx(res), pol[i])
+    return res
 
 
 def cheb2poly(cs) :
-    """Convert a Chebyshev series to a polynomial.
+    """
+    Convert a Chebyshev series to a polynomial.
 
-    Covert a series containing Chebyshev series coefficients orderd from
-    low to high to an equivalent polynomial ordered from low to
-    high by degree.
+    Convert an array representing the coefficients of a Chebyshev series,
+    ordered from lowest degree to highest, to an array of the coefficients
+    of the equivalent polynomial (relative to the "standard" basis) ordered
+    from lowest to highest degree.
 
-    Inputs
-    ------
+    Parameters
+    ----------
     cs : array_like
-        1-d array containing the Chebyshev series coeffients ordered from
-        low to high.
+        1-d array containing the Chebyshev series coefficients, ordered
+        from lowest order term to highest.
 
     Returns
     -------
     pol : ndarray
         1-d array containing the coefficients of the equivalent polynomial
-        ordered from low to high by degree.
+        (relative to the "standard" basis) ordered from lowest order term
+        to highest.
 
     See Also
     --------
     poly2cheb
 
+    Notes
+    -----
+    The easy way to do conversions between polynomial basis sets
+    is to use the convert method of a class instance.
+
+    Examples
+    --------
+    >>> from numpy import polynomial as P
+    >>> c = P.Chebyshev(range(4))
+    >>> c
+    Chebyshev([ 0.,  1.,  2.,  3.], [-1.,  1.])
+    >>> p = c.convert(kind=P.Polynomial)
+    >>> p
+    Polynomial([ -2.,  -8.,   4.,  12.], [-1.,  1.])
+    >>> P.cheb2poly(range(4))
+    array([ -2.,  -8.,   4.,  12.])
+
     """
+    from polynomial import polyadd, polysub, polymulx
+
     [cs] = pu.as_series([cs])
-    pol = np.zeros(len(cs), dtype=cs.dtype)
-    quo = _cseries_to_zseries(cs)
-    x = np.array([.5, 0, .5], dtype=pol.dtype)
-    for i in range(0, len(cs) - 1) :
-        quo, rem = _zseries_div(quo, x)
-        pol[i] = rem[0]
-    pol[-1] = quo[0]
-    return pol
+    n = len(cs)
+    if n < 3:
+        return cs
+    else:
+        c0 = cs[-2]
+        c1 = cs[-1]
+        for i in range(n - 3, -1, -1) :
+            tmp = c0
+            c0 = polysub(cs[i], c1)
+            c1 = polyadd(tmp, polymulx(c1)*2)
+        return polyadd(c0, polymulx(c1))
+
 
 #
 # These are constant arrays are of integer type so as to be compatible
@@ -373,19 +433,33 @@ chebone = np.array([1])
 chebx = np.array([0,1])
 
 def chebline(off, scl) :
-    """Chebyshev series whose graph is a straight line
+    """
+    Chebyshev series whose graph is a straight line.
 
-    The line has the formula ``off + scl*x``
 
-    Parameters:
-    -----------
+
+    Parameters
+    ----------
     off, scl : scalars
         The specified line is given by ``off + scl*x``.
 
-    Returns:
+    Returns
+    -------
+    y : ndarray
+        This module's representation of the Chebyshev series for
+        ``off + scl*x``.
+
+    See Also
     --------
-    series : 1d ndarray
-        The Chebyshev series representation of ``off + scl*x``.
+    polyline
+
+    Examples
+    --------
+    >>> import numpy.polynomial.chebyshev as C
+    >>> C.chebline(3,2)
+    array([3, 2])
+    >>> C.chebval(-3, C.chebline(3,2)) # should be -3
+    -3.0
 
     """
     if scl != 0 :
@@ -394,25 +468,55 @@ def chebline(off, scl) :
         return np.array([off])
 
 def chebfromroots(roots) :
-    """Generate a Chebyschev series with given roots.
+    """
+    Generate a Chebyshev series with the given roots.
 
-    Generate a Chebyshev series whose roots are given by `roots`. The
-    resulting series is the produet `(x - roots[0])*(x - roots[1])*...`
+    Return the array of coefficients for the C-series whose roots (a.k.a.
+    "zeros") are given by *roots*.  The returned array of coefficients is
+    ordered from lowest order "term" to highest, and zeros of multiplicity
+    greater than one must be included in *roots* a number of times equal
+    to their multiplicity (e.g., if `2` is a root of multiplicity three,
+    then [2,2,2] must be in *roots*).
 
-    Inputs
-    ------
+    Parameters
+    ----------
     roots : array_like
-        1-d array containing the roots in sorted order.
+        Sequence containing the roots.
 
     Returns
     -------
-    series : ndarray
-        1-d array containing the coefficients of the Chebeshev series
-        ordered from low to high.
+    out : ndarray
+        1-d array of the C-series' coefficients, ordered from low to
+        high.  If all roots are real, ``out.dtype`` is a float type;
+        otherwise, ``out.dtype`` is a complex type, even if all the
+        coefficients in the result are real (see Examples below).
 
     See Also
     --------
-    chebroots
+    polyfromroots
+
+    Notes
+    -----
+    What is returned are the :math:`c_i` such that:
+
+    .. math::
+
+        \\sum_{i=0}^{n} c_i*T_i(x) = \\prod_{i=0}^{n} (x - roots[i])
+
+    where ``n == len(roots)`` and :math:`T_i(x)` is the `i`-th Chebyshev
+    (basis) polynomial over the domain `[-1,1]`.  Note that, unlike
+    `polyfromroots`, due to the nature of the C-series basis set, the
+    above identity *does not* imply :math:`c_n = 1` identically (see
+    Examples).
+
+    Examples
+    --------
+    >>> import numpy.polynomial.chebyshev as C
+    >>> C.chebfromroots((-1,0,1)) # x^3 - x relative to the standard basis
+    array([ 0.  , -0.25,  0.  ,  0.25])
+    >>> j = complex(0,1)
+    >>> C.chebfromroots((-j,j)) # x^2 + 1 relative to the standard basis
+    array([ 1.5+0.j,  0.0+0.j,  0.5+0.j])
 
     """
     if len(roots) == 0 :
@@ -420,34 +524,49 @@ def chebfromroots(roots) :
     else :
         [roots] = pu.as_series([roots], trim=False)
         prd = np.array([1], dtype=roots.dtype)
-        for r in roots :
-            fac = np.array([.5, -r, .5], dtype=roots.dtype)
-            prd = _zseries_mul(fac, prd)
-        return _zseries_to_cseries(prd)
+        for r in roots:
+            prd = chebsub(chebmulx(prd), r*prd)
+        return prd
 
 
 def chebadd(c1, c2):
-    """Add one Chebyshev series to another.
+    """
+    Add one Chebyshev series to another.
 
-    Returns the sum of two Chebyshev series `c1` + `c2`. The arguments are
-    sequences of coefficients ordered from low to high, i.e., [1,2,3] is
-    the series "T_0 + 2*T_1 + 3*T_2".
+    Returns the sum of two Chebyshev series `c1` + `c2`.  The arguments
+    are sequences of coefficients ordered from lowest order term to
+    highest, i.e., [1,2,3] represents the series ``T_0 + 2*T_1 + 3*T_2``.
 
     Parameters
     ----------
     c1, c2 : array_like
-        1d arrays of Chebyshev series coefficients ordered from low to
+        1-d arrays of Chebyshev series coefficients ordered from low to
         high.
 
     Returns
     -------
     out : ndarray
-        Chebyshev series of the sum.
+        Array representing the Chebyshev series of their sum.
 
     See Also
     --------
     chebsub, chebmul, chebdiv, chebpow
 
+    Notes
+    -----
+    Unlike multiplication, division, etc., the sum of two Chebyshev series
+    is a Chebyshev series (without having to "reproject" the result onto
+    the basis set) so addition, just like that of "standard" polynomials,
+    is simply "component-wise."
+
+    Examples
+    --------
+    >>> from numpy.polynomial import chebyshev as C
+    >>> c1 = (1,2,3)
+    >>> c2 = (3,2,1)
+    >>> C.chebadd(c1,c2)
+    array([ 4.,  4.,  4.])
+
     """
     # c1, c2 are trimmed copies
     [c1, c2] = pu.as_series([c1, c2])
@@ -461,29 +580,44 @@ def chebadd(c1, c2):
 
 
 def chebsub(c1, c2):
-    """Subtract one Chebyshev series from another.
+    """
+    Subtract one Chebyshev series from another.
 
-    Returns the difference of two Chebyshev series `c1` - `c2`. The
-    sequences of coefficients are ordered from low to high, i.e., [1,2,3]
-    is the series ``T_0 + 2*T_1 + 3*T_2.``
+    Returns the difference of two Chebyshev series `c1` - `c2`.  The
+    sequences of coefficients are from lowest order term to highest, i.e.,
+    [1,2,3] represents the series ``T_0 + 2*T_1 + 3*T_2``.
 
     Parameters
     ----------
     c1, c2 : array_like
-        1d arrays of Chebyshev series coefficients ordered from low to
+        1-d arrays of Chebyshev series coefficients ordered from low to
         high.
 
     Returns
     -------
     out : ndarray
-        Chebyshev series of the difference.
+        Of Chebyshev series coefficients representing their difference.
 
     See Also
     --------
     chebadd, chebmul, chebdiv, chebpow
 
+    Notes
+    -----
+    Unlike multiplication, division, etc., the difference of two Chebyshev
+    series is a Chebyshev series (without having to "reproject" the result
+    onto the basis set) so subtraction, just like that of "standard"
+    polynomials, is simply "component-wise."
+
     Examples
     --------
+    >>> from numpy.polynomial import chebyshev as C
+    >>> c1 = (1,2,3)
+    >>> c2 = (3,2,1)
+    >>> C.chebsub(c1,c2)
+    array([-2.,  0.,  2.])
+    >>> C.chebsub(c2,c1) # -C.chebsub(c1,c2)
+    array([ 2.,  0., -2.])
 
     """
     # c1, c2 are trimmed copies
@@ -498,28 +632,84 @@ def chebsub(c1, c2):
     return pu.trimseq(ret)
 
 
-def chebmul(c1, c2):
-    """Multiply one Chebyshev series by another.
+def chebmulx(cs):
+    """Multiply a Chebyshev series by x.
+
+    Multiply the polynomial `cs` by x, where x is the independent
+    variable.
 
-    Returns the product of two Chebyshev series `c1` * `c2`. The arguments
-    are sequences of coefficients ordered from low to high, i.e., [1,2,3]
-    is the series ``T_0 + 2*T_1 + 3*T_2.``
 
     Parameters
     ----------
-    c1, c2 : array_like
-        1d arrays of chebyshev series coefficients ordered from low to
+    cs : array_like
+        1-d array of Chebyshev series coefficients ordered from low to
         high.
 
     Returns
     -------
     out : ndarray
-        Chebyshev series of the product.
+        Array representing the result of the multiplication.
+
+    Notes
+    -----
+    .. versionadded:: 1.5.0
+
+    """
+    # cs is a trimmed copy
+    [cs] = pu.as_series([cs])
+    # The zero series needs special treatment
+    if len(cs) == 1 and cs[0] == 0:
+        return cs
+
+    prd = np.empty(len(cs) + 1, dtype=cs.dtype)
+    prd[0] = cs[0]*0
+    prd[1] = cs[0]
+    if len(cs) > 1:
+        tmp = cs[1:]/2
+        prd[2:] = tmp
+        prd[0:-2] += tmp
+    return prd
+
+
+def chebmul(c1, c2):
+    """
+    Multiply one Chebyshev series by another.
+
+    Returns the product of two Chebyshev series `c1` * `c2`.  The arguments
+    are sequences of coefficients, from lowest order "term" to highest,
+    e.g., [1,2,3] represents the series ``T_0 + 2*T_1 + 3*T_2``.
+
+    Parameters
+    ----------
+    c1, c2 : array_like
+        1-d arrays of Chebyshev series coefficients ordered from low to
+        high.
+
+    Returns
+    -------
+    out : ndarray
+        Of Chebyshev series coefficients representing their product.
 
     See Also
     --------
     chebadd, chebsub, chebdiv, chebpow
 
+    Notes
+    -----
+    In general, the (polynomial) product of two C-series results in terms
+    that are not in the Chebyshev polynomial basis set.  Thus, to express
+    the product as a C-series, it is typically necessary to "re-project"
+    the product onto said basis set, which typically produces
+    "un-intuitive" (but correct) results; see Examples section below.
+
+    Examples
+    --------
+    >>> from numpy.polynomial import chebyshev as C
+    >>> c1 = (1,2,3)
+    >>> c2 = (3,2,1)
+    >>> C.chebmul(c1,c2) # multiplication requires "reprojection"
+    array([  6.5,  12. ,  12. ,   4. ,   1.5])
+
     """
     # c1, c2 are trimmed copies
     [c1, c2] = pu.as_series([c1, c2])
@@ -531,29 +721,49 @@ def chebmul(c1, c2):
 
 
 def chebdiv(c1, c2):
-    """Divide one Chebyshev series by another.
+    """
+    Divide one Chebyshev series by another.
 
-    Returns the quotient of two Chebyshev series `c1` / `c2`. The arguments
-    are sequences of coefficients ordered from low to high, i.e., [1,2,3]
-    is the series ``T_0 + 2*T_1 + 3*T_2.``
+    Returns the quotient-with-remainder of two Chebyshev series
+    `c1` / `c2`.  The arguments are sequences of coefficients from lowest
+    order "term" to highest, e.g., [1,2,3] represents the series
+    ``T_0 + 2*T_1 + 3*T_2``.
 
     Parameters
     ----------
     c1, c2 : array_like
-        1d arrays of chebyshev series coefficients ordered from low to
+        1-d arrays of Chebyshev series coefficients ordered from low to
         high.
 
     Returns
     -------
-    [quo, rem] : ndarray
-        Chebyshev series of the quotient and remainder.
+    [quo, rem] : ndarrays
+        Of Chebyshev series coefficients representing the quotient and
+        remainder.
 
     See Also
     --------
     chebadd, chebsub, chebmul, chebpow
 
+    Notes
+    -----
+    In general, the (polynomial) division of one C-series by another
+    results in quotient and remainder terms that are not in the Chebyshev
+    polynomial basis set.  Thus, to express these results as C-series, it
+    is typically necessary to "re-project" the results onto said basis
+    set, which typically produces "un-intuitive" (but correct) results;
+    see Examples section below.
+
     Examples
     --------
+    >>> from numpy.polynomial import chebyshev as C
+    >>> c1 = (1,2,3)
+    >>> c2 = (3,2,1)
+    >>> C.chebdiv(c1,c2) # quotient "intuitive," remainder not
+    (array([ 3.]), array([-8., -4.]))
+    >>> c2 = (0,1,2,3)
+    >>> C.chebdiv(c2,c1) # neither "intuitive"
+    (array([ 0.,  2.]), array([-2., -4.]))
 
     """
     # c1, c2 are trimmed copies
@@ -627,24 +837,25 @@ def chebpow(cs, pow, maxpower=16) :
         return _zseries_to_cseries(prd)
 
 def chebder(cs, m=1, scl=1) :
-    """Differentiate a Chebyshev series.
+    """
+    Differentiate a Chebyshev series.
 
-    Returns the series `cs` differentiated `m` times. At each iteration the
-    result is multiplied by `scl`. The scaling factor is for use in a
-    linear change of variable. The argument `cs` is a sequence of
-    coefficients ordered from low to high. i.e., [1,2,3] is the series
+    Returns the series `cs` differentiated `m` times.  At each iteration the
+    result is multiplied by `scl` (the scaling factor is for use in a linear
+    change of variable).  The argument `cs` is the sequence of coefficients
+    from lowest order "term" to highest, e.g., [1,2,3] represents the series
     ``T_0 + 2*T_1 + 3*T_2``.
 
     Parameters
     ----------
     cs: array_like
-        1d array of chebyshev series coefficients ordered from low to high.
+        1-d array of Chebyshev series coefficients ordered from low to high.
     m : int, optional
-        Order of differentiation, must be non-negative. (default: 1)
+        Number of derivatives taken, must be non-negative. (Default: 1)
     scl : scalar, optional
-        The result of each derivation is multiplied by `scl`. The end
-        result is multiplication by `scl`**`m`. This is for use in a linear
-        change of variable. (default: 1)
+        Each differentiation is multiplied by `scl`.  The end result is
+        multiplication by ``scl**m``.  This is for use in a linear change of
+        variable. (Default: 1)
 
     Returns
     -------
@@ -655,93 +866,151 @@ def chebder(cs, m=1, scl=1) :
     --------
     chebint
 
+    Notes
+    -----
+    In general, the result of differentiating a C-series needs to be
+    "re-projected" onto the C-series basis set. Thus, typically, the
+    result of this function is "un-intuitive," albeit correct; see Examples
+    section below.
+
     Examples
     --------
+    >>> from numpy.polynomial import chebyshev as C
+    >>> cs = (1,2,3,4)
+    >>> C.chebder(cs)
+    array([ 14.,  12.,  24.])
+    >>> C.chebder(cs,3)
+    array([ 96.])
+    >>> C.chebder(cs,scl=-1)
+    array([-14., -12., -24.])
+    >>> C.chebder(cs,2,-1)
+    array([ 12.,  96.])
 
     """
+    cnt = int(m)
+
+    if cnt != m:
+        raise ValueError, "The order of derivation must be integer"
+    if cnt < 0 :
+        raise ValueError, "The order of derivation must be non-negative"
+
     # cs is a trimmed copy
     [cs] = pu.as_series([cs])
-    if m < 0 :
-        raise ValueError, "The order of derivation must be positive"
-    if not np.isscalar(scl) :
-        raise ValueError, "The scl parameter must be a scalar"
-
-    if m == 0 :
+    if cnt == 0:
         return cs
-    elif m >= len(cs) :
+    elif cnt >= len(cs):
         return cs[:1]*0
     else :
         zs = _cseries_to_zseries(cs)
-        for i in range(m) :
+        for i in range(cnt):
             zs = _zseries_der(zs)*scl
         return _zseries_to_cseries(zs)
 
 
-def chebint(cs, m=1, k=[], lbnd=0, scl=1) :
-    """Integrate a Chebyshev series.
-
-    Returns the series integrated from `lbnd` to x `m` times. At each
-    iteration the resulting series is multiplied by `scl` and an
-    integration constant specified by `k` is added. The scaling factor is
-    for use in a linear change of variable. The argument `cs` is a sequence
-    of coefficients ordered from low to high. i.e., [1,2,3] is the series
-    ``T_0 + 2*T_1 + 3*T_2``.
+def chebint(cs, m=1, k=[], lbnd=0, scl=1):
+    """
+    Integrate a Chebyshev series.
 
+    Returns, as a C-series, the input C-series `cs`, integrated `m` times
+    from `lbnd` to `x`.  At each iteration the resulting series is
+    **multiplied** by `scl` and an integration constant, `k`, is added.
+    The scaling factor is for use in a linear change of variable.  ("Buyer
+    beware": note that, depending on what one is doing, one may want `scl`
+    to be the reciprocal of what one might expect; for more information,
+    see the Notes section below.)  The argument `cs` is a sequence of
+    coefficients, from lowest order C-series "term" to highest, e.g.,
+    [1,2,3] represents the series :math:`T_0(x) + 2T_1(x) + 3T_2(x)`.
 
     Parameters
     ----------
-    cs: array_like
-        1d array of chebyshev series coefficients ordered from low to high.
+    cs : array_like
+        1-d array of C-series coefficients, ordered from low to high.
     m : int, optional
-        Order of integration, must be positeve. (default: 1)
+        Order of integration, must be positive. (Default: 1)
     k : {[], list, scalar}, optional
-        Integration constants. The value of the first integral at zero is
-        the first value in the list, the value of the second integral at
-        zero is the second value in the list, and so on. If ``[]``
-        (default), all constants are set zero.  If `m = 1`, a single scalar
-        can be given instead of a list.
+        Integration constant(s).  The value of the first integral at zero
+        is the first value in the list, the value of the second integral
+        at zero is the second value, etc.  If ``k == []`` (the default),
+        all constants are set to zero.  If ``m == 1``, a single scalar can
+        be given instead of a list.
     lbnd : scalar, optional
-        The lower bound of the integral. (default: 0)
+        The lower bound of the integral. (Default: 0)
     scl : scalar, optional
-        Following each integration the result is multiplied by `scl` before
-        the integration constant is added. (default: 1)
+        Following each integration the result is *multiplied* by `scl`
+        before the integration constant is added. (Default: 1)
 
     Returns
     -------
-    der : ndarray
-        Chebyshev series of the integral.
+    S : ndarray
+        C-series coefficients of the integral.
 
     Raises
     ------
     ValueError
+        If ``m < 1``, ``len(k) > m``, ``np.isscalar(lbnd) == False``, or
+        ``np.isscalar(scl) == False``.
 
     See Also
     --------
     chebder
 
+    Notes
+    -----
+    Note that the result of each integration is *multiplied* by `scl`.
+    Why is this important to note?  Say one is making a linear change of
+    variable :math:`u = ax + b` in an integral relative to `x`.  Then
+    :math:`dx = du/a`, so one will need to set `scl` equal to :math:`1/a`
+    - perhaps not what one would have first thought.
+
+    Also note that, in general, the result of integrating a C-series needs
+    to be "re-projected" onto the C-series basis set.  Thus, typically,
+    the result of this function is "un-intuitive," albeit correct; see
+    Examples section below.
+
     Examples
     --------
+    >>> from numpy.polynomial import chebyshev as C
+    >>> cs = (1,2,3)
+    >>> C.chebint(cs)
+    array([ 0.5, -0.5,  0.5,  0.5])
+    >>> C.chebint(cs,3)
+    array([ 0.03125   , -0.1875    ,  0.04166667, -0.05208333,  0.01041667,
+            0.00625   ])
+    >>> C.chebint(cs, k=3)
+    array([ 3.5, -0.5,  0.5,  0.5])
+    >>> C.chebint(cs,lbnd=-2)
+    array([ 8.5, -0.5,  0.5,  0.5])
+    >>> C.chebint(cs,scl=-2)
+    array([-1.,  1., -1., -1.])
 
     """
-    if np.isscalar(k) :
+    cnt = int(m)
+    if not np.iterable(k):
         k = [k]
-    if m < 1 :
-        raise ValueError, "The order of integration must be positive"
-    if len(k) > m :
+
+    if cnt != m:
+        raise ValueError, "The order of integration must be integer"
+    if cnt < 0 :
+        raise ValueError, "The order of integration must be non-negative"
+    if len(k) > cnt :
         raise ValueError, "Too many integration constants"
-    if not np.isscalar(lbnd) :
-        raise ValueError, "The lbnd parameter must be a scalar"
-    if not np.isscalar(scl) :
-        raise ValueError, "The scl parameter must be a scalar"
 
     # cs is a trimmed copy
     [cs] = pu.as_series([cs])
-    k = list(k) + [0]*(m - len(k))
-    for i in range(m) :
-        zs = _cseries_to_zseries(cs)*scl
-        zs = _zseries_int(zs)
-        cs = _zseries_to_cseries(zs)
-        cs[0] += k[i] - chebval(lbnd, cs)
+    if cnt == 0:
+        return cs
+
+    k = list(k) + [0]*(cnt - len(k))
+    for i in range(cnt) :
+        n = len(cs)
+        cs *= scl
+        if n == 1 and cs[0] == 0:
+            cs[0] += k[i]
+        else:
+            zs = _cseries_to_zseries(cs)
+            zs = _zseries_int(zs)
+            cs = _zseries_to_cseries(zs)
+            cs[0] += k[i] - chebval(lbnd, cs)
     return cs
 
 def chebval(x, cs):
@@ -817,7 +1086,8 @@ def chebvander(x, deg) :
     Parameters
     ----------
     x : array_like
-        Array of points. The values are converted to double or complex doubles.
+        Array of points. The values are converted to double or complex
+        doubles. If x is scalar it is converted to a 1D array.
     deg : integer
         Degree of the resulting matrix.
 
@@ -828,21 +1098,30 @@ def chebvander(x, deg) :
         index is the degree.
 
     """
-    x = np.asarray(x) + 0.0
-    order = int(deg) + 1
-    v = np.ones(x.shape + (order,), dtype=x.dtype)
-    if order > 1 :
+    ideg = int(deg)
+    if ideg != deg:
+        raise ValueError("deg must be integer")
+    if ideg < 0:
+        raise ValueError("deg must be non-negative")
+
+    x = np.array(x, copy=0, ndmin=1) + 0.0
+    v = np.empty((ideg + 1,) + x.shape, dtype=x.dtype)
+    # Use forward recursion to generate the entries.
+    v[0] = x*0 + 1
+    if ideg > 0 :
         x2 = 2*x
-        v[...,1] = x
-        for i in range(2, order) :
-            v[...,i] = x2*v[...,i-1] - v[...,i-2]
-    return v
+        v[1] = x
+        for i in range(2, ideg + 1) :
+            v[i] = v[i-1]*x2 - v[i-2]
+    return np.rollaxis(v, 0, v.ndim)
 
-def chebfit(x, y, deg, rcond=None, full=False):
-    """Least squares fit of Chebyshev series to data.
 
-    Fit a Chebyshev series ``p(x) = p[0] * T_{deq}(x) + ... + p[deg] *
-    T_{0}(x)`` of degree `deg` to points `(x, y)`. Returns a vector of
+def chebfit(x, y, deg, rcond=None, full=False, w=None):
+    """
+    Least squares fit of Chebyshev series to data.
+
+    Fit a Chebyshev series ``p(x) = p[0] * T_{0}(x) + ... + p[deg] *
+    T_{deg}(x)`` of degree `deg` to points `(x, y)`. Returns a vector of
     coefficients `p` that minimises the squared error.
 
     Parameters
@@ -864,6 +1143,12 @@ def chebfit(x, y, deg, rcond=None, full=False):
         Switch determining nature of return value. When it is False (the
         default) just the coefficients are returned, when True diagnostic
         information from the singular value decomposition is also returned.
+    w : array_like, shape (`M`,), optional
+        Weights. If not None, the contribution of each point
+        ``(x[i],y[i])`` to the fit is weighted by `w[i]`. Ideally the
+        weights are chosen so that the errors of the products ``w[i]*y[i]``
+        all have the same variance.  The default value is None.
+        .. versionadded:: 1.5.0
 
     Returns
     -------
@@ -900,7 +1185,7 @@ def chebfit(x, y, deg, rcond=None, full=False):
     The solution are the coefficients ``c[i]`` of the Chebyshev series
     ``T(x)`` that minimizes the squared error
 
-    ``E = \sum_j |y_j - T(x_j)|^2``.
+    ``E = \\sum_j |y_j - T(x_j)|^2``.
 
     This problem is solved by setting up as the overdetermined matrix
     equation
@@ -946,17 +1231,33 @@ def chebfit(x, y, deg, rcond=None, full=False):
         raise TypeError, "expected non-empty vector for x"
     if y.ndim < 1 or y.ndim > 2 :
         raise TypeError, "expected 1D or 2D array for y"
-    if x.shape[0] != y.shape[0] :
+    if len(x) != len(y):
         raise TypeError, "expected x and y to have same length"
 
+    # set up the least squares matrices
+    lhs = chebvander(x, deg)
+    rhs = y
+    if w is not None:
+        w = np.asarray(w) + 0.0
+        if w.ndim != 1:
+            raise TypeError, "expected 1D vector for w"
+        if len(x) != len(w):
+            raise TypeError, "expected x and w to have same length"
+        # apply weights
+        if rhs.ndim == 2:
+            lhs *= w[:, np.newaxis]
+            rhs *= w[:, np.newaxis]
+        else:
+            lhs *= w[:, np.newaxis]
+            rhs *= w
+
     # set rcond
     if rcond is None :
         rcond = len(x)*np.finfo(x.dtype).eps
 
-    # set up the design matrix and solve the least squares equation
-    A = chebvander(x, deg)
-    scl = np.sqrt((A*A).sum(0))
-    c, resids, rank, s = la.lstsq(A/scl, y, rcond)
+    # scale the design matrix and solve the least squares equation
+    scl = np.sqrt((lhs*lhs).sum(0))
+    c, resids, rank, s = la.lstsq(lhs/scl, rhs, rcond)
     c = (c.T/scl).T
 
     # warn on rank reduction
@@ -971,24 +1272,45 @@ def chebfit(x, y, deg, rcond=None, full=False):
 
 
 def chebroots(cs):
-    """Roots of a Chebyshev series.
+    """
+    Compute the roots of a Chebyshev series.
 
-    Compute the roots of the Chebyshev series `cs`. The argument `cs` is a
-    sequence of coefficients ordered from low to high. i.e., [1,2,3] is the
-    series ``T_0 + 2*T_1 + 3*T_2``.
+    Return the roots (a.k.a "zeros") of the C-series represented by `cs`,
+    which is the sequence of the C-series' coefficients from lowest order
+    "term" to highest, e.g., [1,2,3] represents the C-series
+    ``T_0 + 2*T_1 + 3*T_2``.
 
     Parameters
     ----------
     cs : array_like
-        1D array of Chebyshev coefficients ordered from low to high.
+        1-d array of C-series coefficients ordered from low to high.
 
     Returns
     -------
     out : ndarray
-        An array containing the complex roots of the chebyshev series.
+        Array of the roots.  If all the roots are real, then so is the
+        dtype of ``out``; otherwise, ``out``'s dtype is complex.
+
+    See Also
+    --------
+    polyroots
+
+    Notes
+    -----
+    Algorithm(s) used:
+
+    Remember: because the C-series basis set is different from the
+    "standard" basis set, the results of this function *may* not be what
+    one is expecting.
 
     Examples
     --------
+    >>> import numpy.polynomial as P
+    >>> import numpy.polynomial.chebyshev as C
+    >>> P.polyroots((-1,1,-1,1)) # x^3 - x^2 + x - 1 has two complex roots
+    array([ -4.99600361e-16-1.j,  -4.99600361e-16+1.j,   1.00000e+00+0.j])
+    >>> C.chebroots((-1,1,-1,1)) # T3 - T2 + T1 - T0 has only real roots
+    array([ -5.00000000e-01,   2.60860684e-17,   1.00000000e+00])
 
     """
     # cs is a trimmed copy
@@ -997,20 +1319,86 @@ def chebroots(cs):
         return np.array([], dtype=cs.dtype)
     if len(cs) == 2 :
         return np.array([-cs[0]/cs[1]])
+
     n = len(cs) - 1
+    cs /= cs[-1]
     cmat = np.zeros((n,n), dtype=cs.dtype)
-    cmat.flat[1::n+1] = .5
-    cmat.flat[n::n+1] = .5
     cmat[1, 0] = 1
-    cmat[:,-1] -= cs[:-1]*(.5/cs[-1])
+    for i in range(1, n):
+        cmat[i - 1, i] = .5
+        if i != n - 1:
+            cmat[i + 1, i] = .5
+        else:
+            cmat[:, i] -= cs[:-1]*.5
     roots = la.eigvals(cmat)
     roots.sort()
     return roots
 
 
+def chebpts1(npts):
+    """Chebyshev points of the first kind.
+
+    Chebyshev points of the first kind are the set ``{cos(x_k)}``,
+    where ``x_k = pi*(k + .5)/npts`` for k in ``range(npts}``.
+
+    Parameters
+    ----------
+    npts: int
+        Number of sample points desired.
+
+    Returns
+    -------
+    pts: ndarray
+        The Chebyshev points of the second kind.
+
+    Notes
+    -----
+    .. versionadded:: 1.5.0
+
+    """
+    _npts = int(npts)
+    if _npts != npts:
+        raise ValueError("npts must be integer")
+    if _npts < 1:
+        raise ValueError("npts must be >= 1")
+
+    x = np.linspace(-np.pi, 0, _npts, endpoint=False) + np.pi/(2*_npts)
+    return np.cos(x)
+
+
+def chebpts2(npts):
+    """Chebyshev points of the second kind.
+
+    Chebyshev points of the second kind are the set ``{cos(x_k)}``,
+    where ``x_k = pi*/(npts - 1)`` for k in ``range(npts}``.
+
+    Parameters
+    ----------
+    npts: int
+        Number of sample points desired.
+
+    Returns
+    -------
+    pts: ndarray
+        The Chebyshev points of the second kind.
+
+    Notes
+    -----
+    .. versionadded:: 1.5.0
+
+    """
+    _npts = int(npts)
+    if _npts != npts:
+        raise ValueError("npts must be integer")
+    if _npts < 2:
+        raise ValueError("npts must be >= 2")
+
+    x = np.linspace(-np.pi, 0, _npts)
+    return np.cos(x)
+
+
 #
 # Chebyshev series class
 #
 
 exec polytemplate.substitute(name='Chebyshev', nick='cheb', domain='[-1,1]')
-
