@@ -1,11 +1,25 @@
-"""Template for the Chebyshev and Polynomial classes.
+"""
+Template for the Chebyshev and Polynomial classes.
+
+This module houses a Python string module Template object (see, e.g.,
+http://docs.python.org/library/string.html#template-strings) used by
+the `polynomial` and `chebyshev` modules to implement their respective
+`Polynomial` and `Chebyshev` classes.  It provides a mechanism for easily
+creating additional specific polynomial classes (e.g., Legendre, Jacobi,
+etc.) in the future, such that all these classes will have a common API.
 
 """
 import string
+import sys
+
+if sys.version_info[0] >= 3:
+    rel_import = "from . import"
+else:
+    rel_import = "import"
 
 polytemplate = string.Template('''
 from __future__ import division
-import polyutils as pu
+REL_IMPORT polyutils as pu
 import numpy as np
 
 class $name(pu.PolyBase) :
@@ -289,11 +303,48 @@ class $name(pu.PolyBase) :
     # Extra numeric functions.
     #
 
+    def degree(self) :
+        """The degree of the series.
+
+        Notes
+        -----
+        .. versionadded:: 1.5.0
+
+        """
+        return len(self) - 1
+
+    def cutdeg(self, deg) :
+        """Truncate series to the given degree.
+
+        Reduce the degree of the $name series to `deg` by discarding the
+        high order terms. If `deg` is greater than the current degree a
+        copy of the current series is returned. This can be useful in least
+        squares where the coefficients of the high degree terms may be very
+        small.
+
+        Parameters
+        ----------
+        deg : non-negative int
+            The series is reduced to degree `deg` by discarding the high
+            order terms. The value of `deg` must be a non-negative integer.
+
+        Returns
+        -------
+        new_instance : $name
+            New instance of $name with reduced degree.
+
+        Notes
+        -----
+        .. versionadded:: 1.5.0
+
+        """
+        return self.truncate(deg + 1)
+
     def convert(self, domain=None, kind=None) :
         """Convert to different class and/or domain.
 
-        Parameters:
-        -----------
+        Parameters
+        ----------
         domain : {None, array_like}
             The domain of the new series type instance. If the value is is
             ``None``, then the default domain of `kind` is used.
@@ -302,17 +353,17 @@ class $name(pu.PolyBase) :
             should be converted. If kind is ``None``, then the class of the
             current instance is used.
 
-        Returns:
-        --------
+        Returns
+        -------
         new_series_instance : `kind`
             The returned class can be of different type than the current
             instance and/or have a different domain.
 
-        Examples:
-        ---------
+        Examples
+        --------
 
-        Notes:
-        ------
+        Notes
+        -----
         Conversion between domains and class types can result in
         numerically ill defined series.
 
@@ -334,8 +385,8 @@ class $name(pu.PolyBase) :
         separately, then the linear function must be substituted for the
         ``x`` in the standard representation of the base polynomials.
 
-        Returns:
-        --------
+        Returns
+        -------
         off, scl : floats or complex
             The mapping function is defined by ``off + scl*x``.
 
@@ -360,12 +411,12 @@ class $name(pu.PolyBase) :
         ``[0]``. A new $name instance is returned with the new coefficients.
         The current instance remains unchanged.
 
-        Parameters:
-        -----------
+        Parameters
+        ----------
         tol : non-negative number.
             All trailing coefficients less than `tol` will be removed.
 
-        Returns:
+        Returns
         -------
         new_instance : $name
             Contains the new set of coefficients.
@@ -374,31 +425,32 @@ class $name(pu.PolyBase) :
         return self.__class__(pu.trimcoef(self.coef, tol), self.domain)
 
     def truncate(self, size) :
-        """Truncate series by discarding trailing coefficients.
+        """Truncate series to length `size`.
 
-        Reduce the $name series to length `size` by removing trailing
-        coefficients. The value of `size` must be greater than zero.  This
-        is most likely to be useful in least squares fits when the high
-        order coefficients are very small.
+        Reduce the $name series to length `size` by discarding the high
+        degree terms. The value of `size` must be a positive integer. This
+        can be useful in least squares where the coefficients of the
+        high degree terms may be very small.
 
-        Parameters:
-        -----------
-        size : int
-            The series is reduced to length `size` by discarding trailing
-            coefficients. The value of `size` must be greater than zero.
+        Parameters
+        ----------
+        size : positive int
+            The series is reduced to length `size` by discarding the high
+            degree terms. The value of `size` must be a positive integer.
 
-        Returns:
+        Returns
         -------
         new_instance : $name
             New instance of $name with truncated coefficients.
 
         """
-        if size < 1 :
-            raise ValueError("size must be > 0")
-        if size >= len(self.coef) :
+        isize = int(size)
+        if isize != size or isize < 1 :
+            raise ValueError("size must be a positive integer")
+        if isize >= len(self.coef) :
             return self.__class__(self.coef, self.domain)
         else :
-            return self.__class__(self.coef[:size], self.domain)
+            return self.__class__(self.coef[:isize], self.domain)
 
     def copy(self) :
         """Return a copy.
@@ -406,8 +458,8 @@ class $name(pu.PolyBase) :
         A new instance of $name is returned that has the same
         coefficients and domain as the current instance.
 
-        Returns:
-        --------
+        Returns
+        -------
         new_instance : $name
             New instance of $name with the same coefficients and domain.
 
@@ -420,9 +472,9 @@ class $name(pu.PolyBase) :
         Return an instance of $name that is the definite integral of the
         current series. Refer to `${nick}int` for full documentation.
 
-        Parameters:
-        -----------
-        m : positive integer
+        Parameters
+        ----------
+        m : non-negative int
             The number of integrations to perform.
         k : array_like
             Integration constants. The first constant is applied to the
@@ -432,11 +484,10 @@ class $name(pu.PolyBase) :
         lbnd : Scalar
             The lower bound of the definite integral.
 
-        Returns:
-        --------
+        Returns
+        -------
         integral : $name
-            The integral of the original series defined with the same
-            domain.
+            The integral of the series using the same domain.
 
         See Also
         --------
@@ -458,16 +509,15 @@ class $name(pu.PolyBase) :
         Return an instance of $name that is the derivative of the current
         series.  Refer to `${nick}der` for full documentation.
 
-        Parameters:
-        -----------
-        m : positive integer
+        Parameters
+        ----------
+        m : non-negative int
             The number of integrations to perform.
 
-        Returns:
-        --------
+        Returns
+        -------
         derivative : $name
-            The derivative of the original series defined with the same
-            domain.
+            The derivative of the series using the same domain.
 
         See Also
         --------
@@ -495,8 +545,35 @@ class $name(pu.PolyBase) :
         roots = ${nick}roots(self.coef)
         return pu.mapdomain(roots, $domain, self.domain)
 
+    def linspace(self, n=100):
+        """Return x,y values at equally spaced points in domain.
+
+        Returns x, y values at `n` equally spaced points across domain.
+        Here y is the value of the polynomial at the points x.  This is
+        intended as a plotting aid.
+
+        Paramters
+        ---------
+        n : int, optional
+            Number of point pairs to return. The default value is 100.
+
+        Returns
+        -------
+        x, y : ndarrays
+            ``x`` is equal to linspace(self.domain[0], self.domain[1], n)
+            ``y`` is the polynomial evaluated at ``x``.
+
+        .. versionadded:: 1.5.0
+
+        """
+        x = np.linspace(self.domain[0], self.domain[1], n)
+        y = self(x)
+        return x, y
+
+
+
     @staticmethod
-    def fit(x, y, deg, domain=$domain, rcond=None, full=False) :
+    def fit(x, y, deg, domain=None, rcond=None, full=False, w=None) :
         """Least squares fit to data.
 
         Return a `$name` instance that is the least squares fit to the data
@@ -515,10 +592,12 @@ class $name(pu.PolyBase) :
             passing in a 2D-array that contains one dataset per column.
         deg : int
             Degree of the fitting polynomial
-        domain : {None, [beg, end]}, optional
+        domain : {None, [beg, end], []}, optional
             Domain to use for the returned $name instance. If ``None``,
-            then a minimal domain that covers the points `x` is chosen. The
-            default value is ``$domain``.
+            then a minimal domain that covers the points `x` is chosen.  If
+            ``[]`` the default domain ``$domain`` is used. The default
+            value is $domain in numpy 1.4.x and ``None`` in later versions.
+            The ``'[]`` value was added in numpy 1.5.0.
         rcond : float, optional
             Relative condition number of the fit. Singular values smaller
             than this relative to the largest singular value will be
@@ -530,6 +609,13 @@ class $name(pu.PolyBase) :
             (the default) just the coefficients are returned, when True
             diagnostic information from the singular value decomposition is
             also returned.
+        w : array_like, shape (M,), optional
+            Weights. If not None the contribution of each point
+            ``(x[i],y[i])`` to the fit is weighted by `w[i]`. Ideally the
+            weights are chosen so that the errors of the products
+            ``w[i]*y[i]`` all have the same variance.  The default value is
+            None.
+            .. versionadded:: 1.5.0
 
         Returns
         -------
@@ -550,8 +636,10 @@ class $name(pu.PolyBase) :
         """
         if domain is None :
             domain = pu.getdomain(x)
+        elif domain == [] :
+            domain = $domain
         xnew = pu.mapdomain(x, domain, $domain)
-        res = ${nick}fit(xnew, y, deg, rcond=None, full=full)
+        res = ${nick}fit(xnew, y, deg, w=w, rcond=rcond, full=full)
         if full :
             [coef, status] = res
             return $name(coef, domain=domain), status
@@ -597,4 +685,4 @@ class $name(pu.PolyBase) :
         off, scl = pu.mapparms($domain, domain)
         coef = ${nick}line(off, scl)
         return $name(coef, domain)
-''')
+'''.replace('REL_IMPORT', rel_import))
